@@ -30,6 +30,8 @@
 #define	INCLUDED_GS_CATEGORIES_H
 #include "GNUstepBase/GSVersionMacros.h"
 
+#include <Foundation/Foundation.h>
+
 /* The following ifndef prevents the categories declared in this file being
  * seen in GNUstep code.  This is necessary because those category
  * declarations are also present in the header files for the corresponding
@@ -40,7 +42,6 @@
 #ifndef GNUSTEP
 
 #include <string.h>
-#include <Foundation/Foundation.h>
 
 #ifdef NeXT_Foundation_LIBRARY
 #include <CoreFoundation/CFString.h>
@@ -63,10 +64,6 @@ extern "C" {
  * Macros
  */
 
-// Following are also defined in base/Headers/Foundation/NSObject.h
-#define IF_NO_GC(x)	\
-    x
-
 // Following are also defined in base/Headers/Foundation/NSDebug.h
 #ifdef DEBUG
 #define NSDebugLLog(level, format, args...) \
@@ -181,7 +178,17 @@ typedef enum _NSGNUstepStringEncoding
   NSUTF7StringEncoding = 64,		// RFC 2152
   NSGSM0338StringEncoding,		// GSM (mobile phone) default alphabet
   NSBIG5StringEncoding,			// Traditional chinese
-  NSKoreanEUCStringEncoding
+  NSKoreanEUCStringEncoding,
+#if MAC_OS_X_VERSION_10_4 >= MAC_OS_X_VERSION_MAX_ALLOWED
+    NSUTF16BigEndianStringEncoding = 0x90000100,          /* NSUTF16StringEncoding encoding with explicit endianness specified */
+    NSUTF16LittleEndianStringEncoding = 0x94000100,       /* NSUTF16StringEncoding encoding with explicit endianness specified */
+
+    NSUTF32StringEncoding = 0x8c000100,
+    NSUTF32BigEndianStringEncoding = 0x98000100,          /* NSUTF32StringEncoding encoding with explicit endianness specified */
+    NSUTF32LittleEndianStringEncoding = 0x9c000100,        /* NSUTF32StringEncoding encoding with explicit endianness specified */
+#endif
+
+  GSEncodingUnusedLast
 } NSGNUstepStringEncoding;
 
 
@@ -200,7 +207,7 @@ typedef enum _NSGNUstepStringEncoding
 */
 
 @interface NSCalendarDate (GSCategories)
-- (int) weekOfYear;
+- (NSInteger) weekOfYear;
 @end
 
 @interface NSData (GSCategories)
@@ -209,10 +216,10 @@ typedef enum _NSGNUstepStringEncoding
 - (NSData*) md5Digest;
 - (BOOL) uudecodeInto: (NSMutableData*)decoded
 		 name: (NSString**)namePtr
-		 mode: (int*)modePtr;
+		 mode: (NSInteger*)modePtr;
 - (BOOL) uuencodeInto: (NSMutableData*)encoded
 		 name: (NSString*)name
-		 mode: (int)mode;
+		 mode: (NSInteger)mode;
 @end
 
 @interface NSString (GSCategories)
@@ -268,11 +275,11 @@ typedef enum _NSGNUstepStringEncoding
 
 @interface NSArray (GSCompatibility)
 - (id) initWithArray: (NSArray*)array copyItems: (BOOL)shouldCopy;
-- (unsigned) insertionPosition: (id)item
-		 usingFunction: (NSComparisonResult (*)(id, id, void *))sorter
-		       context: (void *)context;
-- (unsigned) insertionPosition: (id)item
-		 usingSelector: (SEL)comp;
+- (NSUInteger) insertionPosition: (id)item
+		   usingFunction: (NSComparisonResult (*)(id, id, void *))sorter
+		         context: (void *)context;
+- (NSUInteger) insertionPosition: (id)item
+		   usingSelector: (SEL)comp;
 @end
 
 @interface      NSAttributedString (GSCategories)
@@ -286,7 +293,7 @@ typedef enum _NSGNUstepStringEncoding
 @end
 
 @interface NSDistantObject (GSCompatibility)
-+ (void) setDebug: (int)val;
++ (void) setDebug: (NSInteger)val;
 @end
 
 @interface NSFileHandle(GSCompatibility)
@@ -320,19 +327,10 @@ typedef enum _NSGNUstepStringEncoding
 + (id) newLockAt: (id *)location;
 @end
 
-@interface NSLock (GSCompatibility)
-- (void) gcFinalize;
-@end
-
 @interface NSRecursiveLock (GSCategories)
 + (id)newLockAt:(id *)location;
 @end
 
-@interface NSRecursiveLock (GSCompatibility)
-- (void) gcFinalize;
-@end
-
-
 /* ------------------------------------------------------------------------
  * Functions
  */
@@ -362,6 +360,8 @@ GS_EXPORT NSString *GSDebugFunctionMsg(const char *func, const char *file,
 
 #endif	/* OS_API_VERSION(GS_API_NONE,GS_API_NONE) */
 
+#include	<GNUstepBase/NSTask+GS.h>
+
 #if	defined(__cplusplus)
 }
 #endif
