@@ -28,6 +28,7 @@
 #if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
 #include <Foundation/NSObject.h>
+#include <Foundation/NSGarbageCollector.h>
 #include <Foundation/NSZone.h>
 
 #if	defined(__cplusplus)
@@ -80,6 +81,13 @@ extern "C" {
  *		values do not need to be released when the map is emptied.
  *		This permits some optimisation.
  *
+ *      GSI_MAP_NODES()
+ *              Define this macro to allocate nodes for the map using typed
+ *              memory when working with garbage collection.
+ *
+ *      GSI_MAP_ZEROED()
+ *              Define this macro to check whether a map uses keys which may
+ *              be zeroed weak pointers.  This is only used when GC is enabled.
  */
 
 #ifndef	GSI_MAP_HAS_VALUE
@@ -104,6 +112,13 @@ extern "C" {
 #ifndef	GSI_MAP_EQUAL
 #define	GSI_MAP_EQUAL(M, X, Y)		[(X).obj isEqual: (Y).obj]
 #endif
+#ifndef GSI_MAP_NODES
+#define GSI_MAP_NODES(M, X) \
+(GSIMapNode)NSAllocateCollectable(X*sizeof(GSIMapNode_t), NSScannedOption)
+#endif
+#ifndef GSI_MAP_ZEROED
+#define GSI_MAP_ZEROED(M)		0
+#endif
 
 /*
  *      If there is no bitmask defined to supply the types that
@@ -270,13 +285,13 @@ extern "C" {
  *  of size "increment".
  */
 
-typedef struct _GSIMapTable GSIMapTable_t;
+#if	!defined(GSI_MAP_TABLE_T)
 typedef struct _GSIMapBucket GSIMapBucket_t;
 typedef struct _GSIMapNode GSIMapNode_t;
 
-typedef GSIMapTable_t *GSIMapTable;
 typedef GSIMapBucket_t *GSIMapBucket;
 typedef GSIMapNode_t *GSIMapNode;
+#endif
 
 struct	_GSIMapNode {
   GSIMapNode	nextInBucket;	/* Linked list of bucket.	*/
@@ -291,6 +306,12 @@ struct	_GSIMapBucket {
   GSIMapNode	firstNode;	/* The linked list of nodes.	*/
 };
 
+#if	defined(GSI_MAP_TABLE_T)
+typedef GSI_MAP_TABLE_T	*GSIMapTable;
+#else
+typedef struct _GSIMapTable GSIMapTable_t;
+typedef GSIMapTable_t *GSIMapTable;
+
 struct	_GSIMapTable {
   NSZone	*zone;
   size_t	nodeCount;	/* Number of used nodes in map.	*/
@@ -304,6 +325,7 @@ struct	_GSIMapTable {
   GSI_MAP_EXTRA	extra;
 #endif
 };
+#endif
 
 typedef struct	_GSIMapEnumerator {
   GSIMapTable	map;		/* the map being enumerated.	*/
@@ -389,11 +411,63 @@ GSIMapRemoveNodeFromMap(GSIMapTable map, GSIMapBucket bkt, GSIMapNode node)
   GSIMapRemoveNodeFromBucket(bkt, node);
 }
 
+static INLINE void
+GSIMapFreeNode(GSIMapTable map, GSIMapNode node)
+{
+  GSI_MAP_RELEASE_KEY(map, node->key);
+  GSI_MAP_CLEAR_KEY(node);
+#if	GSI_MAP_HAS_VALUE
+  GSI_MAP_RELEASE_VAL(map, node->value);
+  GSI_MAP_CLEAR_VAL(node);
+#endif
+  
+  node->nextInBucket = map->freeNodes;
+  map->freeNodes = node;
+}
+
+static INLINE GSIMapNode
+GSIMapRemoveAndFreeNode(GSIMapTable map, size_t bkt, GSIMapNode node)
+{
+  GSIMapNode	next = node->nextInBucket;
+  GSIMapRemoveNodeFromMap(map, &(map->buckets[bkt]), node);
+  GSIMapFreeNode(map, node);
+  return next;
+}
+
 static INLINE void
 GSIMapRemangleBuckets(GSIMapTable map,
   GSIMapBucket old_buckets, size_t old_bucketCount,
   GSIMapBucket new_buckets, size_t new_bucketCount)
 {
+#if	GS_WITH_GC
+  if (GSI_MAP_ZEROED(map))
+    {
+      while (old_bucketCount-- > 0)
+	{
+	  GSIMapNode	node;
+
+	  while ((node = old_buckets->firstNode) != 0)
+	    {
+	      if (node->key.addr == 0)
+		{
+		  GSIMapRemoveNodeFromMap(map, old_buckets, node);
+		  GSIMapFreeNode(map, node);
+		}
+	      else
+		{
+		  GSIMapBucket	bkt;
+
+		  GSIMapRemoveNodeFromBucket(old_buckets, node);
+		  bkt = GSIMapPickBucket(GSI_MAP_HASH(map, node->key),
+		    new_buckets, new_bucketCount);
+		  GSIMapAddNodeToBucket(bkt, node);
+		}
+	    }
+	  old_buckets++;
+	}
+      return;
+    }
+#endif
   while (old_bucketCount-- > 0)
     {
       GSIMapNode	node;
@@ -417,29 +491,26 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
   GSIMapNode	*newArray;
   size_t	arraySize = (map->chunkCount+1)*sizeof(GSIMapNode);
 
-#if	GS_WITH_GC == 1
-  /*
-   * Our nodes may be allocated from the atomic zone - but we don't want
-   * them freed - so we must keep the array of pointers to memory chunks in
-   * the default zone
+#if     GS_WITH_GC
+  /* We don't want our nodes collected before we have finished with them,
+   * so we must keep the array of pointers to memory chunks in scanned memory.
    */
-  if (map->zone == GSAtomicMallocZone())
-    {
-      newArray = (GSIMapNode*)NSZoneMalloc(NSDefaultMallocZone(), arraySize);
-    }
-  else
-#endif
+  newArray = (GSIMapNode*)NSAllocateCollectable(arraySize, NSScannedOption);
+#else
   newArray = (GSIMapNode*)NSZoneMalloc(map->zone, arraySize);
+#endif
   if (newArray)
     {
       GSIMapNode	newNodes;
       size_t		chunkCount;
-      size_t		chunkSize;
 
       if (map->nodeChunks != 0)
 	{
-	  memcpy(newArray, map->nodeChunks, (map->chunkCount)*sizeof(GSIMapNode));
+	  memcpy(newArray, map->nodeChunks,
+	    (map->chunkCount)*sizeof(GSIMapNode));
+#if     !GS_WITH_GC
 	  NSZoneFree(map->zone, map->nodeChunks);
+#endif
 	}
       map->nodeChunks = newArray;
 
@@ -458,8 +529,12 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
 	{
 	  chunkCount = required;
 	}
-      chunkSize = chunkCount * sizeof(GSIMapNode_t);
-      newNodes = (GSIMapNode)NSZoneMalloc(map->zone, chunkSize);
+#if     GS_WITH_GC
+      newNodes = GSI_MAP_NODES(map, chunkCount);
+#else
+      newNodes
+        = (GSIMapNode)NSZoneMalloc(map->zone, chunkCount*sizeof(GSIMapNode_t));
+#endif
       if (newNodes)
 	{
 	  map->nodeChunks[map->chunkCount++] = newNodes;
@@ -473,71 +548,28 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
     }
 }
 
-#if	GSI_MAP_HAS_VALUE
-static INLINE GSIMapNode
-GSIMapNewNode(GSIMapTable map, GSIMapKey key, GSIMapVal value)
-{
-  GSIMapNode	node = map->freeNodes;
-
-  if (node == 0)
-    {
-      GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
-      node = map->freeNodes;
-      if (node == 0)
-	{
-	  return 0;
-	}
-    }
-
-  map->freeNodes = node->nextInBucket;
-  node->key = key;
-  node->value = value;
-  node->nextInBucket = 0;
-
-  return node;
-}
-#else
-static INLINE GSIMapNode
-GSIMapNewNode(GSIMapTable map, GSIMapKey key)
-{
-  GSIMapNode	node = map->freeNodes;
-
-  if (node == 0)
-    {
-      GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
-      node = map->freeNodes;
-      if (node == 0)
-	{
-	  return 0;
-	}
-    }
-
-  map->freeNodes = node->nextInBucket;
-  node->key = key;
-  node->nextInBucket = 0;
-  return node;
-}
-#endif
-
-static INLINE void
-GSIMapFreeNode(GSIMapTable map, GSIMapNode node)
-{
-  GSI_MAP_RELEASE_KEY(map, node->key);
-  GSI_MAP_CLEAR_KEY(node);
-#if	GSI_MAP_HAS_VALUE
-  GSI_MAP_RELEASE_VAL(map, node->value);
-  GSI_MAP_CLEAR_VAL(node);
-#endif
-  
-  node->nextInBucket = map->freeNodes;
-  map->freeNodes = node;
-}
-
 static INLINE GSIMapNode 
 GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
 {
   GSIMapNode	node = bucket->firstNode;
 
+#if	GS_WITH_GC
+  if (GSI_MAP_ZEROED(map))
+    {
+      while ((node != 0) && GSI_MAP_EQUAL(map, node->key, key) == NO)
+	{
+	  GSIMapNode	tmp = node->nextInBucket;
+
+	  if (node->key.addr == 0)
+	    {
+	      GSIMapRemoveNodeFromMap(map, bucket, node);
+	      GSIMapFreeNode(map, node);
+	    }
+	  node = tmp;
+	}
+      return node;
+    }
+#endif
   while ((node != 0) && GSI_MAP_EQUAL(map, node->key, key) == NO)
     {
       node = node->nextInBucket;
@@ -560,6 +592,51 @@ GSIMapNodeForKey(GSIMapTable map, GSIMapKey key)
   return node;
 }
 
+static INLINE GSIMapNode 
+GSIMapFirstNode(GSIMapTable map)
+{
+  if (map->nodeCount > 0)
+    {
+      size_t		count = map->bucketCount;
+      size_t		bucket = 0;
+      GSIMapNode	node = 0;
+
+#if	GS_WITH_GC
+      if (GSI_MAP_ZEROED(map))
+	{
+	  while (bucket < count)
+	    {
+	      node = map->buckets[bucket].firstNode;
+	      while (node != 0 && node->key.addr == 0)
+		{
+		  node = GSIMapRemoveAndFreeNode(map, bucket, node);
+		}
+	      if (node != 0)
+		{
+		  break;
+		}
+	      bucket++;
+	    }
+	  return node;
+	}
+#endif
+      while (bucket < count)
+	{
+	  node = map->buckets[bucket].firstNode;
+	  if (node != 0)
+	    {
+	      break;
+	    }
+	  bucket++;
+	}
+      return node;
+    }
+  else
+    {
+      return 0;
+    }
+}
+
 #if     (GSI_MAP_KTYPES & GSUNION_INT)
 /*
  * Specialized lookup for the case where keys are known to be simple integer
@@ -578,6 +655,23 @@ GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
     }
   bucket = map->buckets + ((unsigned)key.addr) % map->bucketCount;
   node = bucket->firstNode;
+#if	GS_WITH_GC
+  if (GSI_MAP_ZEROED(map))
+    {
+      while ((node != 0) && node->key.addr != key.addr)
+	{
+	  GSIMapNode	tmp = node->nextInBucket;
+
+	  if (node->key.addr == 0)
+	    {
+	      GSIMapRemoveNodeFromMap(map, bucket, node);
+	      GSIMapFreeNode(map, node);
+	    }
+	  node = tmp;
+	}
+      return node;
+    }
+#endif
   while ((node != 0) && node->key.addr != key.addr)
     {
       node = node->nextInBucket;
@@ -613,20 +707,30 @@ GSIMapResize(GSIMapTable map, size_t new_capacity)
       size++;
     }
 
-  /*
-   *	Make a new set of buckets for this map
+#if     GS_WITH_GC
+  /* We don't need to use scanned memory because the nodes are not 'owned'
+   * by the bucket they are in, but rather are in chunks pointed to by
+   * the nodeChunks array.
+   */
+  new_buckets = (GSIMapBucket)NSAllocateCollectable
+    (size * sizeof(GSIMapBucket_t), 0);
+#else
+  /* Use the zone specified for this map.
    */
   new_buckets = (GSIMapBucket)NSZoneCalloc(map->zone, size,
     sizeof(GSIMapBucket_t));
+#endif
+
   if (new_buckets != 0)
     {
       GSIMapRemangleBuckets(map, map->buckets, map->bucketCount, new_buckets,
 	size);
-
+#if     !GS_WITH_GC
       if (map->buckets != 0)
 	{
 	  NSZoneFree(map->zone, map->buckets);
 	}
+#endif
       map->buckets = new_buckets;
       map->bucketCount = size;
     }
@@ -677,12 +781,31 @@ GSIMapEnumeratorForMap(GSIMapTable map)
   /*
    * Locate next bucket and node to be returned.
    */
+#if	GS_WITH_GC
+  if (GSI_MAP_ZEROED(map))
+    {
+      while (enumerator.bucket < map->bucketCount)
+	{
+	  GSIMapNode	node = map->buckets[enumerator.bucket].firstNode;
+
+	  while (node != 0 && node->key.addr == 0)
+	    {
+	      node = GSIMapRemoveAndFreeNode(map, enumerator.bucket, node);
+	    }
+	  if ((enumerator.node = node) != 0)
+	    {
+	      return enumerator;
+	    }
+	  enumerator.bucket++;
+	}
+    }
+#endif
   while (enumerator.bucket < map->bucketCount)
     {
       enumerator.node = map->buckets[enumerator.bucket].firstNode;
       if (enumerator.node != 0)
 	{
-	  break;	// Got first node, and recorded its bucket.
+	  return enumerator;	// Got first node, and recorded its bucket.
 	}
       enumerator.bucket++;
     }
@@ -726,17 +849,70 @@ static INLINE GSIMapNode
 GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 {
   GSIMapNode	node = ((_GSIE)enumerator)->node;
+  GSIMapTable	map = ((_GSIE)enumerator)->map;
+
+#if	GS_WITH_GC
+  /* Find the frst available non-zeroed node.
+   */
+  if (node != 0 && GSI_MAP_ZEROED(map) && node->key.addr == 0)
+    {
+      size_t		bucketCount = map->bucketCount;
+      size_t		bucket = ((_GSIE)enumerator)->bucket;
+
+      while (node != 0 && node->key.addr == 0)
+	{
+	  node = GSIMapRemoveAndFreeNode(map, bucket, node);
+	  while (node == 0 && ++bucket < bucketCount)
+	    {
+	      node = (map->buckets[bucket]).firstNode;
+	      while (node != 0 && node->key.addr == 0)
+		{
+		  node = GSIMapRemoveAndFreeNode(map, bucket, node);
+		}
+	    }
+	  ((_GSIE)enumerator)->bucket = bucket;
+	  ((_GSIE)enumerator)->node = node;
+	}
+    }
+#endif
 
   if (node != 0)
     {
       GSIMapNode	next = node->nextInBucket;
 
+#if	GS_WITH_GC
+      if (GSI_MAP_ZEROED(map))
+	{
+	  size_t	bucket = ((_GSIE)enumerator)->bucket;
+
+	  while (next != 0 && next->key.addr == 0)
+	    {
+	      next = GSIMapRemoveAndFreeNode(map, bucket, next);
+	    }
+	}
+#endif
+
       if (next == 0)
 	{
-	  GSIMapTable	map = ((_GSIE)enumerator)->map;
 	  size_t	bucketCount = map->bucketCount;
 	  size_t	bucket = ((_GSIE)enumerator)->bucket;
 
+#if	GS_WITH_GC
+	  if (GSI_MAP_ZEROED(map))
+	    {
+	      while (next == 0 && ++bucket < bucketCount)
+		{
+		  next = (map->buckets[bucket]).firstNode;
+		  while (next != 0 && next->key.addr == 0)
+		    {
+		      next = GSIMapRemoveAndFreeNode(map, bucket, next);
+		    }
+		}
+	      ((_GSIE)enumerator)->bucket = bucket;
+	      ((_GSIE)enumerator)->node = next;
+	      return node;
+	    }
+#endif
 	  while (next == 0 && ++bucket < bucketCount)
 	    {
 	      next = (map->buckets[bucket]).firstNode;
@@ -752,63 +928,93 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 static INLINE GSIMapNode
 GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 {
-  GSIMapNode node;
+  GSIMapNode	node = map->freeNodes;
 
-  node = GSIMapNewNode(map, key, value);
-
-  if (node != 0)
+  if (node == 0)
     {
-      GSIMapRightSizeMap(map, map->nodeCount);
-      GSIMapAddNodeToMap(map, node);
+      GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
+      node = map->freeNodes;
+      if (node == 0)
+	{
+	  return 0;
+	}
     }
+  map->freeNodes = node->nextInBucket;
+  node->key = key;
+  node->value = value;
+  node->nextInBucket = 0;
+  GSIMapRightSizeMap(map, map->nodeCount);
+  GSIMapAddNodeToMap(map, node);
   return node;
 }
 
 static INLINE GSIMapNode
 GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 {
-  GSIMapNode node;
+  GSIMapNode	node = map->freeNodes;
 
-  GSI_MAP_RETAIN_KEY(map, key);
-  GSI_MAP_RETAIN_VAL(map, value);
-  node = GSIMapNewNode(map, key, value);
-
-  if (node != 0)
+  if (node == 0)
     {
-      GSIMapRightSizeMap(map, map->nodeCount);
-      GSIMapAddNodeToMap(map, node);
+      GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
+      node = map->freeNodes;
+      if (node == 0)
+	{
+	  return 0;
+	}
     }
+  map->freeNodes = node->nextInBucket;
+  node->key = key;
+  node->value = value;
+  GSI_MAP_RETAIN_KEY(map, node->key);
+  GSI_MAP_RETAIN_VAL(map, node->value);
+  node->nextInBucket = 0;
+  GSIMapRightSizeMap(map, map->nodeCount);
+  GSIMapAddNodeToMap(map, node);
   return node;
 }
 #else
 static INLINE GSIMapNode
 GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
 {
-  GSIMapNode node;
+  GSIMapNode	node = map->freeNodes;
 
-  node = GSIMapNewNode(map, key);
-
-  if (node != 0)
+  if (node == 0)
     {
-      GSIMapRightSizeMap(map, map->nodeCount);
-      GSIMapAddNodeToMap(map, node);
+      GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
+      node = map->freeNodes;
+      if (node == 0)
+	{
+	  return 0;
+	}
     }
+  map->freeNodes = node->nextInBucket;
+  node->key = key;
+  node->nextInBucket = 0;
+  GSIMapRightSizeMap(map, map->nodeCount);
+  GSIMapAddNodeToMap(map, node);
   return node;
 }
 
 static INLINE GSIMapNode
 GSIMapAddKey(GSIMapTable map, GSIMapKey key)
 {
-  GSIMapNode node;
+  GSIMapNode	node = map->freeNodes;
 
-  GSI_MAP_RETAIN_KEY(map, key);
-  node = GSIMapNewNode(map, key);
-
-  if (node != 0)
+  if (node == 0)
     {
-      GSIMapRightSizeMap(map, map->nodeCount);
-      GSIMapAddNodeToMap(map, node);
+      GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
+      node = map->freeNodes;
+      if (node == 0)
+	{
+	  return 0;
+	}
     }
+  map->freeNodes = node->nextInBucket;
+  node->key = key;
+  GSI_MAP_RETAIN_KEY(map, node->key);
+  node->nextInBucket = 0;
+  GSIMapRightSizeMap(map, map->nodeCount);
+  GSIMapAddNodeToMap(map, node);
   return node;
 }
 #endif
@@ -879,8 +1085,6 @@ GSIMapCleanMap(GSIMapTable map)
 static INLINE void
 GSIMapEmptyMap(GSIMapTable map)
 {
-  unsigned int	i;
-
 #ifdef	GSI_MAP_NOCLEAN
   if (GSI_MAP_NOCLEAN)
     {
@@ -895,18 +1099,24 @@ GSIMapEmptyMap(GSIMapTable map)
 #endif
   if (map->buckets != 0)
     {
+#if	!GS_WITH_GC
       NSZoneFree(map->zone, map->buckets);
+#endif
       map->buckets = 0;
       map->bucketCount = 0;
     }
   if (map->nodeChunks != 0)
     {
+#if	!GS_WITH_GC
+      unsigned int	i;
+
       for (i = 0; i < map->chunkCount; i++)
 	{
 	  NSZoneFree(map->zone, map->nodeChunks[i]);
 	}
-      map->chunkCount = 0;
       NSZoneFree(map->zone, map->nodeChunks);
+#endif
+      map->chunkCount = 0;
       map->nodeChunks = 0;
     }
   map->freeNodes = 0;
