@@ -11,7 +11,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +30,7 @@
 #import	<GNUstepBase/GSVersionMacros.h>
 #import	<objc/objc.h>
 
+#import	<Foundation/FoundationErrors.h>
 #import	<Foundation/NSObjCRuntime.h>
 #import <GNUstepBase/GSConfig.h>
 #import	<Foundation/NSDebug.h>
@@ -64,6 +65,7 @@
 #import	<Foundation/NSFileHandle.h>
 #import	<Foundation/NSFileManager.h>
 #import	<Foundation/NSFormatter.h>
+#import	<Foundation/NSGarbageCollector.h>
 #import	<Foundation/NSGeometry.h>
 #import	<Foundation/NSHashTable.h>
 #import	<Foundation/NSHost.h>
@@ -83,6 +85,8 @@
 #import	<Foundation/NSNull.h>
 #import	<Foundation/NSNumberFormatter.h>
 #import	<Foundation/NSPathUtilities.h>
+#import	<Foundation/NSPointerArray.h>
+#import	<Foundation/NSPointerFunctions.h>
 #import	<Foundation/NSPortCoder.h>
 #import	<Foundation/NSPortMessage.h>
 #import	<Foundation/NSPortNameServer.h>
@@ -120,6 +124,12 @@
 #import	<Foundation/NSUserDefaults.h>
 #import	<Foundation/NSValue.h>
 #import	<Foundation/NSValueTransformer.h>
+#import <Foundation/NSXMLDocument.h>
+#import <Foundation/NSXMLDTD.h>
+#import <Foundation/NSXMLDTDNode.h>
+#import <Foundation/NSXMLElement.h>
+#import <Foundation/NSXMLNode.h>
+#import <Foundation/NSXMLNodeOptions.h>
 #import	<Foundation/NSXMLParser.h>
 #import	<Foundation/NSZone.h>
 
