@@ -6,7 +6,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,13 +32,24 @@ extern "C" {
 @class	NSArray;
 @class	NSDate;
 @class	NSDictionary;
+@class  NSLocale;
 @class	NSString;
 
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+enum {
+  NSTimeZoneNameStyleStandard,
+  NSTimeZoneNameStyleShortStandard,
+  NSTimeZoneNameStyleDaylightSaving,
+  NSTimeZoneNameStyleShortDaylightSaving
+};
+typedef NSInteger NSTimeZoneNameStyle;
+#endif
+
 @interface NSTimeZone : NSObject
 
 //Creating and Initializing an NSTimeZone
 + (NSTimeZone*) localTimeZone;
-+ (NSTimeZone*) timeZoneForSecondsFromGMT: (int)seconds;
++ (NSTimeZone*) timeZoneForSecondsFromGMT: (NSInteger)seconds;
 + (NSTimeZone*) timeZoneWithName: (NSString*)aTimeZoneName;
 
 //Managing Time Zones
@@ -71,8 +82,24 @@ extern "C" {
 - (BOOL) isDaylightSavingTimeForDate: (NSDate*)aDate;
 - (BOOL) isEqualToTimeZone: (NSTimeZone*)aTimeZone;
 - (NSString*) name;
-- (int) secondsFromGMT;
-- (int) secondsFromGMTForDate: (NSDate*)aDate;
+- (NSInteger) secondsFromGMT;
+- (NSInteger) secondsFromGMTForDate: (NSDate*)aDate;
+#endif
+
+#if OS_API_VERSION(100500,GS_API_LATEST) 
+/** Not implemented */
+- (NSTimeInterval) daylightSavingTimeOffsetForDate: (NSDate *)aDate;
+/** Not implemented */
+- (NSDate *) nextDaylightSavingTimeTransitionAfterDate: (NSDate *)aDate;
+/** Not implemented */
+- (NSTimeInterval) daylightSavingTimeOffset;
+/** Not implemented */
+- (NSDate *) nextDaylightSavingTimeTransition;
+
+- (NSString *)localizedName: (NSTimeZoneNameStyle)style
+                     locale: (NSLocale *)locale;
+GS_EXPORT NSString * const NSSystemTimeZoneDidChangeNotification;
+
 #endif
 
 #if	OS_API_VERSION(GS_API_OPENSTEP, GS_API_MACOSX)
@@ -87,13 +114,14 @@ extern "C" {
  */
 + (NSTimeZone*) defaultTimeZone;
 + (NSTimeZone*) timeZoneWithAbbreviation: (NSString*)abbreviation;  
+
 @end
 
 #if	OS_API_VERSION(GS_API_OPENSTEP, GS_API_MACOSX)
 @interface NSTimeZoneDetail : NSTimeZone
 - (BOOL) isDaylightSavingTimeZone;
 - (NSString*) timeZoneAbbreviation;
-- (int) timeZoneSecondsFromGMT;
+- (NSInteger) timeZoneSecondsFromGMT;
 @end
 #endif
 
