@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -63,7 +63,7 @@ extern "C" {
  */
 - (id) initWithURL: (NSURL *)URL
   MIMEType: (NSString *)MIMEType
-  expectedContentLength: (int)length
+  expectedContentLength: (NSInteger)length
   textEncodingName: (NSString *)name;
 
 /**
@@ -107,7 +107,7 @@ extern "C" {
 /**
  * Returns a string representation of a status code.
  */
-+ (NSString *) localizedStringForStatusCode: (int)statusCode;
++ (NSString *) localizedStringForStatusCode: (NSInteger)statusCode;
 
 /**
  * Returns a dictionary containing all the HTTP header fields.
@@ -117,7 +117,7 @@ extern "C" {
 /**
  * Returns the HTTP status code for the response.
  */
-- (int) statusCode;
+- (NSInteger) statusCode;
 
 @end
 
