@@ -107,7 +107,7 @@ sslError(int err)
 }
 
 
-@interface	GSSSLHandle : GSFileHandle <GCFinalization>
+@interface	GSSSLHandle : GSFileHandle
 {
   SSL_CTX	*ctx;
   SSL		*ssl;
@@ -149,10 +149,10 @@ sslError(int err)
   [super closeFile];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   [self sslDisconnect];
-  [super gcFinalize];
+  [super finalize];
 }
 
 - (int) read: (void*)buf length: (int)len
@@ -195,7 +195,7 @@ sslError(int err)
    * Set non-blocking so accept won't hang if remote end goes wrong.
    */
   [self setNonBlocking: YES];
-  RETAIN(self);		// Don't get destroyed during runloop
+  IF_NO_GC([self retain];)		// Don't get destroyed during runloop
   loop = [NSRunLoop currentRunLoop];
   ret = SSL_set_fd(ssl, descriptor);
   if (ret == 1)
@@ -293,7 +293,7 @@ sslError(int err)
     {
       ssl = SSL_new(ctx);
     }
-  RETAIN(self);		// Don't get destroyed during runloop
+  IF_NO_GC([self retain];)		// Don't get destroyed during runloop
   /*
    * Set non-blocking so accept won't hang if remote end goes wrong.
    */
