@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-01-12 11:36:37 -0700 (Mon, 12 Jan 2009) $ $Revision: 27581 $
 */
 
 #include "config.h"
@@ -75,14 +75,14 @@
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   [[NSNotificationCenter defaultCenter] removeObserver: self];
-  [super gcFinalize];
+  [super finalize];
 }
 
 - (id) init
@@ -226,14 +226,14 @@
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   [[NSNotificationCenter defaultCenter] removeObserver: self];
-  [super gcFinalize];
+  [super finalize];
 }
 
 - (id) init
