@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>GSObjCRuntime function and macro reference</title>
-   $Date: 2008-12-04 18:04:00 +0000 (Thu, 04 Dec 2008) $ $Revision: 27209 $
+   $Date: 2009-02-09 00:43:17 -0700 (Mon, 09 Feb 2009) $ $Revision: 27822 $
    */
 
 #include "config.h"
@@ -1129,10 +1129,10 @@ GSSelectorTypesMatch(const char *types1, const char *types2)
       /* Ignore structure name yet compare layout.  */
       if (*types1 == '{' && *types2 == '{')
 	{
-	  while (*types1 != '=')
+	  while (*types1 != '=' && *types1 != '}')
 	    types1++;
 
-	  while (*types2 != '=')
+	  while (*types2 != '=' && *types2 != '}')
 	    types2++;
 	}
 
@@ -2342,7 +2342,7 @@ void *
 GSAutoreleasedBuffer(unsigned size)
 {
 #if GS_WITH_GC
-  return GC_malloc(size);
+  return NSAllocateCollectable(size, NSScannedOption);
 #else
 #ifdef ALIGN
 #undef ALIGN
