@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-04-19 04:03:18 -0600 (Sun, 19 Apr 2009) $ $Revision: 28234 $
    */
 
 #import "config.h"
@@ -43,7 +43,12 @@
 static SEL	eqSel;
 static SEL	oaiSel;
 
+#if	GS_WITH_GC
+static Class	GSArrayClass;
+#else
 static Class	GSInlineArrayClass;
+@class	GSInlineArray;
+#endif
 
 @class	GSArray;
 
@@ -75,7 +80,8 @@ static Class	GSInlineArrayClass;
     [NSNumber numberWithUnsignedInt: _count], @"Count",
     self, @"Array", nil, nil];
 
-  reason = [NSString stringWithFormat: @"Index %d is out of range %d (in '%@')",
+  reason = [NSString stringWithFormat:
+    @"Index %d is out of range %d (in '%@')",
     index, _count, NSStringFromSelector(sel)];
 
   exception = [NSException exceptionWithName: NSRangeException
@@ -91,7 +97,11 @@ static Class	GSInlineArrayClass;
       [self setVersion: 1];
       eqSel = @selector(isEqual:);
       oaiSel = @selector(objectAtIndex:);
+#if	GS_WITH_GC
+      GSArrayClass = [GSArray class];
+#else
       GSInlineArrayClass = [GSInlineArray class];
+#endif
     }
 }
 
@@ -137,7 +147,12 @@ static Class	GSInlineArrayClass;
     {
       unsigned	i;
 
+#if	GS_WITH_GC
+      _contents_array = NSAllocateCollectable(sizeof(id)*count,
+	NSScannedOption);
+#else
       _contents_array = NSZoneMalloc([self zone], sizeof(id)*count);
+#endif
       if (_contents_array == 0)
 	{
 	  RELEASE(self);
@@ -194,7 +209,12 @@ static Class	GSInlineArrayClass;
 				 at: &_count];
       if (_count > 0)
 	{
+#if	GS_WITH_GC
+          _contents_array = NSAllocateCollectable(sizeof(id) * _count,
+	    NSScannedOption);
+#else
 	  _contents_array = NSZoneCalloc([self zone], _count, sizeof(id));
+#endif
 	  if (_contents_array == 0)
 	    {
 	      [NSException raise: NSMallocException
@@ -347,6 +367,30 @@ static Class	GSInlineArrayClass;
       aBuffer[j++] = _contents_array[i];
     }
 }
+
+- (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
+				   objects: (id*)stackbuf
+				     count: (NSUInteger)len
+{
+  /* For immutable arrays we can return the contents pointer directly. */
+  NSUInteger count = _count - state->state;
+  state->mutationsPtr = (unsigned long *)self;
+  state->itemsPtr = _contents_array + state->state;
+  state->state += count;
+  return count;
+}
+@end
+
+#if	!GS_WITH_GC
+/* This class stores objects inline in data beyond the end of the instance.
+ * However, when GC is enabled the object data is typed, and all data after
+ * the end of the class is ignored by the garbage collector (which would
+ * mean that objects in the array could be collected).
+ * We therefore do not provide the class ewhan GC is being used.
+ */
+@interface GSInlineArray : GSArray
+{
+}
 @end
 
 @implementation	GSInlineArray
@@ -354,14 +398,12 @@ static Class	GSInlineArrayClass;
 {
   if (_contents_array)
     {
-#if	!GS_WITH_GC
       unsigned	i;
 
       for (i = 0; i < _count; i++)
 	{
 	  [_contents_array[i] release];
 	}
-#endif
     }
   NSDeallocateObject(self);
   GSNOSUPERDEALLOC;
@@ -392,6 +434,7 @@ static Class	GSInlineArrayClass;
   return self;
 }
 @end
+#endif
 
 @implementation GSMutableArray
 
@@ -406,6 +449,7 @@ static Class	GSInlineArrayClass;
 
 - (void) addObject: (id)anObject
 {
+  _version++;
   if (anObject == nil)
     {
       [NSException raise: NSInvalidArgumentException
@@ -428,6 +472,7 @@ static Class	GSInlineArrayClass;
     }
   _contents_array[_count] = RETAIN(anObject);
   _count++;	/* Do this AFTER we have retained the object.	*/
+  _version++;
 }
 
 /**
@@ -437,13 +482,18 @@ static Class	GSInlineArrayClass;
 {
   NSArray       *copy;
 
+#if	GS_WITH_GC
+  copy = (id)NSAllocateObject(GSArrayClass, 0, zone);
+#else
   copy = (id)NSAllocateObject(GSInlineArrayClass, sizeof(id)*_count, zone);
+#endif
   return [copy initWithObjects: _contents_array count: _count];
 }
 
 - (void) exchangeObjectAtIndex: (unsigned int)i1
              withObjectAtIndex: (unsigned int)i2
 {
+  _version++;
   if (i1 >= _count)
     {
       [self _raiseRangeExceptionWithIndex: i1 from: _cmd];
@@ -459,6 +509,7 @@ static Class	GSInlineArrayClass;
       _contents_array[i1] = _contents_array[i2];
       _contents_array[i2] = tmp;
     }
+  _version++;
 }
 
 - (id) init
@@ -472,7 +523,11 @@ static Class	GSInlineArrayClass;
     {
       cap = 1;
     }
+#if	GS_WITH_GC
+  _contents_array = NSAllocateCollectable(sizeof(id)*cap, NSScannedOption);
+#else
   _contents_array = NSZoneMalloc([self zone], sizeof(id)*cap);
+#endif
   _capacity = cap;
   _grow_factor = cap > 1 ? cap/2 : 1;
   return self;
@@ -533,8 +588,7 @@ static Class	GSInlineArrayClass;
 
 - (void) insertObject: (id)anObject atIndex: (unsigned)index
 {
-  unsigned	i;
-
+  _version++;
   if (!anObject)
     {
       NSException  *exception;
@@ -568,10 +622,8 @@ static Class	GSInlineArrayClass;
       _capacity += _grow_factor;
       _grow_factor = _capacity/2;
     }
-  for (i = _count; i > index; i--)
-    {
-      _contents_array[i] = _contents_array[i - 1];
-    }
+  memmove(&_contents_array[index+1], &_contents_array[index],
+    (_count - index) * sizeof(id));
   /*
    *	Make sure the array is 'sane' so that it can be deallocated
    *	safely by an autorelease pool if the '[anObject retain]' causes
@@ -580,6 +632,7 @@ static Class	GSInlineArrayClass;
   _contents_array[index] = nil;
   _count++;
   _contents_array[index] = RETAIN(anObject);
+  _version++;
 }
 
 - (id) makeImmutableCopyOnFail: (BOOL)force
@@ -596,6 +649,7 @@ static Class	GSInlineArrayClass;
 
 - (void) removeLastObject
 {
+  _version++;
   if (_count == 0)
     {
       [NSException raise: NSRangeException
@@ -604,12 +658,14 @@ static Class	GSInlineArrayClass;
   _count--;
   RELEASE(_contents_array[_count]);
   _contents_array[_count] = 0;
+  _version++;
 }
 
 - (void) removeObject: (id)anObject
 {
   unsigned	index;
 
+  _version++;
   if (anObject == nil)
     {
       NSWarnMLog(@"attempt to remove nil object");
@@ -655,12 +711,14 @@ static Class	GSInlineArrayClass;
 	}
 #endif
     }
+  _version++;
 }
 
 - (void) removeObjectAtIndex: (unsigned)index
 {
   id	obj;
 
+  _version++;
   if (index >= _count)
     {
       [self _raiseRangeExceptionWithIndex: index from: _cmd];
@@ -674,12 +732,14 @@ static Class	GSInlineArrayClass;
     }
   _contents_array[_count] = 0;
   RELEASE(obj);	/* Adjust array BEFORE releasing object.	*/
+  _version++;
 }
 
 - (void) removeObjectIdenticalTo: (id)anObject
 {
   unsigned	index;
 
+  _version++;
   if (anObject == nil)
     {
       NSWarnMLog(@"attempt to remove nil object");
@@ -704,12 +764,14 @@ static Class	GSInlineArrayClass;
 	  RELEASE(obj);
 	}
     }
+  _version++;
 }
 
 - (void) replaceObjectAtIndex: (unsigned)index withObject: (id)anObject
 {
   id	obj;
 
+  _version++;
   if (index >= _count)
     {
       [self _raiseRangeExceptionWithIndex: index from: _cmd];
@@ -737,6 +799,7 @@ static Class	GSInlineArrayClass;
   IF_NO_GC(RETAIN(anObject));
   _contents_array[index] = anObject;
   RELEASE(obj);
+  _version++;
 }
 
 - (void) sortUsingFunction: (NSComparisonResult(*)(id,id,void*))compare
@@ -754,6 +817,7 @@ static Class	GSInlineArrayClass;
   BOOL		badComparison = NO;
 #endif
 
+  _version++;
   while (stride <= count)
     {
       stride = stride * STRIDE_FACTOR + 1;
@@ -813,6 +877,7 @@ static Class	GSInlineArrayClass;
       NSWarnMLog(@"Detected bad return value from comparison");
     }
 #endif
+  _version++;
 }
 
 - (NSEnumerator*) objectEnumerator
@@ -831,6 +896,34 @@ static Class	GSInlineArrayClass;
   return AUTORELEASE([enumerator initWithArray: (GSArray*)self]);
 }
 
+- (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
+				   objects: (id*)stackbuf
+				     count: (NSUInteger)len
+{
+  NSInteger count;
+
+  /* This is cached in the caller at the start and compared at each
+   * iteration.   If it changes during the iteration then
+   * objc_enumerationMutation() will be called, throwing an exception.
+   */
+  state->mutationsPtr = (unsigned long *)&_version;
+  count = MIN(len, _count - state->state);
+  /* If a mutation has occurred then it's possible that we are being asked to
+   * get objects from after the end of the array.  Don't pass negative values
+   * to memcpy.
+   */
+  if (count > 0)
+    {
+      memcpy(stackbuf, _contents_array + state->state, count * sizeof(id));
+      state->state += count;
+    }
+  else
+    {
+      count = 0;
+    }
+  state->itemsPtr = stackbuf;
+  return count;
+}
 @end
 
 
@@ -1002,7 +1095,11 @@ static Class	GSInlineArrayClass;
 
 + (void) initialize
 {
+#if	GS_WITH_GC
+  GSArrayClass = [GSArray class];
+#else
   GSInlineArrayClass = [GSInlineArray class];
+#endif
 }
 
 - (id) autorelease
@@ -1039,13 +1136,21 @@ static Class	GSInlineArrayClass;
     }
   else
     {
-      GSInlineArray	*a;
       unsigned	c;
+#if	GS_WITH_GC
+      GSArray	*a;
+
+      [aCoder decodeValueOfObjCType: @encode(unsigned) at: &c];
+      a = (id)NSAllocateObject(GSArrayClass, 0, GSObjCZone(self));
+      a->_contents_array = NSAllocateCollectable(sizeof(id)*c, NSScannedOption);
+#else
+      GSInlineArray	*a;
 
       [aCoder decodeValueOfObjCType: @encode(unsigned) at: &c];
       a = (id)NSAllocateObject(GSInlineArrayClass,
 	sizeof(id)*c, GSObjCZone(self));
       a->_contents_array = (id*)&a[1];
+#endif
       if (c > 0)
         {
 	  [aCoder decodeArrayOfObjCType: @encode(id)
@@ -1059,8 +1164,12 @@ static Class	GSInlineArrayClass;
 
 - (id) initWithObjects: (id*)objects count: (unsigned)count
 {
+#if	GS_WITH_GC
+  self = (id)NSAllocateObject(GSArrayClass, 0, GSObjCZone(self));
+#else
   self = (id)NSAllocateObject(GSInlineArrayClass, sizeof(id)*count,
     GSObjCZone(self));
+#endif
   return [self initWithObjects: objects count: count];
 }
 
