@@ -104,6 +104,7 @@ static NSString	*httpVersion = @"1.1";
   BOOL			tunnel;
   BOOL			debug;
   BOOL			keepalive;
+  BOOL			returnAll;
   unsigned char		challenged;
   NSFileHandle          *sock;
   NSURL                 *url;
@@ -242,7 +243,7 @@ static void debugRead(GSHTTPURLHandle *handle, NSData *data)
 #endif
   if (d >= 0)
     {
-      s = [NSString stringWithFormat: @"\nRead for %x at %@ %u bytes - '",
+      s = [NSString stringWithFormat: @"\nRead for %p at %@ %u bytes - '",
 	handle, [NSDate date], [data length]];
       write(d, [s cString], [s cStringLength]);
       write(d, [data bytes], [data length]);
@@ -266,7 +267,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 #endif
   if (d >= 0)
     {
-      s = [NSString stringWithFormat: @"\nWrite for %x at %@ %u bytes - '",
+      s = [NSString stringWithFormat: @"\nWrite for %p at %@ %u bytes - '",
 	handle, [NSDate date], [data length]];
       write(d, [s cString], [s cStringLength]);
       write(d, [data bytes], [data length]);
@@ -291,7 +292,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
         {
 	  [urlOrder removeObjectIdenticalTo: obj];
 	  [urlOrder addObject: obj];
-          AUTORELEASE(RETAIN(obj));
+          IF_NO_GC([[obj retain] autorelease];)
 	}
       [urlLock unlock];
       //NSLog(@"Found handle %@", obj);
@@ -310,7 +311,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       debugFile = [NSString stringWithFormat: @"%@/GSHTTP.%d",
 			     NSTemporaryDirectory(),
 			     [[NSProcessInfo processInfo] processIdentifier]];
-      RETAIN(debugFile);
+      IF_NO_GC([debugFile retain];)
 
 #if	!defined(__MINGW32__)
       sslClass = [NSFileHandle sslClass];
@@ -376,7 +377,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	      [urlOrder removeObjectAtIndex: 0];
 	    }
 	  [urlLock unlock];
-	  //NSLog(@"Cache handle %@ for '%@'", self, page);
+	  //NSLog(@"Cache handle %p for '%@'", self, page);
 	}
     }
   return self;
@@ -404,8 +405,9 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   NSString		*version;
   NSMapEnumerator       enumerator;
 
-  RETAIN(self);
-  if (debug) NSLog(@"%@ %s", NSStringFromSelector(_cmd), keepalive?"K":"");
+  IF_NO_GC([self retain];)
+  if (debug)
+    NSLog(@"%@ %p %s", NSStringFromSelector(_cmd), self, keepalive?"K":"");
 
   s = [basic mutableCopy];
   if ([[u query] length] > 0)
@@ -549,9 +551,10 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   NSRange		r;
   unsigned		readCount;
 
-  RETAIN(self);
+  IF_NO_GC([self retain];)
 
-  if (debug) NSLog(@"%@ %s", NSStringFromSelector(_cmd), keepalive?"K":"");
+  if (debug)
+    NSLog(@"%@ %p %s", NSStringFromSelector(_cmd), self, keepalive?"K":"");
   d = [dict objectForKey: NSFileHandleNotificationDataItem];
   if (debug == YES) debugRead(self, d);
   readCount = [d length];
@@ -565,8 +568,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
        */
       if (debug == YES && [d length] != 0)
 	{
-	  NSLog(@"%@ %s Unexpected data (%*.*s) from remote!",
-	    NSStringFromSelector(_cmd), keepalive?"K":"",
+	  NSLog(@"%@ %p %s Unexpected data (%*.*s) from remote!",
+	    NSStringFromSelector(_cmd), self, keepalive?"K":"",
 	    [d length], [d length], [d bytes]);
 	}
       [nc removeObserver: self name: nil object: sock];
@@ -591,12 +594,12 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  GSMimeHeader	*info;
 	  NSString	*enc;
 	  NSString	*len;
-	  NSString	*status;
+	  int		status;
 	  float		ver;
 
 	  info = [document headerNamed: @"http"];
 	  ver = [[info value] floatValue];
-	  status = [info objectForKey: NSHTTPPropertyStatusCodeKey];
+	  status = [[info objectForKey: NSHTTPPropertyStatusCodeKey] intValue];
 	  len = [[document headerNamed: @"content-length"] value];
 	  enc = [[document headerNamed: @"content-transfer-encoding"] value];
 	  if (enc == nil)
@@ -604,7 +607,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	      enc = [[document headerNamed: @"transfer-encoding"] value];
 	    }
 
-	  if ([status isEqual: @"204"] || [status isEqual: @"304"])
+	  if (status == 204 || status == 304)
 	    {
 	      complete = YES;	// No body expected.
 	    }
@@ -621,6 +624,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	{
 	  GSMimeHeader	*info;
 	  NSString	*val;
+	  NSNumber	*num;
 	  float		ver;
 	  int		code;
 
@@ -640,8 +644,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	   * Retrieve essential keys from document
 	   */
 	  info = [document headerNamed: @"http"];
-	  val = [info objectForKey: NSHTTPPropertyStatusCodeKey];
-	  code = [val intValue];
+	  num = [info objectForKey: NSHTTPPropertyStatusCodeKey];
+	  code = [num intValue];
 	  if (code == 401 && self->challenged < 2)
 	    {
 	      GSMimeHeader	*ah;
@@ -718,9 +722,9 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 		    }
 		}
 	    }
-	  if (val != nil)
+	  if (num != nil)
 	    {
-	      [pageInfo setObject: val forKey: NSHTTPPropertyStatusCodeKey];
+	      [pageInfo setObject: num forKey: NSHTTPPropertyStatusCodeKey];
 	    }
 	  val = [info objectForKey: NSHTTPPropertyServerHTTPVersionKey];
 	  if (val != nil)
@@ -741,7 +745,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  bodyPos = 0;
 	  DESTROY(wData);
 	  NSResetMapTable(wProperties);
-	  if (code >= 200 && code < 300)
+	  if (returnAll || (code >= 200 && code < 300))
 	    {
 	      [self didLoadBytes: [d subdataWithRange: r]
 		    loadComplete: YES];
@@ -801,8 +805,9 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   NSData		*d;
   GSMimeParser		*p = [GSMimeParser new];
 
-  RETAIN(self);
-  if (debug) NSLog(@"%@ %s", NSStringFromSelector(_cmd), keepalive?"K":"");
+  IF_NO_GC([self retain];)
+  if (debug)
+    NSLog(@"%@ %p %s", NSStringFromSelector(_cmd), self, keepalive?"K":"");
   d = [dict objectForKey: NSFileHandleNotificationDataItem];
   if (debug == YES) debugRead(self, d);
 
@@ -815,15 +820,16 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
     {
       GSMimeHeader	*info;
       NSString		*val;
+      NSNumber		*num;
 
       [p parse: nil];
       info = [[p mimeDocument] headerNamed: @"http"];
       val = [info objectForKey: NSHTTPPropertyServerHTTPVersionKey];
       if (val != nil)
 	[pageInfo setObject: val forKey: NSHTTPPropertyServerHTTPVersionKey];
-      val = [info objectForKey: NSHTTPPropertyStatusCodeKey];
-      if (val != nil)
-	[pageInfo setObject: val forKey: NSHTTPPropertyStatusCodeKey];
+      num = [info objectForKey: NSHTTPPropertyStatusCodeKey];
+      if (num != nil)
+	[pageInfo setObject: num forKey: NSHTTPPropertyStatusCodeKey];
       val = [info objectForKey: NSHTTPPropertyStatusReasonKey];
       if (val != nil)
 	[pageInfo setObject: val forKey: NSHTTPPropertyStatusReasonKey];
@@ -875,8 +881,9 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   NSString		*method;
   NSString		*path;
 
-  RETAIN(self);
-  if (debug) NSLog(@"%@ %s", NSStringFromSelector(_cmd), keepalive?"K":"");
+  IF_NO_GC([self retain];)
+  if (debug)
+    NSLog(@"%@ %p %s", NSStringFromSelector(_cmd), self, keepalive?"K":"");
 
   path = [[u path] stringByTrimmingSpaces];
   if ([path length] == 0)
@@ -922,7 +929,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       NSTimeInterval	limit = 0.01;
       NSData		*buf;
       NSDate		*when;
-      NSString		*status;
+      int		status;
       NSString		*version;
 
       version = [request objectForKey: NSHTTPPropertyServerHTTPVersionKey];
@@ -945,7 +952,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
        * Set up default status for if connection is lost.
        */
       [pageInfo setObject: @"1.0" forKey: NSHTTPPropertyServerHTTPVersionKey];
-      [pageInfo setObject: @"503" forKey: NSHTTPPropertyStatusCodeKey];
+      [pageInfo setObject: [NSNumber numberWithInt: 503]
+		   forKey: NSHTTPPropertyStatusCodeKey];
       [pageInfo setObject: @"Connection dropped by proxy server"
 		   forKey: NSHTTPPropertyStatusReasonKey];
 
@@ -974,8 +982,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	}
       RELEASE(when);
 
-      status = [pageInfo objectForKey: NSHTTPPropertyStatusCodeKey];
-      if ([status isEqual: @"200"] == NO)
+      status = [[pageInfo objectForKey: NSHTTPPropertyStatusCodeKey] intValue];
+      if (status != 200)
 	{
 	  [self endLoadInBackground];
 	  [self backgroundLoadDidFailWithReason: @"Failed proxy tunneling"];
@@ -1046,8 +1054,9 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   NSDictionary    	*userInfo = [notification userInfo];
   NSString        	*e;
 
-  RETAIN(self);
-  if (debug) NSLog(@"%@ %s", NSStringFromSelector(_cmd), keepalive?"K":"");
+  IF_NO_GC([self retain];)
+  if (debug)
+    NSLog(@"%@ %p %s", NSStringFromSelector(_cmd), self, keepalive?"K":"");
   e = [userInfo objectForKey: GSFileHandleNotificationError];
   if (e != nil)
     {
@@ -1065,11 +1074,13 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  DESTROY(sock);
 	  connectionState = idle;
 	  if (debug)
-	    NSLog(@"%@ restart on new connection", NSStringFromSelector(_cmd));
+	    NSLog(@"%@ %p restart on new connection",
+	      NSStringFromSelector(_cmd), self);
 	  [self _tryLoadInBackground: u];
 	  return;
 	}
-      NSLog(@"Failed to write command to socket - %@", e);
+      NSLog(@"Failed to write command to socket - %@ %p %s",
+	e, self, keepalive?"K":"");
       /*
        * Tell superclass that the load failed - let it do housekeeping.
        */
@@ -1186,6 +1197,11 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   debug = flag;
 }
 
+- (void) setReturnAll: (BOOL)flag
+{
+  returnAll = flag;
+}
+
 - (void) _tryLoadInBackground: (NSURL*)fromURL
 {
   NSNotificationCenter	*nc;
@@ -1259,7 +1275,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       
       if (debug)
         {
-	  NSLog(@"%@ check for reusable socket", NSStringFromSelector(_cmd));
+	  NSLog(@"%@ %p check for reusable socket",
+	    NSStringFromSelector(_cmd), self);
 	}
       [nc addObserver: self
 	     selector: @selector(bgdRead:)
@@ -1280,7 +1297,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 
       if (debug)
         {
-	  NSLog(@"%@ check for reusable socket", NSStringFromSelector(_cmd));
+	  NSLog(@"%@ %p check for reusable socket",
+	    NSStringFromSelector(_cmd), self);
 	}
       if (fd >= 0)
         {
@@ -1306,11 +1324,13 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	{
 	  if (sock == nil)
 	    {
-	      NSLog(@"%@ socket closed by remote", NSStringFromSelector(_cmd));
+	      NSLog(@"%@ %p socket closed by remote",
+		NSStringFromSelector(_cmd), self);
 	    }
 	  else
 	    {
-	      NSLog(@"%@ socket is still open", NSStringFromSelector(_cmd));
+	      NSLog(@"%@ %p socket is still open",
+		NSStringFromSelector(_cmd), self);
 	    }
 	}
     }
@@ -1410,7 +1430,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	    host, port, [NSError _last]]];
 	  return;
 	}
-      RETAIN(sock);
+      IF_NO_GC([sock retain];)
       nc = [NSNotificationCenter defaultCenter];
       [nc addObserver: self
 	     selector: @selector(bgdConnect:)
@@ -1419,8 +1439,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       connectionState = connecting;
       if (debug)
         {
-          NSLog(@"%@ start connect to %@:%@",
-	    NSStringFromSelector(_cmd), host, port);
+          NSLog(@"%@ %p start connect to %@:%@",
+	    NSStringFromSelector(_cmd), self, host, port);
 	}
     }
   else
@@ -1509,7 +1529,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
     || [propertyKey isKindOfClass: [NSString class]] == NO)
     {
       [NSException raise: NSInvalidArgumentException
-		  format: @"%@ with invalid key", NSStringFromSelector(_cmd)];
+        format: @"%@ %p with invalid key", NSStringFromSelector(_cmd), self];
     }
   if ([propertyKey hasPrefix: @"GSHTTPProperty"]
     || [propertyKey hasPrefix: @"NSHTTPProperty"])
