@@ -1,5 +1,29 @@
 #ifndef __GSRunLoopWatcher_h_GNUSTEP_BASE_INCLUDE
 #define __GSRunLoopWatcher_h_GNUSTEP_BASE_INCLUDE
+/** 
+   Copyright (C) 2008-2009 Free Software Foundation, Inc.
+
+   By: Richard Frith-Macdonald <richard@brainstorm.co.uk>
+
+   This file is part of the GNUstep Base Library.
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
+
+   $Date: 2009-02-26 04:09:05 -0700 (Thu, 26 Feb 2009) $ $Revision: 27988 $
+*/
 
 /*
  *	The 'GSRunLoopWatcher' class was written to permit the (relatively)
