@@ -103,14 +103,20 @@ GSPrivateSockaddrLength(struct sockaddr *addr)
 - (void) bye;           /* Close down the handled session.   */
 - (BOOL) handshake;     /* A handshake/hello is in progress. */
 - (void) hello;         /* Start up the session handshake.   */
-- (int) read: (uint8_t *)buffer maxLength: (unsigned int)len;
+- (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len;
 - (void) stream: (NSStream*)stream handleEvent: (NSStreamEvent)event;
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len;
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len;
 @end
 
 
 @implementation GSStreamHandler
 
++ (void) initialize
+{
+  GSMakeWeakPointer(self, "istream");
+  GSMakeWeakPointer(self, "ostream");
+}
+
 + (void) tryInput: (GSSocketInputStream*)i output: (GSSocketOutputStream*)o
 {
   [self subclassResponsibility: _cmd];
@@ -150,7 +156,7 @@ GSPrivateSockaddrLength(struct sockaddr *addr)
   return ostream;
 }
 
-- (int) read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   [self subclassResponsibility: _cmd];
   return 0;
@@ -161,7 +167,7 @@ GSPrivateSockaddrLength(struct sockaddr *addr)
   [self subclassResponsibility: _cmd];
 }
 
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   [self subclassResponsibility: _cmd];
   return 0;
@@ -524,7 +530,7 @@ static gnutls_anon_client_credentials_t anoncred;
   return ostream;
 }
 
-- (int) read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   return gnutls_record_recv (session, buffer, len);
 }
@@ -576,7 +582,7 @@ static gnutls_anon_client_credentials_t anoncred;
     }
 }
 
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   return gnutls_record_send (session, buffer, len);
 }
@@ -797,7 +803,7 @@ static NSString * const GSSOCKSAckConn = @"GSSOCKSAckConn";
 	  address = [[NSString alloc] initWithUTF8String:
 	    (char*)inet_ntoa(addr->sin_addr)];
 	  port = [[NSString alloc] initWithFormat: @"%d",
-	    (int)GSSwapBigI16ToHost(addr->sin_port)];
+	    (NSInteger)GSSwapBigI16ToHost(addr->sin_port)];
 
           /* Now reconfigure the streams so they will actually connect
            * to the socks proxy server.
@@ -812,7 +818,7 @@ static NSString * const GSSOCKSAckConn = @"GSSOCKSAckConn";
   return self;
 }
 
-- (int) read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   return [istream _read: buffer maxLength: len];
 }
@@ -1202,7 +1208,7 @@ static NSString * const GSSOCKSAckConn = @"GSSOCKSAckConn";
     }
 }
 
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   return [ostream _write: buffer maxLength: len];
 }
@@ -1264,10 +1270,6 @@ setNonBlocking(SOCKET fd)
   [_sibling _setSibling: nil];
   _sibling = nil;
   DESTROY(_handler);
-  if (_address != 0)
-    {
-      NSZoneFree(NSDefaultMallocZone(), _address);
-    }
   [super dealloc];
 }
 
@@ -1286,24 +1288,25 @@ setNonBlocking(SOCKET fd)
 #endif
       _sock = INVALID_SOCKET;
       _handler = nil;
+      _address.s.sa_family = AF_UNSPEC;
     }
   return self;
 }
 
 - (struct sockaddr*) _address
 {
-  return (struct sockaddr*)_address;
+  return &_address.s;
 }
 
 - (id) propertyForKey: (NSString *)key
 {
   id	result = [super propertyForKey: key];
 
-  if (result == nil && _address != 0)
+  if (result == nil && _address.s.sa_family != AF_UNSPEC)
     {
       SOCKET    s = [self _sock];
 
-      switch (_address->sa_family)
+      switch (_address.s.sa_family)
         {
           case AF_INET:
             {
@@ -1402,7 +1405,7 @@ setNonBlocking(SOCKET fd)
   return result;
 }
 
-- (int) _read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) _read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   [self subclassResponsibility: _cmd];
   return -1;
@@ -1432,8 +1435,8 @@ setNonBlocking(SOCKET fd)
 }
 
 - (BOOL) _setSocketAddress: (NSString*)address
-                      port: (int)port
-                    family: (int)family
+                      port: (NSInteger)port
+                    family: (NSInteger)family
 {
   uint16_t	p = (uint16_t)port;
 
@@ -1514,18 +1517,7 @@ setNonBlocking(SOCKET fd)
 
 - (void) _setAddress: (struct sockaddr*)address
 {
-  if (_address != 0
-    && GSPrivateSockaddrLength(_address) != GSPrivateSockaddrLength(address))
-    {
-      NSZoneFree(NSDefaultMallocZone(), _address);
-      _address = 0;
-    }
-  if (_address == 0)
-    {
-      _address = (struct sockaddr*)
-	NSZoneMalloc(NSDefaultMallocZone(), GSPrivateSockaddrLength(address));
-    }
-  memcpy(_address, address, GSPrivateSockaddrLength(address));
+  memcpy(&_address.s, address, GSPrivateSockaddrLength(address));
 }
 
 - (void) _setLoopID: (void *)ref
@@ -1578,7 +1570,7 @@ setNonBlocking(SOCKET fd)
   return _sock;
 }
 
-- (int) _write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) _write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   [self subclassResponsibility: _cmd];
   return -1;
@@ -1591,6 +1583,7 @@ setNonBlocking(SOCKET fd)
 
 + (void) initialize
 {
+  GSMakeWeakPointer(self, "_sibling");
   if (self == [GSSocketInputStream class])
     {
       GSObjCAddClassBehavior(self, [GSSocketStream class]);
@@ -1622,7 +1615,7 @@ setNonBlocking(SOCKET fd)
             {
               [GSSOCKS tryInput: self output: _sibling];
             }
-          s = socket(_address->sa_family, SOCK_STREAM, 0);
+          s = socket(_address.s.sa_family, SOCK_STREAM, 0);
           if (BADSOCKET(s))
             {
               [self _recordError];
@@ -1639,8 +1632,8 @@ setNonBlocking(SOCKET fd)
         {
           [GSTLS tryInput: self output: _sibling];
         }
-      result = connect([self _sock], _address,
-        GSPrivateSockaddrLength(_address));
+      result = connect([self _sock], &_address.s,
+        GSPrivateSockaddrLength(&_address.s));
       if (socketError(result))
         {
           if (!socketWouldBlock())
@@ -1745,7 +1738,7 @@ setNonBlocking(SOCKET fd)
   _sock = INVALID_SOCKET;
 }
 
-- (int) read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   if (buffer == 0)
     {
@@ -1764,7 +1757,7 @@ setNonBlocking(SOCKET fd)
     return [_handler read: buffer maxLength: len];
 }
 
-- (int) _read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) _read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   int readLen;
 
@@ -1953,7 +1946,7 @@ setNonBlocking(SOCKET fd)
       int result;
       socklen_t len = sizeof(error);
 
-      AUTORELEASE(RETAIN(self));
+      IF_NO_GC([[self retain] autorelease];)
       [self _unschedule];
       result = getsockopt([self _sock], SOL_SOCKET, SO_ERROR, &error, &len);
 
@@ -2004,13 +1997,14 @@ setNonBlocking(SOCKET fd)
 
 + (void) initialize
 {
+  GSMakeWeakPointer(self, "_sibling");
   if (self == [GSSocketOutputStream class])
     {
       GSObjCAddClassBehavior(self, [GSSocketStream class]);
     }
 }
 
-- (int) _write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) _write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   int writeLen;
 
@@ -2092,7 +2086,7 @@ setNonBlocking(SOCKET fd)
             {
               [GSSOCKS tryInput: _sibling output: self];
             }
-          s = socket(_address->sa_family, SOCK_STREAM, 0);
+          s = socket(_address.s.sa_family, SOCK_STREAM, 0);
           if (BADSOCKET(s))
             {
               [self _recordError];
@@ -2110,8 +2104,8 @@ setNonBlocking(SOCKET fd)
           [GSTLS tryInput: _sibling output: self];
         }
 
-      result = connect([self _sock], _address,
-        GSPrivateSockaddrLength(_address));
+      result = connect([self _sock], &_address.s,
+        GSPrivateSockaddrLength(&_address.s));
       if (socketError(result))
         {
           if (!socketWouldBlock())
@@ -2218,7 +2212,7 @@ setNonBlocking(SOCKET fd)
   _sock = INVALID_SOCKET;
 }
 
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   if (buffer == 0)
     {
@@ -2356,7 +2350,7 @@ setNonBlocking(SOCKET fd)
       socklen_t len = sizeof(error);
       int result;
 
-      AUTORELEASE(RETAIN(self));
+      IF_NO_GC([[self retain] autorelease];)
       [self _schedule];
       result
 	= getsockopt((intptr_t)_loopID, SOL_SOCKET, SO_ERROR, &error, &len);
@@ -2414,6 +2408,7 @@ setNonBlocking(SOCKET fd)
 
 + (void) initialize
 {
+  GSMakeWeakPointer(self, "_sibling");
   if (self == [GSSocketServerStream class])
     {
       GSObjCAddClassBehavior(self, [GSSocketStream class]);
@@ -2447,7 +2442,7 @@ setNonBlocking(SOCKET fd)
       return;
     }
 
-  s = socket(_address->sa_family, SOCK_STREAM, 0);
+  s = socket(_address.s.sa_family, SOCK_STREAM, 0);
   if (BADSOCKET(s))
     {
       [self _recordError];
@@ -2460,9 +2455,9 @@ setNonBlocking(SOCKET fd)
     }
 
 #ifndef	BROKEN_SO_REUSEADDR
-  if (_address->sa_family == AF_INET
+  if (_address.s.sa_family == AF_INET
 #ifdef  AF_INET6
-    || _address->sa_family == AF_INET6
+    || _address.s.sa_family == AF_INET6
 #endif
   )
     {
@@ -2479,7 +2474,8 @@ setNonBlocking(SOCKET fd)
     }
 #endif
 
-  bindReturn = bind([self _sock], _address, GSPrivateSockaddrLength(_address));
+  bindReturn = bind([self _sock],
+    &_address.s, GSPrivateSockaddrLength(&_address.s));
   if (socketError(bindReturn))
     {
       [self _recordError];
@@ -2599,7 +2595,7 @@ setNonBlocking(SOCKET fd)
 
 @implementation GSInetInputStream
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   if ((self = [super init]) != nil)
     {
@@ -2616,7 +2612,7 @@ setNonBlocking(SOCKET fd)
 @implementation GSInet6InputStream
 #if	defined(AF_INET6)
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   if ((self = [super init]) != nil)
     {
@@ -2629,7 +2625,7 @@ setNonBlocking(SOCKET fd)
 }
 
 #else
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   RELEASE(self);
   return nil;
@@ -2639,7 +2635,7 @@ setNonBlocking(SOCKET fd)
 
 @implementation GSInetOutputStream
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   if ((self = [super init]) != nil)
     {
@@ -2656,7 +2652,7 @@ setNonBlocking(SOCKET fd)
 @implementation GSInet6OutputStream
 #if	defined(AF_INET6)
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   if ((self = [super init]) != nil)
     {
@@ -2669,7 +2665,7 @@ setNonBlocking(SOCKET fd)
 }
 
 #else
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   RELEASE(self);
   return nil;
@@ -2689,7 +2685,7 @@ setNonBlocking(SOCKET fd)
   return [GSInetOutputStream class];
 }
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   if ((self = [super init]) != nil)
     {
@@ -2719,7 +2715,7 @@ setNonBlocking(SOCKET fd)
   return [GSInet6OutputStream class];
 }
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   if ([super init] != nil)
     {
@@ -2735,7 +2731,7 @@ setNonBlocking(SOCKET fd)
   return self;
 }
 #else
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   RELEASE(self);
   return nil;
