@@ -143,6 +143,11 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 @implementation GSStream
 
++ (void) initialize
+{
+  GSMakeWeakPointer(self, "delegate");
+}
+
 - (void) close
 {
   if (_currentStatus == NSStreamStatusNotOpen)
@@ -151,19 +156,25 @@ static RunLoopEventType typeForStream(NSStream *aStream)
     }
   [self _unschedule];
   [self _setStatus: NSStreamStatusClosed];
-  /* We don't want to send any events the the delegate after the
+  /* We don't want to send any events to the delegate after the
    * stream has been closed.
    */
   _delegateValid = NO;
 }
 
-- (void) dealloc
+- (void) finalize
 {
   if (_currentStatus != NSStreamStatusNotOpen
     && _currentStatus != NSStreamStatusClosed)
     {
       [self close];
     }
+  GSAssignZeroingWeakPointer((void**)&_delegate, (void*)0);
+}
+
+- (void) dealloc
+{
+  [self finalize];
   if (_loops != 0)
     {
       NSFreeMapTable(_loops);
@@ -270,17 +281,30 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) setDelegate: (id)delegate
 {
-  if (delegate)
+  if ([self streamStatus] == NSStreamStatusClosed
+    || [self streamStatus] == NSStreamStatusError)
     {
-      _delegate = delegate;
+      _delegateValid = NO;
+      GSAssignZeroingWeakPointer((void**)&_delegate, (void*)0);
     }
   else
     {
-      _delegate = self;
-    }
-  if ([self streamStatus] != NSStreamStatusClosed
-    && [self streamStatus] != NSStreamStatusError)
-    {
+      if (delegate == nil)
+	{
+	  _delegate = self;
+	}
+      if (delegate == self)
+	{
+	  if (_delegate != nil && _delegate != self)
+	    {
+              GSAssignZeroingWeakPointer((void**)&_delegate, (void*)0);
+	    }
+	  _delegate = delegate;
+	}
+      else
+	{
+          GSAssignZeroingWeakPointer((void**)&_delegate, (void*)delegate);
+	}
       /* We don't want to send any events the the delegate after the
        * stream has been closed.
        */
@@ -337,7 +361,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   return;
 }
 
-- (void) _resetEvents: (int)mask
+- (void) _resetEvents: (NSUInteger)mask
 {
   return;
 }
@@ -398,7 +422,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   _currentStatus = NSStreamStatusError;
 }
 
-- (void) _resetEvents: (int)mask
+- (void) _resetEvents: (NSUInteger)mask
 {
   _events &= ~mask;
 }
@@ -626,6 +650,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   if (self == [GSInputStream class])
     {
       GSObjCAddClassBehavior(self, [GSStream class]);
+      GSMakeWeakPointer(self, "delegate");
     }
 }
 
@@ -658,6 +683,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   if (self == [GSOutputStream class])
     {
       GSObjCAddClassBehavior(self, [GSStream class]);
+      GSMakeWeakPointer(self, "delegate");
     }
 }
 
@@ -696,7 +722,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   [super dealloc];
 }
 
-- (int) read: (uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   unsigned long dataSize;
   unsigned long copySize;
@@ -741,7 +767,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   return copySize;
 }
 
-- (BOOL) getBuffer: (uint8_t **)buffer length: (unsigned int *)len
+- (BOOL) getBuffer: (uint8_t **)buffer length: (NSUInteger *)len
 {
   unsigned long dataSize = [_data length];
 
@@ -781,7 +807,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 @implementation GSBufferOutputStream
 
-- (id) initToBuffer: (uint8_t *)buffer capacity: (unsigned int)capacity
+- (id) initToBuffer: (uint8_t *)buffer capacity: (NSUInteger)capacity
 {
   if ((self = [super init]) != nil)
     {
@@ -792,7 +818,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   return self;
 }
 
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   if (buffer == 0)
     {
@@ -864,7 +890,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   [super dealloc];
 }
 
-- (int) write: (const uint8_t *)buffer maxLength: (unsigned int)len
+- (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
   if (buffer == 0)
     {
@@ -929,7 +955,12 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 @implementation GSServerStream
 
-+ (id) serverStreamToAddr: (NSString*)addr port: (int)port
++ (void) initialize
+{
+  GSMakeWeakPointer(self, "delegate");
+}
+
++ (id) serverStreamToAddr: (NSString*)addr port: (NSInteger)port
 {
   GSServerStream *s;
 
@@ -945,7 +976,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   return AUTORELEASE([[GSLocalServerStream alloc] initToAddr: addr]);
 }
 
-- (id) initToAddr: (NSString*)addr port: (int)port
+- (id) initToAddr: (NSString*)addr port: (NSInteger)port
 {
   RELEASE(self);
   // try inet first, then inet6
