@@ -20,7 +20,8 @@
   
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 /*
@@ -40,7 +41,7 @@
 
 /*
  *	Some standard selectors for frequently used methods. Set in NSString
- *      +initialize.
+ *      +initialize or the GSString.m setup() function.
  */
 static SEL	caiSel = NULL;
 static SEL	gcrSel = NULL;
@@ -380,10 +381,13 @@ GSEQ_STRCOMP(NSString *ss, NSString *os, unsigned mask, NSRange aRange)
   unsigned	oLength;			/* Length of other.	*/
   unsigned	sLength = GSEQ_SLEN;
 
+#if	0
+  /* Range should be checked in calling code */
   if (aRange.location > sLength)
     [NSException raise: NSRangeException format: @"Invalid location."];
   if (aRange.length > (sLength - aRange.location))
     [NSException raise: NSRangeException format: @"Invalid location+length."];
+#endif
 
   oLength = GSEQ_OLEN;
   if (aRange.length == 0)
@@ -603,11 +607,14 @@ GSEQ_STRRANGE(NSString *ss, NSString *os, unsigned mask, NSRange aRange)
   
   /* Check that the search range is reasonable */
   myLength = GSEQ_SLEN;
+
+#if	0
+  /* Range should be checked in calling code */
   if (aRange.location > myLength)
     [NSException raise: NSRangeException format: @"Invalid location."];
   if (aRange.length > (myLength - aRange.location))
     [NSException raise: NSRangeException format: @"Invalid location+length."];
-
+#endif
 
   /* Ensure the string can be found */
   strLength = GSEQ_OLEN;
