@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSArchiver class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-09 09:11:51 -0600 (Mon, 09 Mar 2009) $ $Revision: 28054 $
    */
 
 #import "config.h"
@@ -36,6 +36,11 @@
 #define	GSI_MAP_HASH(M, X)	((X).uint)
 #define	GSI_MAP_EQUAL(M, X,Y)	((X).ptr == (Y).ptr)
 #define	GSI_MAP_NOCLEAN	1
+#if	GS_WITH_GC
+#define	GSI_MAP_NODES(M, X) \
+(GSIMapNode)NSAllocateCollectable(X * sizeof(GSIMapNode_t), 0)
+#endif
+
 
 #include "GNUstepBase/GSIMap.h"
 
@@ -137,7 +142,12 @@ static Class	NSMutableDataMallocClass;
       /*
        *	Set up map tables.
        */
+#if	GS_WITH_GC
+      _clsMap = (GSIMapTable)NSAllocateCollectable(sizeof(GSIMapTable_t)*6,
+	NSScannedOption);
+#else
       _clsMap = (GSIMapTable)NSZoneMalloc(zone, sizeof(GSIMapTable_t)*6);
+#endif
       _cIdMap = &_clsMap[1];
       _uIdMap = &_clsMap[2];
       _ptrMap = &_clsMap[3];
@@ -234,9 +244,10 @@ static Class	NSMutableDataMallocClass;
 }
 
 - (void) encodeArrayOfObjCType: (const char*)type
-			 count: (unsigned)count
+			 count: (NSUInteger)count
 			    at: (const void*)buf
 {
+  unsigned      c = count;
   unsigned	i;
   unsigned	offset = 0;
   unsigned	size = objc_sizeof_type(type);
@@ -269,9 +280,9 @@ static Class	NSMutableDataMallocClass;
       if (_initialPass == NO)
 	{
 	  (*_tagImp)(_dst, tagSel, _GSC_ARY_B);
-	  (*_serImp)(_dst, serSel, &count, @encode(unsigned), nil);
+	  (*_serImp)(_dst, serSel, &c, @encode(unsigned), nil);
 	}
-      for (i = 0; i < count; i++)
+      for (i = 0; i < c; i++)
 	{
 	  (*_eValImp)(self, eValSel, type, (char*)buf + offset);
 	  offset += size;
@@ -280,10 +291,10 @@ static Class	NSMutableDataMallocClass;
   else if (_initialPass == NO)
     {
       (*_tagImp)(_dst, tagSel, _GSC_ARY_B);
-      (*_serImp)(_dst, serSel, &count, @encode(unsigned), nil);
+      (*_serImp)(_dst, serSel, &c, @encode(unsigned), nil);
 
       (*_tagImp)(_dst, tagSel, info);
-      for (i = 0; i < count; i++)
+      for (i = 0; i < c; i++)
 	{
 	  (*_serImp)(_dst, serSel, (char*)buf + offset, type, nil);
 	  offset += size;
@@ -302,7 +313,7 @@ static Class	NSMutableDataMallocClass;
 
       case _C_ARY_B:
 	{
-	  int		count = atoi(++type);
+	  unsigned	count = atoi(++type);
 
 	  while (isdigit(*type))
 	    {
