@@ -27,7 +27,7 @@
    Boston, MA 02111 USA.
 
    <title>NSArray class reference</title>
-   $Date: 2008-11-17 13:45:32 +0000 (Mon, 17 Nov 2008) $ $Revision: 27080 $
+   $Date: 2009-04-28 14:00:59 -0600 (Tue, 28 Apr 2009) $ $Revision: 28262 $
    */
 
 #import "config.h"
@@ -57,13 +57,9 @@
 
 static BOOL GSMacOSXCompatiblePropertyLists(void)
 {
-#if	defined(HAVE_LIBXML)
   if (GSPrivateDefaultsFlag(NSWriteOldStylePropertyLists) == YES)
     return NO;
   return GSPrivateDefaultsFlag(GSMacOSXCompatible);
-#else
-  return NO;
-#endif
 }
 
 extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
@@ -84,7 +80,6 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
 
 static Class NSArrayClass;
 static Class GSArrayClass;
-static Class GSInlineArrayClass;
 static Class NSMutableArrayClass;
 static Class GSMutableArrayClass;
 static Class GSPlaceholderArrayClass;
@@ -125,7 +120,6 @@ static SEL	rlSel;
       NSArrayClass = [NSArray class];
       NSMutableArrayClass = [NSMutableArray class];
       GSArrayClass = [GSArray class];
-      GSInlineArrayClass = [GSInlineArray class];
       GSMutableArrayClass = [GSMutableArray class];
       GSPlaceholderArrayClass = [GSPlaceholderArray class];
 
@@ -271,7 +265,7 @@ static SEL	rlSel;
  * Returns an autoreleased array containing the specified
  * objects, preserving order.
  */
-+ (id) arrayWithObjects: (id*)objects count: (unsigned)count
++ (id) arrayWithObjects: (id*)objects count: (NSUInteger)count
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithObjects: objects count: count]);
@@ -382,12 +376,49 @@ static SEL	rlSel;
 /** <override-subclass />
  * Returns the number of elements contained in the receiver.
  */
-- (unsigned) count
+- (NSUInteger) count
 {
   [self subclassResponsibility: _cmd];
   return 0;
 }
 
+- (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
+				   objects: (id*)stackbuf
+				     count: (NSUInteger)len
+{
+  NSUInteger size = [self count];
+  NSInteger count;
+
+  /* This is cached in the caller at the start and compared at each
+   * iteration.   If it changes during the iteration then
+   * objc_enumerationMutation() will be called, throwing an exception.
+   */
+  state->mutationsPtr = (unsigned long *)size;
+  count = MIN(len, size - state->state);
+  /* If a mutation has occurred then it's possible that we are being asked to
+   * get objects from after the end of the array.  Don't pass negative values
+   * to memcpy.
+   */
+  if (count > 0)
+    {
+      IMP	imp = [self methodForSelector: @selector(objectAtIndex:)];
+      int	p = state->state;
+      int	i;
+
+      for (i = 0; i < count; i++, p++)
+	{
+	  stackbuf[i] = (*imp)(self, @selector(objectAtIndex:), p);
+	}
+      state->state += count;
+    }
+  else
+    {
+      count = 0;
+    }
+  state->itemsPtr = stackbuf;
+  return count;
+}
+
 /**
  * Encodes the receiver for storing to archive or sending over an
  * [NSConnection].
@@ -469,7 +500,7 @@ static SEL	rlSel;
 /**
  * Returns the same value as -count
  */
-- (unsigned) hash
+- (NSUInteger) hash
 {
   return [self count];
 }
@@ -478,7 +509,7 @@ static SEL	rlSel;
  * Returns the index of the specified object in the receiver, or
  * NSNotFound if the object is not present.
  */
-- (unsigned) indexOfObjectIdenticalTo: (id)anObject
+- (NSUInteger) indexOfObjectIdenticalTo: (id)anObject
 {
   unsigned c = [self count];
 
@@ -498,7 +529,7 @@ static SEL	rlSel;
  * Returns the index of the specified object in the range of the receiver,
  * or NSNotFound if the object is not present.
  */
-- (unsigned) indexOfObjectIdenticalTo: anObject inRange: (NSRange)aRange
+- (NSUInteger) indexOfObjectIdenticalTo: anObject inRange: (NSRange)aRange
 {
   unsigned i, e = aRange.location + aRange.length, c = [self count];
   IMP	get = [self methodForSelector: oaiSel];
@@ -516,7 +547,7 @@ static SEL	rlSel;
  * which is equal to anObject (using anObject's [NSObject-isEqual:] method).
  * Returns NSNotFound on failure.
  */
-- (unsigned) indexOfObject: (id)anObject
+- (NSUInteger) indexOfObject: (id)anObject
 {
   unsigned	c = [self count];
 
@@ -539,7 +570,7 @@ static SEL	rlSel;
  * which is equal to anObject (using anObject's [NSObject-isEqual:] method).
  * Returns NSNotFound on failure.
  */
-- (unsigned) indexOfObject: (id)anObject inRange: (NSRange)aRange
+- (NSUInteger) indexOfObject: (id)anObject inRange: (NSRange)aRange
 {
   unsigned i, e = aRange.location + aRange.length, c = [self count];
   IMP	get = [self methodForSelector: oaiSel];
@@ -837,7 +868,7 @@ static SEL	rlSel;
  * and needs to be re-implemented in subclasses in order to have all
  * other initialisers work.
  */
-- (id) initWithObjects: (id*)objects count: (unsigned)count
+- (id) initWithObjects: (id*)objects count: (NSUInteger)count
 {
   self = [self init];
   return self;
@@ -872,7 +903,7 @@ static SEL	rlSel;
  * Returns the object at the specified index.
  * Raises an exception of the index is beyond the array.
  */
-- (id) objectAtIndex: (unsigned)index
+- (id) objectAtIndex: (NSUInteger)index
 {
   [self subclassResponsibility: _cmd];
   return nil;
@@ -1226,7 +1257,7 @@ compare(id elem1, id elem2, void* context)
  * they appear in the receiver.
  */
 - (NSString*) descriptionWithLocale: (NSDictionary*)locale
-			     indent: (unsigned int)level
+			     indent: (NSUInteger)level
 {
   NSString	*result = nil;
 
@@ -1357,7 +1388,7 @@ compare(id elem1, id elem2, void* context)
 
 - (id) valueForKeyPath: (NSString*)path
 {
-  id result = nil;
+  id	result = nil;
 
   if ([path hasPrefix: @"@"])
     {
@@ -1635,7 +1666,7 @@ compare(id elem1, id elem2, void* context)
  * and needs to be re-implemented in subclasses in order to have all
  * other initialisers work.
  */
-- (id) initWithCapacity: (unsigned)numItems
+- (id) initWithCapacity: (NSUInteger)numItems
 {
   self = [self init];
   return self;
@@ -1654,12 +1685,12 @@ compare(id elem1, id elem2, void* context)
  * Swaps the positions of two objects in the array.  Raises an exception
  * if either array index is out of bounds.
  */
-- (void) exchangeObjectAtIndex: (unsigned int)i1
-             withObjectAtIndex: (unsigned int)i2
+- (void) exchangeObjectAtIndex: (NSUInteger)i1
+             withObjectAtIndex: (NSUInteger)i2
 {
   id	tmp = [self objectAtIndex: i1];
 
-  RETAIN(tmp);
+  IF_NO_GC([tmp retain];)
   [self replaceObjectAtIndex: i1 withObject: [self objectAtIndex: i2]];
   [self replaceObjectAtIndex: i2 withObject: tmp];
   RELEASE(tmp);
@@ -1670,16 +1701,27 @@ compare(id elem1, id elem2, void* context)
  * Raises an exception if given an array index which is too large.<br />
  * The object is retained by the array.
  */
-- (void) replaceObjectAtIndex: (unsigned)index withObject: (id)anObject
+- (void) replaceObjectAtIndex: (NSUInteger)index withObject: (id)anObject
 {
   [self subclassResponsibility: _cmd];
 }
 
-/** Not implemented
+/** Replaces the values in the receiver at the locations given by the
+ * indexes set with values from the objects array.
  */
 - (void) replaceObjectsAtIndexes: (NSIndexSet *)indexes
                      withObjects: (NSArray *)objects
 {
+  NSUInteger	index = [indexes firstIndex];
+  NSEnumerator	*enumerator = [objects objectEnumerator];
+  id		object = [enumerator nextObject];
+
+  while (object != nil && index != NSNotFound)
+    {
+      [self replaceObjectAtIndex: index withObject: object];
+      object = [enumerator nextObject];
+      index = [indexes indexGreaterThanIndex: index];
+    }
 }
 
 /**
@@ -1718,15 +1760,28 @@ compare(id elem1, id elem2, void* context)
  * The size of the array increases by one.<br />
  * The object is retained by the array.
  */
-- (void) insertObject: anObject atIndex: (unsigned)index
+- (void) insertObject: anObject atIndex: (NSUInteger)index
 {
   [self subclassResponsibility: _cmd];
 }
 
-/** Not implemented
+/** Inserts the values from the objects array into the receiver at the
+ * locations given by the indexes set.<br />
+ * The values are inserted in the same order that they appear in the
+ * array.
  */
 - (void) insertObjects: (NSArray *)objects atIndexes: (NSIndexSet *)indexes
 {
+  NSUInteger	index = [indexes firstIndex];
+  NSEnumerator	*enumerator = [objects objectEnumerator];
+  id		object = [enumerator nextObject];
+
+  while (object != nil && index != NSNotFound)
+    {
+      [self insertObject: object atIndex: index];
+      object = [enumerator nextObject];
+      index = [indexes indexGreaterThanIndex: index];
+    }
 }
 
 /** <override-subclass />
@@ -1734,7 +1789,7 @@ compare(id elem1, id elem2, void* context)
  * The size of the array decreases by one.<br />
  * Raises an exception if given an array index which is too large.<br />
  */
-- (void) removeObjectAtIndex: (unsigned)index
+- (void) removeObjectAtIndex: (NSUInteger)index
 {
   [self subclassResponsibility: _cmd];
 }
@@ -1743,7 +1798,7 @@ compare(id elem1, id elem2, void* context)
  * Creates an autoreleased mutable array able to store at least numItems.
  * See the -initWithCapacity: method.
  */
-+ (id) arrayWithCapacity: (unsigned)numItems
++ (id) arrayWithCapacity: (NSUInteger)numItems
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithCapacity: numItems]);
@@ -1752,7 +1807,7 @@ compare(id elem1, id elem2, void* context)
 /**
  * Override our superclass's designated initializer to go our's
  */
-- (id) initWithObjects: (id*)objects count: (unsigned)count
+- (id) initWithObjects: (id*)objects count: (NSUInteger)count
 {
   self = [self initWithCapacity: count];
   if (count > 0)
@@ -1858,7 +1913,7 @@ compare(id elem1, id elem2, void* context)
 		   * first equal object we don't get left with a bad object
 		   * pointer for later comparisons.
 		   */
-		  RETAIN(anObject);
+		  IF_NO_GC([anObject retain];)
 		}
 	      (*rem)(self, remSel, i);
 	    }
@@ -1950,7 +2005,7 @@ compare(id elem1, id elem2, void* context)
 		   * first equal object we don't get left with a bad object
 		   * pointer for later comparisons.
 		   */
-		  RETAIN(anObject);
+		  IF_NO_GC([anObject retain];)
 		}
 	      (*rem)(self, remSel, i);
 	    }
@@ -2015,8 +2070,8 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObjectsAtIndexes: (NSIndexSet *)indexes
 {
-  unsigned count = [indexes count];
-  unsigned indexArray[count];
+  NSUInteger count = [indexes count];
+  NSUInteger indexArray[count];
 
   [indexes getIndexes: indexArray 
              maxCount: count 
@@ -2032,15 +2087,15 @@ compare(id elem1, id elem2, void* context)
  * removed in such a way that the removal is <em>safe</em> irrespective
  * of the order in which they are specified in the indices array.
  */
-- (void) removeObjectsFromIndices: (unsigned*)indices
-		       numIndices: (unsigned)count
+- (void) removeObjectsFromIndices: (NSUInteger*)indices
+		       numIndices: (NSUInteger)count
 {
   if (count > 0)
     {
       unsigned	to = 0;
       unsigned	from = 0;
       unsigned	i;
-      GS_BEGINITEMBUF(sorted, count, unsigned int);
+      GS_BEGINITEMBUF(sorted, count, NSUInteger);
 
       while (from < count)
 	{
