@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSAssertionHandler class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
    */
 
 #include "config.h"
@@ -89,7 +89,7 @@ static NSString *dict_key = @"NSAssertionHandler";
  */
 - (void) handleFailureInFunction: (NSString*)functionName
 			    file: (NSString*)fileName
-		      lineNumber: (int)line
+		      lineNumber: (NSInteger)line
 		     description: (NSString*)format,...
 {
   id		message;
@@ -116,7 +116,7 @@ static NSString *dict_key = @"NSAssertionHandler";
 - (void) handleFailureInMethod: (SEL) aSelector
                         object: object
                           file: (NSString *) fileName
-                    lineNumber: (int) line
+                    lineNumber: (NSInteger) line
                    description: (NSString *) format,...
 {
   id		message;
