@@ -23,12 +23,13 @@
    Boston, MA 02111 USA.
 
    <title>NSAutoreleasePool class reference</title>
-   $Date: 2008-08-25 16:39:02 +0100 (Mon, 25 Aug 2008) $ $Revision: 26819 $
+   $Date: 2009-01-22 08:59:06 -0700 (Thu, 22 Jan 2009) $ $Revision: 27654 $
    */
 
 #include "config.h"
 #include "GNUstepBase/preface.h"
 #include "Foundation/NSAutoreleasePool.h"
+#include "Foundation/NSGarbageCollector.h"
 #include "Foundation/NSException.h"
 #include "Foundation/NSThread.h"
 #include "Foundation/NSZone.h"
@@ -307,7 +308,20 @@ static IMP	initImp;
 
 - (void) drain
 {
-  [self dealloc];
+#if	GS_WITH_GC
+  static NSGarbageCollector	*collector = nil;
+  static SEL			sel;
+  static IMP			imp;
+
+  if (collector == nil)
+    {
+      collector = [NSGarbageCollector defaultCollector];
+      sel = @selector(collectIfNeeded);
+      imp = [collector methodForSelector: sel];
+    }
+  (*imp)(collector, sel);
+#endif
+  return;
 }
 
 - (id) retain
