@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSClassDescription class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-01-12 05:48:46 -0700 (Mon, 12 Jan 2009) $ $Revision: 27578 $
 */
 
 #include "Foundation/NSClassDescription.h"
@@ -74,7 +74,7 @@ static NSMapTable	*classMap;
                         object: aClass];
       description = NSMapGet(classMap, aClass);
     }
-  RETAIN(description);
+  IF_NO_GC([description retain];)
   [mapLock unlock];
 
   return AUTORELEASE(description);
