@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSConnection class reference</title>
-   $Date: 2008-12-17 08:49:50 +0000 (Wed, 17 Dec 2008) $ $Revision: 27320 $
+   $Date: 2009-03-09 09:11:51 -0600 (Mon, 09 Mar 2009) $ $Revision: 28054 $
    */
 
 #import "config.h"
@@ -48,6 +48,14 @@
 #define	GSI_MAP_HASH(M, X)	((X).uint ^ ((X).uint >> 3))
 #define	GSI_MAP_EQUAL(M, X,Y)	((X).ptr == (Y).ptr)
 #define	GSI_MAP_NOCLEAN	1
+#if	GS_WITH_GC
+// FIXME ... 
+#include	<gc_typed.h>
+static GC_descr	nodeDesc;	// Type descriptor for map node.
+#define	GSI_MAP_NODES(M, X) \
+(GSIMapNode)GC_calloc_explicitly_typed(X, sizeof(GSIMapNode_t), nodeDesc)
+#endif
+
 
 #include "GNUstepBase/GSIMap.h"
 
@@ -111,8 +119,8 @@ GSRunLoopForThread(NSThread *aThread)
 - (NSMutableArray*) _components;
 @end
 
-@interface NSConnection (GNUstepExtensions) <GCFinalization>
-- (void) gcFinalize;
+@interface NSConnection (GNUstepExtensions)
+- (void) finalize;
 - (retval_t) forwardForProxy: (NSDistantObject*)object 
 		    selector: (SEL)sel 
 		    argFrame: (arglist_t)argframe;
@@ -298,7 +306,7 @@ existingConnection(NSPort *receivePort, NSPort *sendPort)
 	   * We don't want this connection to be destroyed by another thread
 	   * between now and when it's returned from this function and used!
 	   */
-	  AUTORELEASE(RETAIN(c));
+	  IF_NO_GC([[c retain] autorelease];)
 	  break;
 	}
     }
@@ -386,7 +394,7 @@ static NSLock	*cached_proxies_gate = nil;
     {
       c = [self allocWithZone: NSDefaultMallocZone()];
       c = [c initWithReceivePort: r sendPort: s];
-      AUTORELEASE(c);
+      IF_NO_GC([c autorelease];)
     }
   return c;
 }
@@ -545,6 +553,17 @@ static NSLock	*cached_proxies_gate = nil;
     {
       NSNotificationCenter	*nc;
 
+      GSMakeWeakPointer(self, "delegate");
+
+#if	GS_WITH_GC
+      /* We create a typed memory descriptor for map nodes.
+       * FIXME
+       */
+      GC_word	w[GC_BITMAP_SIZE(GSIMapNode_t)] = {0};
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, key));
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, value));
+      nodeDesc = GC_make_descriptor(w, GC_WORD_LEN(GSIMapNode_t));
+#endif
       connectionClass = self;
       dateClass = [NSDate class];
       distantObjectClass = [NSDistantObject class];
@@ -766,7 +785,7 @@ static NSLock	*cached_proxies_gate = nil;
 {
   if (debug_connection)
     NSLog(@"deallocating %@", self);
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -952,7 +971,12 @@ static NSLock	*cached_proxies_gate = nil;
    * This maps request sequence numbers to the NSPortCoder objects representing
    * replies arriving from the remote connection.
    */
+#if	GS_WITH_GC
+  _replyMap = (GSIMapTable)NSAllocateCollectable(sizeof(GSIMapTable_t),
+    NSScannedOption);
+#else
   _replyMap = (GSIMapTable)NSZoneMalloc(z, sizeof(GSIMapTable_t));
+#endif
   GSIMapInitWithZoneAndCapacity(_replyMap, z, 4);
 
   /*
@@ -960,19 +984,34 @@ static NSLock	*cached_proxies_gate = nil;
    * We use this instead of an NSHashTable because we only care about
    * the object's address, and don't want to send the -hash message to it.
    */
+#if	GS_WITH_GC
+  _localObjects= (GSIMapTable)NSAllocateCollectable(sizeof(GSIMapTable_t),
+    NSScannedOption);
+#else
   _localObjects = (GSIMapTable)NSZoneMalloc(z, sizeof(GSIMapTable_t));
+#endif
   GSIMapInitWithZoneAndCapacity(_localObjects, z, 4);
 
   /*
    * This maps handles for local objects to their local proxies.
    */
+#if	GS_WITH_GC
+  _localTargets = (GSIMapTable)NSAllocateCollectable(sizeof(GSIMapTable_t),
+    NSScannedOption);
+#else
   _localTargets = (GSIMapTable)NSZoneMalloc(z, sizeof(GSIMapTable_t));
+#endif
   GSIMapInitWithZoneAndCapacity(_localTargets, z, 4);
 
   /*
    * This maps targets to remote proxies.
    */
+#if	GS_WITH_GC
+  _remoteProxies = (GSIMapTable)NSAllocateCollectable(sizeof(GSIMapTable_t),
+    NSScannedOption);
+#else
   _remoteProxies = (GSIMapTable)NSZoneMalloc(z, sizeof(GSIMapTable_t));
+#endif
   GSIMapInitWithZoneAndCapacity(_remoteProxies, z, 4);
 
   _requestDepth = 0;
@@ -1150,7 +1189,7 @@ static NSLock	*cached_proxies_gate = nil;
    */
   [self setRequestMode: nil];
 
-  RETAIN(self);
+  IF_NO_GC([self retain];)
 
   if (debug_connection)
     {
@@ -1742,7 +1781,7 @@ static NSLock	*cached_proxies_gate = nil;
   return conn;
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   CREATE_AUTORELEASE_POOL(arp);
 
@@ -3058,8 +3097,7 @@ static void callEncoder (DOContext *ctxt)
   if (cacheCoders == YES && _cachedDecoders != nil
     && (count = [_cachedDecoders count]) > 0)
     {
-      coder = [_cachedDecoders objectAtIndex: --count];
-      RETAIN(coder);
+      coder = RETAIN([_cachedDecoders objectAtIndex: --count]);
       [_cachedDecoders removeObjectAtIndex: count];
     }
   else
@@ -3112,8 +3150,7 @@ static void callEncoder (DOContext *ctxt)
   if (cacheCoders == YES && _cachedEncoders != nil
     && (count = [_cachedEncoders count]) > 0)
     {
-      coder = [_cachedEncoders objectAtIndex: --count];
-      RETAIN(coder);
+      coder = RETAIN([_cachedEncoders objectAtIndex: --count]);
       [_cachedEncoders removeObjectAtIndex: count];
     }
   else
@@ -3262,7 +3299,7 @@ static void callEncoder (DOContext *ctxt)
   node = GSIMapNodeForKey(_localTargets, (GSIMapKey)target);
   NSAssert(node == 0, NSInternalInconsistencyException);
 
-  RETAIN(anObj);
+  IF_NO_GC([anObj retain];)
   GSIMapAddPair(_localObjects, (GSIMapKey)object, (GSIMapVal)((id)anObj));
   GSIMapAddPair(_localTargets, (GSIMapKey)target, (GSIMapVal)((id)anObj));
 
@@ -3288,8 +3325,7 @@ static void callEncoder (DOContext *ctxt)
     }
   else
     {
-      p = node->value.obj;
-      RETAIN(p);
+      p = RETAIN(node->value.obj);
       DESTROY(proxy);
     }
   if (p == nil && proxy != nil)
@@ -3642,8 +3678,7 @@ static void callEncoder (DOContext *ctxt)
     }
   else
     {
-      p = node->value.obj;
-      RETAIN(p);
+      p = RETAIN(node->value.obj);
       DESTROY(aProxy);
     }
   if (p == nil && aProxy != nil)
