@@ -22,14 +22,14 @@
    Boston, MA 02111 USA.
 
    <title>NSCopyObject class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
    */
 
 #include "config.h"
 #include "GNUstepBase/preface.h"
 #include "Foundation/NSObject.h"
 
-NSObject *NSCopyObject(NSObject *anObject, unsigned extraBytes, NSZone *zone)
+NSObject *NSCopyObject(NSObject *anObject, NSUInteger extraBytes, NSZone *zone)
 {
   id copy = NSAllocateObject(((id)anObject)->class_pointer, extraBytes, zone);
   memcpy(copy, anObject,
