@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDecimalNumber class reference</title>
-   $Date: 2008-11-17 13:45:32 +0000 (Mon, 17 Nov 2008) $ $Revision: 27080 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
    */
 
 #define _GNU_SOURCE
@@ -354,8 +354,16 @@ static NSDecimalNumber *one;
 	  {
 	    NSString *s;
 	    float v = *(float *)value;
-	    if (GSIsNAN(v)) return notANumber;
-	    if (GSIsInf(v)) return (v < 0.0) ? minNumber : maxNumber;
+	    if (GSIsNAN(v))
+	      {
+		DESTROY(self);
+		return RETAIN(notANumber);
+	      }
+	    if (GSIsInf(v))
+	      {
+		DESTROY(self);
+		return (v < 0.0) ? RETAIN(minNumber) : RETAIN(maxNumber);
+	      }
 	    s = [[NSString alloc] initWithFormat: @"%g"
 				  locale: GSPrivateDefaultLocale(), (double)v];
 	    self = [self initWithString: s];
@@ -368,8 +376,16 @@ static NSDecimalNumber *one;
 	  {
 	    NSString *s;
 	    double v = *(double *)value;
-	    if (GSIsNAN(v)) return notANumber;
-	    if (GSIsInf(v)) return (v < 0.0) ? minNumber : maxNumber;
+	    if (GSIsNAN(v))
+	      {
+		DESTROY(self);
+		return RETAIN(notANumber);
+	      }
+	    if (GSIsInf(v))
+	      {
+		DESTROY(self);
+		return (v < 0.0) ? RETAIN(minNumber) : RETAIN(maxNumber);
+	      }
 	    s = [[NSString alloc] initWithFormat: @"%g"
 				  locale: GSPrivateDefaultLocale(), v];
 	    self = [self initWithString: s];
@@ -460,7 +476,7 @@ static NSDecimalNumber *one;
   return [self initWithBytes: &d objCType: "d"];
 }
 
-- (id) initWithInt: (signed int)value
+- (id) initWithInt: (int)value
 {
   if (value < 0)
     {
@@ -531,7 +547,7 @@ static NSDecimalNumber *one;
 		     isNegative: NO];
 }
 
-- (id) initWithUnsignedInt: (unsigned int)value
+- (id) initWithUnsignedInt: (NSUInteger)value
 {
   return [self initWithMantissa: value
 		       exponent: 0
@@ -753,13 +769,13 @@ static NSDecimalNumber *one;
   return [NSDecimalNumber decimalNumberWithDecimal: result];
 }
 
-- (NSDecimalNumber*) decimalNumberByRaisingToPower: (unsigned)power
+- (NSDecimalNumber*) decimalNumberByRaisingToPower: (NSUInteger)power
 {
   return [self decimalNumberByRaisingToPower: power
 				withBehavior: [isa defaultBehavior]];
 }
 
-- (NSDecimalNumber*) decimalNumberByRaisingToPower: (unsigned)power
+- (NSDecimalNumber*) decimalNumberByRaisingToPower: (NSUInteger)power
   withBehavior: (id <NSDecimalNumberBehaviors>)behavior
 {
   NSDecimal result;
@@ -851,8 +867,8 @@ static NSDecimalNumber *one;
   double num;
   NSDecimalNumber *dnum;
   num = [self doubleValue];
-  dnum = [[NSDecimalNumber alloc] initWithBytes: &num objCType: "d"];
-  AUTORELEASE(dnum);
+  dnum
+    = AUTORELEASE([[NSDecimalNumber alloc] initWithBytes: &num objCType: "d"]);
   return [dnum decimalValue];
 }
 @end
