@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDictionary class reference</title>
-   $Date: 2008-12-04 18:04:00 +0000 (Thu, 04 Dec 2008) $ $Revision: 27209 $
+   $Date: 2009-04-28 14:00:59 -0600 (Tue, 28 Apr 2009) $ $Revision: 28262 $
    */
 
 #include "config.h"
@@ -47,13 +47,9 @@
 
 static BOOL GSMacOSXCompatiblePropertyLists(void)
 {
-#if	defined(HAVE_LIBXML)
   if (GSPrivateDefaultsFlag(NSWriteOldStylePropertyLists) == YES)
     return NO;
   return GSPrivateDefaultsFlag(GSMacOSXCompatible);
-#else
-  return NO;
-#endif
 }
 
 @class	GSDictionary;
@@ -174,7 +170,7 @@ static SEL	appSel;
  */
 - (id) initWithObjects: (id*)objects
 	       forKeys: (id*)keys
-		 count: (unsigned)count
+		 count: (NSUInteger)count
 {
   self = [self init];
   return self;
@@ -184,7 +180,7 @@ static SEL	appSel;
  * Returns an unsigned integer which is the number of elements
  * stored in the dictionary.
  */
-- (unsigned) count
+- (NSUInteger) count
 {
   [self subclassResponsibility: _cmd];
   return 0;
@@ -405,13 +401,13 @@ static SEL	appSel;
  */
 + (id) dictionaryWithObjects: (id*)objects
 		     forKeys: (id*)keys
-		       count: (unsigned)count
+		       count: (NSUInteger)count
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithObjects: objects forKeys: keys count: count]);
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   return [self count];
 }
@@ -877,7 +873,7 @@ compareIt(id o1, id o2, void* context)
   struct foo	*f = (struct foo*)context;
   o1 = (*f->i)(f->d, @selector(objectForKey:), o1);
   o2 = (*f->i)(f->d, @selector(objectForKey:), o2);
-  return (int)(intptr_t)[o1 performSelector: f->s withObject: o2];
+  return (NSInteger)(intptr_t)[o1 performSelector: f->s withObject: o2];
 }
 
 /**
@@ -1081,7 +1077,7 @@ compareIt(id o1, id o2, void* context)
  * items are listed is undefined.
  */
 - (NSString*) descriptionWithLocale: (NSDictionary*)locale
-			     indent: (unsigned int)level
+			     indent: (NSUInteger)level
 {
   NSMutableString	*result = nil;
 
@@ -1185,7 +1181,7 @@ compareIt(id o1, id o2, void* context)
  * and needs to be re-implemented in subclasses in order to have all
  * other initialisers work.
  */
-- (id) initWithCapacity: (unsigned)numItems
+- (id) initWithCapacity: (NSUInteger)numItems
 {
   self = [self init];
   return self;
@@ -1218,7 +1214,7 @@ compareIt(id o1, id o2, void* context)
  *  added, this can avoid the reallocate-and-copy process if the size of the
  *  ultimate contents is known in advance.
  */
-+ (id) dictionaryWithCapacity: (unsigned)numItems
++ (id) dictionaryWithCapacity: (NSUInteger)numItems
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
     initWithCapacity: numItems]);
@@ -1233,7 +1229,7 @@ compareIt(id o1, id o2, void* context)
  */
 - (id) initWithObjects: (id*)objects
 	       forKeys: (id*)keys
-		 count: (unsigned)count
+		 count: (NSUInteger)count
 {
   self = [self initWithCapacity: count];
   if (self != nil)
