@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistantObject class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-01-12 11:36:37 -0700 (Mon, 12 Jan 2009) $ $Revision: 27581 $
    */
 
 #include "config.h"
@@ -40,11 +40,11 @@
 #include <objc/Protocol.h>
 
 
-@interface NSDistantObject(GNUstepExtensions) <GCFinalization>
+@interface NSDistantObject(GNUstepExtensions)
 - (Class) classForPortCoder;
 - (const char *) selectorTypeForProxy: (SEL)selector;
 - (id) forward: (SEL)aSel :(arglist_t)frame;
-- (void) gcFinalize;
+- (void) finalize;
 @end
 
 #define DO_FORWARD_INVOCATION(_SELX, _ARG1) ({			\
@@ -446,7 +446,7 @@ enum proxyLocation
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -674,8 +674,7 @@ enum proxyLocation
 
 	  if (sig == nil)
 	    {
-	      sig = [NSMethodSignature signatureWithObjCTypes: "@@::"];
-	      RETAIN(sig);
+	      sig = RETAIN([NSMethodSignature signatureWithObjCTypes: "@@::"]);
 	    }
 	  return sig;
 	}
@@ -690,8 +689,7 @@ enum proxyLocation
 
 	  if (sig == nil)
 	    {
-	      sig = [NSMethodSignature signatureWithObjCTypes: "r*@:"];
-	      RETAIN(sig);
+	      sig = RETAIN([NSMethodSignature signatureWithObjCTypes: "r*@:"]);
 	    }
 	  return sig;
 	}
@@ -827,7 +825,7 @@ enum proxyLocation
 /**
  * Used by the garbage collection system to tidy up when a proxy is destroyed.
  */
-- (void) gcFinalize
+- (void) finalize
 {
   if (_connection)
     {
