@@ -1,5 +1,5 @@
 /**
-   Copyright (C) 1998-2003 Free Software Foundation, Inc.
+   Copyright (C) 1998-2009 Free Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Created: October 1998
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistributedNotificationCenter class reference</title>
-   $Date: 2008-12-07 06:02:02 +0000 (Sun, 07 Dec 2008) $ $Revision: 27240 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
    */
 
 #import	"config.h"
@@ -55,7 +55,7 @@
 		       object: (NSString*)object
 		     userInfo: (NSData*)info
 		     selector: (NSString*)aSelector
-			   to: (NSUInteger)observer;
+			   to: (uint64_t)observer;
 @end
 
 /**
@@ -339,12 +339,12 @@ static NSDistributedNotificationCenter	*netCenter = nil;
   NS_DURING
     {
       [self _connect];
-      [(id<GDNCProtocol>)_remote addObserver: (NSUInteger)anObserver
-				   selector: NSStringFromSelector(aSelector)
-				       name: notificationName
-				     object: anObject
-			 suspensionBehavior: suspensionBehavior
-					for: (id<GDNCClient>)self];
+      [(id<GDNCProtocol>)_remote addObserver: (uint64_t)anObserver
+				    selector: NSStringFromSelector(aSelector)
+				        name: notificationName
+				      object: anObject
+			  suspensionBehavior: suspensionBehavior
+					 for: (id<GDNCClient>)self];
     }
   NS_HANDLER
     {
@@ -465,10 +465,10 @@ static NSDistributedNotificationCenter	*netCenter = nil;
   NS_DURING
     {
       [self _connect];
-      [(id<GDNCProtocol>)_remote removeObserver: (NSUInteger)anObserver
-					  name: notificationName
-					object: anObject
-					   for: (id<GDNCClient>)self];
+      [(id<GDNCProtocol>)_remote removeObserver: (uint64_t)anObserver
+					   name: notificationName
+					 object: anObject
+					    for: (id<GDNCClient>)self];
     }
   NS_HANDLER
     {
@@ -519,7 +519,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
  * in the source where the '@protocol()' directive is used.
  */
 @interface NSDistributedNotificationCenterDummy : NSObject <GDNCProtocol>
-- (void) addObserver: (NSUInteger)anObserver
+- (void) addObserver: (uint64_t)anObserver
 	    selector: (NSString*)aSelector
 	        name: (NSString*)notificationname
 	      object: (NSString*)anObject
@@ -531,7 +531,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		  deliverImmediately: (BOOL)deliverImmediately
 			         for: (id<GDNCClient>)client;
 - (void) registerClient: (id<GDNCClient>)client;
-- (void) removeObserver: (NSUInteger)anObserver
+- (void) removeObserver: (uint64_t)anObserver
 		   name: (NSString*)notificationname
 		 object: (NSString*)anObject
 		    for: (id<GDNCClient>)client;
@@ -541,7 +541,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 @end
 
 @implementation NSDistributedNotificationCenterDummy
-- (void) addObserver: (NSUInteger)anObserver
+- (void) addObserver: (uint64_t)anObserver
 	    selector: (NSString*)aSelector
 	        name: (NSString*)notificationname
 	      object: (NSString*)anObject
@@ -559,7 +559,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 - (void) registerClient: (id<GDNCClient>)client
 {
 }
-- (void) removeObserver: (NSUInteger)anObserver
+- (void) removeObserver: (uint64_t)anObserver
 		   name: (NSString*)notificationname
 		 object: (NSString*)anObject
 		    for: (id<GDNCClient>)client
@@ -744,8 +744,8 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 	      _remote = [NSConnection
 		rootProxyForConnectionWithRegisteredName: service
 		host: host usingNameServer: ns];
-              RETAIN(_remote);
-              DESTROY(pool);
+              IF_NO_GC([_remote retain];)
+              IF_NO_GC(DESTROY(pool);)
 	    }
 	  if (_remote == nil)
 	    {
@@ -755,10 +755,12 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		@"I attempted to start it at '%@'\n", cmd];
 	    }
 	}
+#if	!GS_WITH_GC
       else
         {
-          RETAIN(_remote);
+          [_remote retain];
         }
+#endif
 
       c = [_remote connectionForProxy];
       [_remote setProtocolForProxy: p];
@@ -802,7 +804,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
 		       object: (NSString*)object
 		     userInfo: (NSData*)info
 		     selector: (NSString*)aSelector
-			   to: (NSUInteger)observer
+			   to: (uint64_t)observer
 {
   id			userInfo;
   NSNotification	*notification;
