@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSEnumerator class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-02-03 22:30:39 -0700 (Tue, 03 Feb 2009) $ $Revision: 27774 $
 */
 
 #import "config.h"
@@ -74,4 +74,25 @@
   return nil;
 }
 
+- (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state 	
+				   objects: (id*)stackbuf
+				     count: (NSUInteger)len
+{
+  IMP nextObject = [self methodForSelector: @selector(nextObject)];
+  int i;
+
+  state->itemsPtr = stackbuf;
+  state->mutationsPtr = (unsigned long*)self;
+  for (i = 0; i < len; i++)
+    {
+      id next = nextObject(self, @selector(nextObject));
+
+      if (nil == next)
+	{
+	  return i;
+	}
+      *(stackbuf+i) = next;
+    }
+  return len;
+}
 @end
