@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2008-11-28 15:38:48 +0000 (Fri, 28 Nov 2008) $ $Revision: 27168 $
+   $Date: 2009-02-23 13:51:20 -0700 (Mon, 23 Feb 2009) $ $Revision: 27963 $
 */
 
 #import "config.h"
@@ -42,8 +42,6 @@
 #include <stdio.h>
 
 
-static  NSUncaughtExceptionHandler *_NSUncaughtExceptionHandler;
-
 #define _e_info (((id*)_reserved)[0])
 #define _e_stack (((id*)_reserved)[1])
 
@@ -60,8 +58,8 @@ typedef struct { @defs(NSThread) } *TInfo;
 - (NSString*) description;
 - (NSEnumerator*) enumerator;
 - (NSMutableArray*) frames;
-- (id) frameAt: (unsigned)index;
-- (unsigned) frameCount;
+- (id) frameAt: (NSUInteger)index;
+- (NSUInteger) frameCount;
 - (id) initWithAddresses: (NSArray*)stack;
 - (NSEnumerator*) reverseEnumerator;
 
@@ -152,8 +150,8 @@ GSPrivateBaseAddress(void *addr, void **base)
 	      address: (void*)address 
 		 file: (NSString*)file 
 	     function: (NSString*)function 
-		 line: (int)lineNo;
-- (int) lineNumber;
+		 line: (NSInteger)lineNo;
+- (NSInteger) lineNumber;
 - (GSBinaryFileInfo*) module;
 
 @end
@@ -216,7 +214,7 @@ GSPrivateBaseAddress(void *addr, void **base)
 	      address: (void*)address 
 		 file: (NSString*)file 
 	     function: (NSString*)function 
-		 line: (int)lineNo
+		 line: (NSInteger)lineNo
 {
   _module = RETAIN(module);
   _address = address;
@@ -227,7 +225,7 @@ GSPrivateBaseAddress(void *addr, void **base)
   return self;
 }
 
-- (int) lineNumber
+- (NSInteger) lineNumber
 {
   return _lineNo;
 }
@@ -552,12 +550,12 @@ GSListModules()
   return [frames objectEnumerator];
 }
 
-- (id) frameAt: (unsigned)index
+- (id) frameAt: (NSUInteger)index
 {
   return [frames objectAtIndex: index];
 }
 
-- (unsigned) frameCount
+- (NSUInteger) frameCount
 {
   return [frames count];
 }
@@ -724,6 +722,10 @@ _NSFoundationUncaughtExceptionHandler (NSException *exception)
   _terminate();
 }
 
+static  NSUncaughtExceptionHandler *_NSUncaughtExceptionHandler
+  = _NSFoundationUncaughtExceptionHandler;
+
+#if	!defined(_NATIVE_OBJC_EXCEPTIONS) || defined(HAVE_UNEXPECTED)
 static void
 callUncaughtHandler(id value)
 {
@@ -733,6 +735,8 @@ callUncaughtHandler(id value)
     }
   _NSFoundationUncaughtExceptionHandler(value);
 }
+#endif
+
 
 @implementation NSException
 
@@ -834,7 +838,7 @@ callUncaughtHandler(id value)
     }
 
   if (_e_stack == nil
-    && GSPrivateEnvironmentFlag("GNUSTEP_STACK_TRACE", YES) == NO)
+    && GSPrivateEnvironmentFlag("GNUSTEP_STACK_TRACE", YES) == YES)
     {
       ASSIGN(_e_stack, GSPrivateStackAddresses());
     }
@@ -1008,8 +1012,8 @@ callUncaughtHandler(id value)
       result = [NSString stringWithFormat: @"%@ NAME:%@ REASON:%@",
         [super description], _e_name, _e_reason];
     }
-  RETAIN(result);
-  DESTROY(pool);
+  IF_NO_GC([result retain];)
+  IF_NO_GC(DESTROY(pool);)
   return AUTORELEASE(result);
 }
 
