@@ -34,7 +34,7 @@
    Boston, MA 02111 USA.
 
    <title>NSFileManager class reference</title>
-   $Date: 2008-10-27 14:47:39 +0000 (Mon, 27 Oct 2008) $ $Revision: 26976 $
+   $Date: 2009-04-27 05:52:44 -0600 (Mon, 27 Apr 2009) $ $Revision: 28253 $
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -238,6 +238,7 @@
  */
 @interface	GSAttrDictionary : NSDictionary
 {
+@public
   struct _STATB	statbuf;
   _CHAR		_path[0];
 }
@@ -245,6 +246,8 @@
 		  traverseLink: (BOOL)traverse;
 @end
 
+static Class	GSAttrDictionaryClass = 0;
+
 /*
  * We also need a special enumerator class to enumerate the dictionary.
  */
@@ -342,6 +345,7 @@ static NSStringEncoding	defaultEncoding;
 + (void) initialize
 {
   defaultEncoding = [NSString defaultCStringEncoding];
+  GSAttrDictionaryClass = [GSAttrDictionary class];
 }
 
 - (void) dealloc
@@ -388,6 +392,7 @@ static NSStringEncoding	defaultEncoding;
 - (BOOL) changeFileAttributes: (NSDictionary*)attributes atPath: (NSString*)path
 {
   const _CHAR	*lpath = 0;
+  NSNumber	*tmpNum;
   unsigned long	num;
   NSString	*str;
   NSDate	*date;
@@ -400,7 +405,15 @@ static NSStringEncoding	defaultEncoding;
   lpath = [defaultManager fileSystemRepresentationWithPath: path];
 
 #ifndef __MINGW32__
-  num = [attributes fileOwnerAccountID];
+  if (GSObjCClass(attributes) == GSAttrDictionaryClass)
+    {
+      num = ((GSAttrDictionary*)attributes)->statbuf.st_uid;
+    }
+  else
+    {
+      tmpNum = [attributes fileOwnerAccountID];
+      num = tmpNum ? [tmpNum unsignedLongValue] : NSNotFound;
+    }
   if (num != NSNotFound)
     {
       if (chown(lpath, num, -1) != 0)
@@ -455,7 +468,15 @@ static NSStringEncoding	defaultEncoding;
 	}
     }
 
-  num = [attributes fileGroupOwnerAccountID];
+  if (GSObjCClass(attributes) == GSAttrDictionaryClass)
+    {
+      num = ((GSAttrDictionary*)attributes)->statbuf.st_gid;
+    }
+  else
+    {
+      tmpNum = [attributes fileGroupOwnerAccountID];
+      num = tmpNum ? [tmpNum unsignedLongValue] : NSNotFound;
+    }
   if (num != NSNotFound)
     {
       if (chown(lpath, -1, num) != 0)
@@ -769,7 +790,7 @@ static NSStringEncoding	defaultEncoding;
    * If there is no file owner specified, and we are running setuid to
    * root, then we assume we need to change ownership to correct user.
    */
-  if (attributes == nil || ([attributes fileOwnerAccountID] == NSNotFound
+  if (attributes == nil || ([attributes fileOwnerAccountID] == nil
     && [attributes fileOwnerAccountName] == nil))
     {
       if (geteuid() == 0 && [@"root" isEqualToString: NSUserName()] == NO)
@@ -932,7 +953,7 @@ static NSStringEncoding	defaultEncoding;
    * If there is no file owner specified, and we are running setuid to
    * root, then we assume we need to change ownership to correct user.
    */
-  if (attributes == nil || ([attributes fileOwnerAccountID] == NSNotFound
+  if (attributes == nil || ([attributes fileOwnerAccountID] == nil
     && [attributes fileOwnerAccountName] == nil))
     {
       if (geteuid() == 0 && [@"root" isEqualToString: NSUserName()] == NO)
@@ -1721,7 +1742,7 @@ static NSStringEncoding	defaultEncoding;
 {
   NSDictionary	*d;
 
-  d = [GSAttrDictionary attributesAt:
+  d = [GSAttrDictionaryClass attributesAt:
     [self fileSystemRepresentationWithPath: path] traverseLink: flag];
   return d;
 }
@@ -2015,7 +2036,7 @@ static NSStringEncoding	defaultEncoding;
     (const GSNativeChar*)[path cStringUsingEncoding: defaultEncoding];
 }
 - (NSString*) stringWithFileSystemRepresentation: (const GSNativeChar*)string
-					  length: (unsigned int)len
+					  length: (NSUInteger)len
 {
   return AUTORELEASE([[NSString allocWithZone: NSDefaultMallocZone()]
     initWithBytes: string length: len encoding: defaultEncoding]);
@@ -2103,7 +2124,11 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
   self = [super init];
 
   _mgr = RETAIN(mgr);
+#if	GS_WITH_GC
+  _stack = NSAllocateCollectable(sizeof(GSIArray_t), NSScannedOption);
+#else
   _stack = NSZoneMalloc([self zone], sizeof(GSIArray_t));
+#endif
   GSIArrayInitWithZoneAndCapacity(_stack, [self zone], 64);
 
   _flags.isRecursive = recurse;
@@ -2235,9 +2260,8 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	    stringWithFileSystemRepresentation: dirbuf->d_name
 	    length: strlen(dirbuf->d_name)];
 #endif
-	  returnFileName = [dir.path stringByAppendingPathComponent:
-	    returnFileName];
-	  RETAIN(returnFileName);
+	  returnFileName = RETAIN([dir.path stringByAppendingPathComponent:
+	    returnFileName]);
 
 	  /* TODO - can this one can be removed ? */
 	  if (!_flags.justContents)
@@ -2337,17 +2361,17 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 /**
  *  Returns HFS creator attribute (OS X).
  */
-- (int) fileHFSCreatorCode
+- (OSType) fileHFSCreatorCode
 {
-  return [[self objectForKey: NSFileHFSCreatorCode] intValue];
+  return [[self objectForKey: NSFileHFSCreatorCode] unsignedLongValue];
 }
 
 /**
  *  Returns HFS type code attribute (OS X).
  */
-- (int) fileHFSTypeCode
+- (OSType) fileHFSTypeCode
 {
-  return [[self objectForKey: NSFileHFSTypeCode] intValue];
+  return [[self objectForKey: NSFileHFSTypeCode] unsignedLongValue];
 }
 
 /**
@@ -2398,18 +2422,12 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 }
 
 /**
- * Return the numeric value of the NSFileOwnerAccountID attribute
- * in the dictionary, or NSNotFound if the attribute is not present.
+ * Return an NSNumber with the numeric value of the NSFileOwnerAccountID attribute
+ * in the dictionary, or nil if the attribute is not present.
  */
-- (unsigned long) fileOwnerAccountID
+- (NSNumber*) fileOwnerAccountID
 {
-  NSNumber	*n = [self objectForKey: NSFileOwnerAccountID];
-
-  if (n == nil)
-    {
-      return NSNotFound;
-    }
-  return [n unsignedIntValue];
+  return [self objectForKey: NSFileOwnerAccountID];
 }
 
 /**
@@ -2421,18 +2439,12 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 }
 
 /**
- * Return the numeric value of the NSFileGroupOwnerAccountID attribute
- * in the dictionary, or NSNotFound if the attribute is not present.
+ * Return an NSNumber with the numeric value of the NSFileGroupOwnerAccountID attribute
+ * in the dictionary, or nil if the attribute is not present.
  */
-- (unsigned long) fileGroupOwnerAccountID
+- (NSNumber*) fileGroupOwnerAccountID
 {
-  NSNumber	*n = [self objectForKey: NSFileGroupOwnerAccountID];
-
-  if (n == nil)
-    {
-      return NSNotFound;
-    }
-  return [n unsignedIntValue];
+  return [self objectForKey: NSFileGroupOwnerAccountID];
 }
 
 /**
@@ -2447,7 +2459,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
  * Return the file posix permissions attribute (or NSNotFound if
  * the attribute is not present in the dictionary).
  */
-- (unsigned long) filePosixPermissions
+- (NSUInteger) filePosixPermissions
 {
   NSNumber	*n = [self objectForKey: NSFilePosixPermissions];
 
@@ -2455,14 +2467,14 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
     {
       return NSNotFound;
     }
-  return [n unsignedLongValue];
+  return [n unsignedIntegerValue];
 }
 
 /**
  * Return the file system number attribute (or NSNotFound if
  * the attribute is not present in the dictionary).
  */
-- (unsigned long) fileSystemNumber
+- (NSUInteger) fileSystemNumber
 {
   NSNumber	*n = [self objectForKey: NSFileSystemNumber];
 
@@ -2470,14 +2482,14 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
     {
       return NSNotFound;
     }
-  return [n unsignedLongValue];
+  return [n unsignedIntegerValue];
 }
 
 /**
  * Return the file system file identification number attribute
  * or NSNotFound if the attribute is not present in the dictionary).
  */
-- (unsigned long) fileSystemFileNumber
+- (NSUInteger) fileSystemFileNumber
 {
   NSNumber	*n = [self objectForKey: NSFileSystemFileNumber];
 
@@ -2485,7 +2497,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
     {
       return NSNotFound;
     }
-  return [n unsignedLongValue];
+  return [n unsignedIntegerValue];
 }
 @end
 
@@ -2905,7 +2917,7 @@ static NSSet	*fileKeys = nil;
 	NSFileSystemNumber,
 	NSFileType,
 	nil];
-      RETAIN(fileKeys);
+      IF_NO_GC([fileKeys retain];)
     }
 }
 
@@ -2931,9 +2943,9 @@ static NSSet	*fileKeys = nil;
   return NO;
 }
 
-- (unsigned long) fileGroupOwnerAccountID
+- (NSNumber*) fileGroupOwnerAccountID
 {
-  return statbuf.st_gid;
+  return [NSNumber numberWithInt: statbuf.st_gid];
 }
 
 - (NSString*) fileGroupOwnerAccountName
@@ -3058,12 +3070,12 @@ static NSSet	*fileKeys = nil;
   return group;
 }
 
-- (int) fileHFSCreatorCode
+- (OSType) fileHFSCreatorCode
 {
   return 0;
 }
 
-- (int) fileHFSTypeCode
+- (OSType) fileHFSTypeCode
 {
   return 0;
 }
@@ -3083,14 +3095,14 @@ static NSSet	*fileKeys = nil;
   return [NSDate dateWithTimeIntervalSince1970: statbuf.st_mtime];
 }
 
-- (unsigned long) filePosixPermissions
+- (NSUInteger) filePosixPermissions
 {
   return (statbuf.st_mode & ~S_IFMT);
 }
 
-- (unsigned long) fileOwnerAccountID
+- (NSNumber*) fileOwnerAccountID
 {
-  return statbuf.st_uid;
+  return [NSNumber numberWithInt: statbuf.st_uid];
 }
 
 - (NSString*) fileOwnerAccountName
@@ -3220,12 +3232,12 @@ static NSSet	*fileKeys = nil;
   return statbuf.st_size;
 }
 
-- (unsigned long) fileSystemFileNumber
+- (NSUInteger) fileSystemFileNumber
 {
   return statbuf.st_ino;
 }
 
-- (unsigned long) fileSystemNumber
+- (NSUInteger) fileSystemNumber
 {
   return statbuf.st_dev;
 }
@@ -3276,11 +3288,11 @@ static NSSet	*fileKeys = nil;
       if (key == NSFileGroupOwnerAccountName)
 	return [self fileGroupOwnerAccountName];
       if (key == NSFileGroupOwnerAccountID)
-	return [NSNumber numberWithInt: [self fileGroupOwnerAccountID]];
+	return [self fileGroupOwnerAccountID];
       if (key == NSFileHFSCreatorCode)
-	return [NSNumber numberWithInt: [self fileHFSCreatorCode]];
+	return [NSNumber numberWithUnsignedLong: [self fileHFSCreatorCode]];
       if (key == NSFileHFSTypeCode)
-	return [NSNumber numberWithInt: [self fileHFSTypeCode]];
+	return [NSNumber numberWithUnsignedLong: [self fileHFSTypeCode]];
       if (key == NSFileImmutable)
 	return [NSNumber numberWithBool: [self fileIsImmutable]];
       if (key == NSFileModificationDate)
@@ -3288,7 +3300,7 @@ static NSSet	*fileKeys = nil;
       if (key == NSFileOwnerAccountName)
 	return [self fileOwnerAccountName];
       if (key == NSFileOwnerAccountID)
-	return [NSNumber numberWithInt: [self fileOwnerAccountID]];
+	return [self fileOwnerAccountID];
       if (key == NSFilePosixPermissions)
 	return [NSNumber numberWithUnsignedInt: [self filePosixPermissions]];
       if (key == NSFileReferenceCount)
