@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSHost class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-01-12 05:48:46 -0700 (Mon, 12 Jan 2009) $ $Revision: 27578 $
   */
 
 #import "config.h"
@@ -335,7 +335,7 @@ myHostName()
 	   * with ALL the IP addresses of any interfaces on the local machine
 	   */
 	  host = [[self alloc] _initWithHostEntry: 0 key: localHostName];
-	  AUTORELEASE(host);
+	  IF_NO_GC([host autorelease];)
 	}
       else
 	{
@@ -369,13 +369,13 @@ myHostName()
 	  else
 	    {
 	      host = [[self alloc] _initWithHostEntry: h key: name];
-	      AUTORELEASE(host);
+	      IF_NO_GC([host autorelease];)
 	    }
 	}
     }
   else
     {
-      AUTORELEASE(RETAIN(host));
+      IF_NO_GC([[host retain] autorelease];)
     }
   [_hostCacheLock unlock];
   if (tryByAddress == YES)
@@ -431,18 +431,18 @@ myHostName()
 	  if (badAddr == NO)
 	    {
 	      host = [[self alloc] _initWithAddress: address];
-	      AUTORELEASE(host);
+	      IF_NO_GC([host autorelease];)
 	    }
 	}
       else
 	{
 	  host = [[self alloc] _initWithHostEntry: h key: address];
-	  AUTORELEASE(host);
+	  IF_NO_GC([host autorelease];)
 	}
     }
   else
     {
-      AUTORELEASE(RETAIN(host));
+      IF_NO_GC([[host retain] autorelease];)
     }
   [_hostCacheLock unlock];
   return host;
@@ -507,7 +507,7 @@ myHostName()
     {
       host = [NSHost currentHost];
     }
-  RETAIN(host);
+  IF_NO_GC([host retain];)
   RELEASE(self);
   return host;
 }
