@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSInvocation class reference</title>
-   $Date: 2008-07-02 11:46:16 +0100 (Wed, 02 Jul 2008) $ $Revision: 26739 $
+   $Date: 2009-03-31 10:04:29 -0600 (Tue, 31 Mar 2009) $ $Revision: 28161 $
    */
 
 #include "Foundation/NSException.h"
@@ -62,7 +62,7 @@
 
 @implementation GSCodeBuffer
 
-+ (GSCodeBuffer*) memoryWithSize: (unsigned)_size
++ (GSCodeBuffer*) memoryWithSize: (NSUInteger)_size
 {
   return [[[self alloc] initWithSize: _size] autorelease];
 }
@@ -87,7 +87,7 @@
   [super dealloc];
 }
 
-- (id) initWithSize: (unsigned)_size
+- (id) initWithSize: (NSUInteger)_size
 {
 #if     defined(HAVE_MMAP)
 #if     defined(HAVE_MPROTECT)
@@ -365,9 +365,9 @@ _arg_addr(NSInvocation *inv, int index)
  * so the actual method arguments start at index 2.
  */
 - (void) getArgument: (void*)buffer
-	     atIndex: (int)index
+	     atIndex: (NSInteger)index
 {
-  if ((unsigned)index >= _numArgs)
+  if ((NSUInteger)index >= _numArgs)
     {
       [NSException raise: NSInvalidArgumentException
 		  format: @"bad invocation argument index"];
@@ -439,9 +439,9 @@ _arg_addr(NSInvocation *inv, int index)
  * receiver is retained by it.
  */
 - (void) setArgument: (void*)buffer
-	     atIndex: (int)index
+	     atIndex: (NSInteger)index
 {
-  if ((unsigned)index >= _numArgs)
+  if ((NSUInteger)index >= _numArgs)
     {
       [NSException raise: NSInvalidArgumentException
 		  format: @"bad invocation argument index"];
@@ -823,8 +823,7 @@ _arg_addr(NSInvocation *inv, int index)
   NSZoneFree(NSDefaultMallocZone(), (void*)types);
 
   RELEASE(self);
-  self  = [NSInvocation invocationWithMethodSignature: newSig];
-  RETAIN(self);
+  self  = RETAIN([NSInvocation invocationWithMethodSignature: newSig]);
 
   [aCoder decodeValueOfObjCType: @encode(id) at: &_target];
 
@@ -924,7 +923,8 @@ _arg_addr(NSInvocation *inv, int index)
   types = sel_get_type(aSelector);
   if (types == 0)
     {
-      types = sel_get_type(sel_get_any_typed_uid(GSNameFromSelector(aSelector)));
+      types
+        = sel_get_type(sel_get_any_typed_uid(GSNameFromSelector(aSelector)));
     }
   if (types == 0)
     {
@@ -937,113 +937,6 @@ _arg_addr(NSInvocation *inv, int index)
   return [self initWithMethodSignature: newSig];
 }
 
-/**
- * Initialises the receiver with the specified target, selector, and
- * a variable number of arguments.
- */
-- (id) initWithTarget: anObject selector: (SEL)aSelector, ...
-{
-  va_list	     ap;
-  NSMethodSignature *newSig;
-
-  if (anObject)
-    {
-      newSig = [anObject methodSignatureForSelector: aSelector];
-      self = [self initWithMethodSignature: newSig];
-    }
-  else
-    {
-      self = [self initWithSelector: aSelector];
-    }
-  if (self)
-    {
-      unsigned int	i;
-
-      [self setTarget: anObject];
-      [self setSelector: aSelector];
-      va_start (ap, aSelector);
-      for (i = 3; i <= _numArgs; i++)
-	{
-	  const char	*type = _info[i].type;
-	  unsigned	size = _info[i].size;
-	  void		*datum;
-
-	  datum = _arg_addr(self, i-1);
-
-#define CASE_TYPE(_C,_T) case _C: *(_T*)datum = va_arg (ap, _T); break
-	  switch (*type)
-	    {
-	      case _C_ID:
-		*(id*)datum = va_arg (ap, id);
-		if (_argsRetained)
-		  {
-		    IF_NO_GC(RETAIN(*(id*)datum));
-		  }
-		break;
-	      case _C_CHARPTR:
-		*(char**)datum = va_arg (ap, char*);
-		if (_argsRetained)
-		  {
-		    char	*old = *(char**)datum;
-
-		    if (old != 0)
-		      {
-			char	*tmp;
-
-			tmp = NSZoneMalloc(NSDefaultMallocZone(),strlen(old)+1);
-			strcpy(tmp, old);
-			*(char**)datum = tmp;
-		      }
-		  }
-		break;
-	      CASE_TYPE(_C_CLASS, Class);
-	      CASE_TYPE(_C_SEL, SEL);
-	      CASE_TYPE(_C_LNG, long);
-	      CASE_TYPE(_C_ULNG, unsigned long);
-	      CASE_TYPE(_C_INT, int);
-	      CASE_TYPE(_C_UINT, unsigned int);
-	      case _C_SHT:
-		*(short*)datum = (short)va_arg(ap, int);
-		break;
-	      case _C_USHT:
-		*(unsigned short*)datum = (unsigned short)va_arg(ap, int);
-		break;
-	      case _C_CHR:
-		*(char*)datum = (char)va_arg(ap, int);
-		break;
-	      case _C_UCHR:
-		*(unsigned char*)datum = (unsigned char)va_arg(ap, int);
-		break;
-	      case _C_FLT:
-		*(float*)datum = (float)va_arg(ap, double);
-		break;
-	      CASE_TYPE(_C_DBL, double);
-	      CASE_TYPE(_C_PTR, void*);
-	      case _C_STRUCT_B:
-	      default:
-#if !defined(USE_LIBFFI) && !defined(USE_FFCALL)
-#if defined(sparc) || defined(powerpc)
-		/* FIXME: This only appears on sparc and ppc machines so far.
-		structures appear to be aligned on word boundaries.
-		Hopefully there is a more general way to figure this out */
-		size = (size<sizeof(int))?4:size;
-#endif
-#endif
-	      NSLog(@"Unsafe handling of type of %d argument.", i-1);
-	      {
-		struct {
-		  char	x[size];
-		} dummy;
-		dummy = va_arg(ap, typeof(dummy));
-	        memcpy(datum, dummy.x, size);
-	      }
-	      break;
-	    }
-	}
-    }
-  return self;
-}
-
 /**
  * Internal use.<br />
  * Provides a return frame that the ObjectiveC runtime can use to
