@@ -21,7 +21,7 @@
    Boston, MA 02111 USA.
 
    <title>NSKeyValueCoding informal protocol reference</title>
-   $Date: 2008-12-01 18:38:58 +0000 (Mon, 01 Dec 2008) $ $Revision: 27183 $
+   $Date: 2009-02-27 05:54:15 -0700 (Fri, 27 Feb 2009) $ $Revision: 27995 $
    */
 
 #include "config.h"
@@ -467,9 +467,9 @@ static id ValueForKey(NSObject *self, const char *key, unsigned size)
 	  name[8] = toupper(name[8]);
 	}
       sel = GSSelectorFromName(name);
-      if (sel != 0
-	&& (imp = (BOOL (*)(id,SEL,id*,id*))[self methodForSelector: sel]) != 0)
+      if (sel != 0 && [self respondsToSelector: sel] == YES)
 	{
+	  imp = (BOOL (*)(id,SEL,id*,id*))[self methodForSelector: sel];
 	  return (*imp)(self, sel, aValue, anError);
 	}
     }
