@@ -21,11 +21,12 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-04-16 03:07:13 -0600 (Thu, 16 Apr 2009) $ $Revision: 28223 $
 */
 
 #include "GNUstepBase/preface.h"
 #import "Foundation/NSCharacterSet.h"
+#import "Foundation/NSDebug.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSEnumerator.h"
 #import "Foundation/NSException.h"
@@ -137,7 +138,7 @@ static inline void setup()
 @interface	GSKVOObservation : NSObject
 {
 @public
-  NSObject      *observer;      // Not retained
+  NSObject      *observer;      // Not retained (zeroing weak pointer)
   void          *context;
   int           options;
 }
@@ -551,16 +552,16 @@ replacementForClass(Class c)
         }
       else
         {
-          NSMapTable depKeys = NSMapGet(dependentKeyTable, original);
+          NSMapTable *depKeys = NSMapGet(dependentKeyTable, original);
 
           if (depKeys)
             {
               NSMapEnumerator enumerator = NSEnumerateMapTable(depKeys);
               NSString *mainKey;
-              NSHashTable dependents;
+              NSHashTable *dependents;
 
               while (NSNextMapEnumeratorPair(&enumerator, (void **)(&mainKey),
-                &dependents))
+                (void**)&dependents))
                 {
                   NSHashEnumerator dependentKeyEnum;
                   NSString *dependentKey;
@@ -585,7 +586,8 @@ replacementForClass(Class c)
 
           if (!found)
             {
-              NSLog(@"class %@ not KVC complient for %@", original, aKey);
+              NSDebugLLog(@"KVC", @"class %@ not KVC complient for %@",
+		original, aKey);
               /*
               [NSException raise: NSInvalidArgumentException
                            format: @"class not KVC complient for %@", aKey];
@@ -803,6 +805,16 @@ replacementForClass(Class c)
 
 
 @implementation	GSKVOObservation
+#if	GS_WITH_GC
++ (void) initialize
+{
+  GSMakeWeakPointer(self, "observer");
+}
+- (void) finalize
+{
+  GSAssignZeroingWeakPointer((void**)&observer, nil);
+}
+#endif
 @end
 
 @implementation	GSKVOPathInfo
@@ -955,17 +967,26 @@ replacementForClass(Class c)
       o = [pathInfo->observations objectAtIndex: count];
       if (o->observer == anObserver)
         {
-          o->observer = anObserver;
           o->context = aContext;
           o->options = options;
           observation = o;
         }
+#if	GS_WITH_GC
+      else if (o->observer == nil)
+	{
+	  /* The observer for thsi observation must have been collected.
+	   */
+	  [pathInfo->observations removeObjectAtIndex: count];
+	  continue;
+	}
+#endif
       pathInfo->allOptions |= o->options;
     }
   if (observation == nil)
     {
       observation = [GSKVOObservation new];
-      observation->observer = anObserver;
+      GSAssignZeroingWeakPointer((void**)&observation->observer,
+	(void*)anObserver);
       observation->context = aContext;
       observation->options = options;
       [pathInfo->observations addObject: observation];
@@ -1049,7 +1070,7 @@ replacementForClass(Class c)
           GSKVOObservation      *o;
 
           o = [pathInfo->observations objectAtIndex: count];
-          if (o->observer == anObserver)
+          if (o->observer == anObserver || o->observer == nil)
             {
               [pathInfo->observations removeObjectAtIndex: count];
               if ([pathInfo->observations count] == 0)
@@ -1087,6 +1108,14 @@ replacementForClass(Class c)
               context = o->context;
               break;
             }
+#if	GS_WITH_GC
+	  else if (o->observer == nil)
+	    {
+	      /* The observer for thsi observation must have been collected.
+	       */
+	      [pathInfo->observations removeObjectAtIndex: count];
+	    }
+#endif
 	}
     }
   [iLock unlock];
@@ -1365,7 +1394,7 @@ replacementForClass(Class c)
        * turn off key-value-observing for it.
        */
       isa = [self class];
-      AUTORELEASE(info);
+      IF_NO_GC(AUTORELEASE(info);)
       [self setObservationInfo: nil];
     }
   [kvoLock unlock];
@@ -1445,11 +1474,11 @@ replacementForClass(Class c)
 
 - (void) willChangeValueForDependentsOfKey: (NSString *)aKey
 {
-  NSMapTable keys = NSMapGet(dependentKeyTable, [self class]);
+  NSMapTable *keys = NSMapGet(dependentKeyTable, [self class]);
 
   if (keys != nil)
     {
-      NSHashTable       dependents = NSMapGet(keys, aKey);
+      NSHashTable       *dependents = NSMapGet(keys, aKey);
 
       if (dependents != 0)
         {
@@ -1468,11 +1497,11 @@ replacementForClass(Class c)
 
 - (void) didChangeValueForDependentsOfKey: (NSString *)aKey
 {
-  NSMapTable keys = NSMapGet(dependentKeyTable, [self class]);
+  NSMapTable *keys = NSMapGet(dependentKeyTable, [self class]);
 
   if (keys != nil)
     {
-      NSHashTable dependents = NSMapGet(keys, aKey);
+      NSHashTable *dependents = NSMapGet(keys, aKey);
 
       if (dependents != nil)
         {
@@ -1778,7 +1807,7 @@ replacementForClass(Class c)
 + (void) setKeys: (NSArray*)triggerKeys
 triggerChangeNotificationsForDependentKey: (NSString*)dependentKey
 {
-  NSMapTable    affectingKeys;
+  NSMapTable    *affectingKeys;
   NSEnumerator  *enumerator;
   NSString      *affectingKey;
 
@@ -1793,7 +1822,8 @@ triggerChangeNotificationsForDependentKey: (NSString*)dependentKey
   enumerator = [triggerKeys objectEnumerator];
   while ((affectingKey = [enumerator nextObject]))
     {
-      NSHashTable dependentKeys = NSMapGet(affectingKeys, affectingKey);
+      NSHashTable *dependentKeys = NSMapGet(affectingKeys, affectingKey);
+
       if (!dependentKeys)
         {
           dependentKeys = NSCreateHashTable(NSObjectHashCallBacks, 10);
@@ -1810,7 +1840,7 @@ triggerChangeNotificationsForDependentKey: (NSString*)dependentKey
   setup();
   [kvoLock lock];
   info = NSMapGet(infoTable, (void*)self);
-  AUTORELEASE(RETAIN((id)info));
+  IF_NO_GC(AUTORELEASE(RETAIN((id)info));)
   [kvoLock unlock];
   return info;
 }
