@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSLock class reference</title>
-   $Date: 2008-12-16 09:12:18 +0000 (Tue, 16 Dec 2008) $ $Revision: 27303 $
+   $Date: 2009-04-10 23:13:33 -0600 (Fri, 10 Apr 2009) $ $Revision: 28203 $
 */
 
 #include "config.h"
@@ -163,7 +163,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -175,7 +175,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
     [super description], _name];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   if (_mutex != 0)
     {
@@ -293,7 +293,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 
 // Designated initializer
 // Initialize lock with condition
-- (id) initWithCondition: (int)value
+- (id) initWithCondition: (NSInteger)value
 {
   self = [super init];
   if (self != nil)
@@ -321,7 +321,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -333,7 +333,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
     [super description], _name];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   if (_condition != 0)
     {
@@ -362,13 +362,13 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 }
 
 // Return the current condition of the lock
-- (int) condition
+- (NSInteger) condition
 {
   return _condition_value;
 }
 
 // Acquiring and release the lock
-- (void) lockWhenCondition: (int)value
+- (void) lockWhenCondition: (NSInteger)value
 {
   CHECK_RECURSIVE_CONDITION_LOCK(_MUTEX);
 
@@ -400,7 +400,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
   ASSIGNCOPY(_name, name);
 }
 
-- (void) unlockWithCondition: (int)value
+- (void) unlockWithCondition: (NSInteger)value
 {
   int depth;
 
@@ -447,8 +447,11 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 
 - (BOOL) tryLock
 {
-  CHECK_RECURSIVE_CONDITION_LOCK(_MUTEX);
-
+  if ((_MUTEX)->owner == objc_thread_id())
+    {
+      NSDebugLog(@"WARNING: Thread attempted to recursively tryLock : %@", self);
+      return NO;
+    }
   // Ask the runtime to acquire a lock on the mutex
   if (objc_mutex_trylock(_MUTEX) == -1)
     return NO;
@@ -456,7 +459,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
     return YES;
 }
 
-- (BOOL) tryLockWhenCondition: (int)value
+- (BOOL) tryLockWhenCondition: (NSInteger)value
 {
   // tryLock message will check for recursive locks
 
@@ -495,7 +498,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 }
 
 
-- (BOOL) lockWhenCondition: (int)condition_to_meet
+- (BOOL) lockWhenCondition: (NSInteger)condition_to_meet
                 beforeDate: (NSDate*)limitDate
 {
 #ifndef HAVE_OBJC_CONDITION_TIMEDWAIT
@@ -547,8 +550,8 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
     return YES;
 
   atimeinterval = [limitDate timeIntervalSince1970];
-  endtime.tv_sec =(unsigned int)atimeinterval; // 941883028;//
-  endtime.tv_nsec = (unsigned int)((atimeinterval - (float)endtime.tv_sec)
+  endtime.tv_sec =(NSUInteger)atimeinterval; // 941883028;//
+  endtime.tv_nsec = (NSUInteger)((atimeinterval - (float)endtime.tv_sec)
 				   * 1000000000.0);
 
   while (_condition_value != condition_to_meet)
@@ -642,7 +645,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -654,7 +657,7 @@ NSString *NSRecursiveLockException = @"NSRecursiveLockException";
     [super description], _name];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   if (_mutex != 0)
     {
