@@ -230,7 +230,7 @@ typedef enum {
   GS_H_CONNECTED	// Currently connected.
 } GSHandleState;
 
-@interface GSMessageHandle : NSObject <GCFinalization, RunLoopEvents>
+@interface GSMessageHandle : NSObject <RunLoopEvents>
 {
   int			desc;		/* File descriptor for I/O.	*/
   unsigned		wItem;		/* Index of item being written.	*/
@@ -393,18 +393,10 @@ static Class	runLoopClass;
 	 type: ET_WDESC
       watcher: self
       forMode: NSConnectionReplyMode];
-  [l addEvent: (void*)(uintptr_t)desc
-	 type: ET_EDESC
-      watcher: self
-      forMode: NSConnectionReplyMode];
   [l addEvent: (void*)(uintptr_t)desc
 	 type: ET_WDESC
       watcher: self
       forMode: NSDefaultRunLoopMode];
-  [l addEvent: (void*)(uintptr_t)desc
-	 type: ET_EDESC
-      watcher: self
-      forMode: NSDefaultRunLoopMode];
 
   while (valid == YES && state == GS_H_TRYCON
     && [when timeIntervalSinceNow] > 0)
@@ -416,18 +408,10 @@ static Class	runLoopClass;
 	    type: ET_WDESC
 	 forMode: NSConnectionReplyMode
 	     all: NO];
-  [l removeEvent: (void*)(uintptr_t)desc
-	    type: ET_EDESC
-	 forMode: NSConnectionReplyMode
-	     all: NO];
   [l removeEvent: (void*)(uintptr_t)desc
 	    type: ET_WDESC
 	 forMode: NSDefaultRunLoopMode
 	     all: NO];
-  [l removeEvent: (void*)(uintptr_t)desc
-	    type: ET_EDESC
-	 forMode: NSDefaultRunLoopMode
-	     all: NO];
 
   if (state == GS_H_TRYCON)
     {
@@ -459,7 +443,7 @@ static Class	runLoopClass;
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   DESTROY(rData);
   DESTROY(rItems);
   DESTROY(wMsgs);
@@ -478,7 +462,7 @@ static Class	runLoopClass;
   return desc;
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   [self invalidate];
   (void)close(desc);
@@ -504,10 +488,6 @@ static Class	runLoopClass;
 		    type: ET_WDESC
 		 forMode: nil
 		     all: YES];
-	  [l removeEvent: (void*)(uintptr_t)desc
-		    type: ET_EDESC
-		 forMode: nil
-		     all: YES];
 	  NSDebugMLLog(@"NSMessagePort", @"invalidated 0x%x", self);
 	  [[self recvPort] removeHandle: self];
 	  [[self sendPort] removeHandle: self];
@@ -536,7 +516,7 @@ static Class	runLoopClass;
 {
   NSDebugMLLog(@"NSMessagePort_details",
     @"received %s event on 0x%x",
-    type == ET_RPORT ? "read" : "write", self);
+    type != ET_WDESC ? "read" : "write", self);
   /*
    * If we have been invalidated (desc < 0) then we should ignore this
    * event and remove ourself from the runloop.
@@ -549,16 +529,12 @@ static Class	runLoopClass;
 		type: ET_WDESC
 	     forMode: mode
 		 all: YES];
-      [l removeEvent: data
-		type: ET_EDESC
-	     forMode: mode
-		 all: YES];
       return;
     }
 
   M_LOCK(myLock);
 
-  if (type == ET_RPORT)
+  if (type != ET_WDESC)
     {
       unsigned	want;
       void	*bytes;
@@ -874,7 +850,7 @@ static Class	runLoopClass;
 	      DESTROY(rItems);
 	      NSDebugMLLog(@"NSMessagePort_details",
 		@"got message %@ on 0x%x", pm, self);
-	      RETAIN(rp);
+	      IF_NO_GC([rp retain];)
 	      M_UNLOCK(myLock);
 	      NS_DURING
 		{
@@ -1023,24 +999,16 @@ static Class	runLoopClass;
 
   l = [runLoopClass currentRunLoop];
 
-  RETAIN(self);
+  IF_NO_GC([self retain];)
 
   [l addEvent: (void*)(uintptr_t)desc
 	 type: ET_WDESC
       watcher: self
       forMode: NSConnectionReplyMode];
-  [l addEvent: (void*)(uintptr_t)desc
-	 type: ET_EDESC
-      watcher: self
-      forMode: NSConnectionReplyMode];
   [l addEvent: (void*)(uintptr_t)desc
 	 type: ET_WDESC
       watcher: self
       forMode: NSDefaultRunLoopMode];
-  [l addEvent: (void*)(uintptr_t)desc
-	 type: ET_EDESC
-      watcher: self
-      forMode: NSDefaultRunLoopMode];
 
   while (valid == YES
     && [wMsgs indexOfObjectIdenticalTo: components] != NSNotFound
@@ -1055,18 +1023,10 @@ static Class	runLoopClass;
 	    type: ET_WDESC
 	 forMode: NSConnectionReplyMode
 	     all: NO];
-  [l removeEvent: (void*)(uintptr_t)desc
-	    type: ET_EDESC
-	 forMode: NSConnectionReplyMode
-	     all: NO];
   [l removeEvent: (void*)(uintptr_t)desc
 	    type: ET_WDESC
 	 forMode: NSDefaultRunLoopMode
 	     all: NO];
-  [l removeEvent: (void*)(uintptr_t)desc
-	    type: ET_EDESC
-	 forMode: NSDefaultRunLoopMode
-	     all: NO];
 
   if ([wMsgs indexOfObjectIdenticalTo: components] == NSNotFound)
     {
@@ -1137,7 +1097,7 @@ static void clean_up_sockets(void)
 	unlink([name bytes]);
     }
   NSEndMapTableEnumeration(&mEnum);
-  DESTROY(arp);
+  IF_NO_GC(DESTROY(arp);)
   if (unknownThread == YES)
     {
       GSUnregisterCurrentThread();
@@ -1334,7 +1294,7 @@ typedef	struct {
   else
     {
       RELEASE(theName);
-      RETAIN(port);
+      IF_NO_GC([port retain];)
       NSDebugMLLog(@"NSMessagePort", @"Using pre-existing port: %@", port);
     }
   IF_NO_GC(AUTORELEASE(port));
@@ -1368,7 +1328,7 @@ typedef	struct {
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -1381,7 +1341,7 @@ typedef	struct {
   return desc;
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   NSDebugMLLog(@"NSMessagePort", @"NSMessagePort 0x%x finalized", self);
   [self invalidate];
@@ -1454,7 +1414,7 @@ typedef	struct {
     {
       if ((NSPort*) [handle recvPort] == recvPort)
 	{
-	  RETAIN(handle);
+	  IF_NO_GC([handle retain];)
 	  NSEndMapTableEnumeration(&me);
 	  M_UNLOCK(myLock);
 	  return AUTORELEASE(handle);
@@ -1573,7 +1533,7 @@ typedef	struct {
 {
   if ([self isValid] == YES)
     {
-      RETAIN(self);
+      IF_NO_GC([self retain];)
       M_LOCK(myLock);
 
       if ([self isValid] == YES)
@@ -1673,7 +1633,6 @@ typedef	struct {
 
 	  if (type == ET_RDESC) t = "rdesc";
 	  else if (type == ET_WDESC) t = "wdesc";
-	  else if (type == ET_EDESC) t = "edesc";
 	  else if (type == ET_RPORT) t = "rport";
 	  else t = "unknown";
 	  NSLog(@"No handle for event %s on descriptor %d", t, desc);
@@ -1709,7 +1668,7 @@ typedef	struct {
 
 - (void) removeHandle: (GSMessageHandle*)handle
 {
-  RETAIN(self);
+  IF_NO_GC([self retain];)
   M_LOCK(myLock);
   if ([handle sendPort] == self)
     {
@@ -1722,7 +1681,7 @@ typedef	struct {
 	   * been retained - we must therefore release this port since the
 	   * handle no longer uses it.
 	   */
-	  AUTORELEASE(self);
+	  IF_NO_GC([self autorelease];)
 	}
       handle->sendPort = nil;
     }
