@@ -20,7 +20,7 @@
    Boston, MA 02111 USA.
 
    <title>NSMessagePortNameServer class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-16 04:54:59 -0600 (Mon, 16 Mar 2009) $ $Revision: 28086 $
    */
 
 #include "Foundation/NSPortNameServer.h"
@@ -74,7 +74,7 @@ Since we _have_to_ deal with this anyway, we handle it in -removePort: and
 -removePort:forName:, and we don't bother removing entries in the map when
 unregistering a name not for a specific port.
 */
-static NSMapTable portToNamesMap;
+static NSMapTable *portToNamesMap;
 
 
 @interface NSMessagePortNameServer (private)
@@ -97,7 +97,7 @@ static void clean_up_names(void)
       [defaultServer removePort: port];
     }
   NSEndMapTableEnumeration(&mEnum);
-  DESTROY(arp);
+  IF_NO_GC(DESTROY(arp);)
   if (unknownThread == YES)
     {
       GSUnregisterCurrentThread();
@@ -206,7 +206,7 @@ static void clean_up_names(void)
       data = [GSMimeDocument encodeBase64: data];
       name = [[NSString alloc] initWithData: data
 				   encoding: NSASCIIStringEncoding];
-      AUTORELEASE(name);
+      IF_NO_GC([name autorelease];)
     }
   [serverLock lock];
   if (!base_path)
