@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSMethodSignature class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
    */
 
 #include "config.h"
@@ -53,7 +53,7 @@
   return newMs;
 }
 
-- (NSArgumentInfo) argumentInfoAtIndex: (unsigned)index
+- (NSArgumentInfo) argumentInfoAtIndex: (NSUInteger)index
 {
   if (index >= _numArgs)
     {
@@ -67,12 +67,12 @@
   return _info[index+1];
 }
 
-- (unsigned) frameLength
+- (NSUInteger) frameLength
 {
   return _argFrameLength;
 }
 
-- (const char*) getArgumentTypeAtIndex: (unsigned)index
+- (const char*) getArgumentTypeAtIndex: (NSUInteger)index
 {
   if (index >= _numArgs)
     {
@@ -95,7 +95,7 @@
   return (_info[0].qual & _F_ONEWAY) ? YES : NO;
 }
 
-- (unsigned) methodReturnLength
+- (NSUInteger) methodReturnLength
 {
   if (_info == 0)
     {
@@ -113,7 +113,7 @@
   return _info[0].type;
 }
 
-- (unsigned) numberOfArguments
+- (NSUInteger) numberOfArguments
 {
   return _numArgs;
 }
