@@ -26,17 +26,18 @@
    Boston, MA 02111 USA.
 
    <title>NSNotificationCenter class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-10 05:10:27 -0600 (Tue, 10 Mar 2009) $ $Revision: 28061 $
 */
 
 #include "config.h"
-#include "Foundation/NSNotification.h"
-#include "Foundation/NSException.h"
-#include "Foundation/NSLock.h"
-#include "Foundation/NSThread.h"
-#include "Foundation/NSDebug.h"
-#include "GNUstepBase/GSLock.h"
+#import "Foundation/NSNotification.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSLock.h"
+#import "Foundation/NSThread.h"
+#import "Foundation/NSDebug.h"
+#import "GNUstepBase/GSLock.h"
 
+static NSZone	*_zone = 0;
 
 /**
  * Concrete class implementing NSNotification.
@@ -211,6 +212,13 @@ static void obsFree(Observation *o);
 #define GSI_MAP_VEXTRA Observation*
 #define	GSI_MAP_EXTRA	void*
 
+#if	GS_WITH_GC
+#include	<gc_typed.h>
+static GC_descr	nodeDesc;	// Type descriptor for map node.
+#define	GSI_MAP_NODES(M, X) \
+(GSIMapNode)GC_calloc_explicitly_typed(X, sizeof(GSIMapNode_t), nodeDesc)
+#endif
+
 #include "GNUstepBase/GSIMap.h"
 
 @class	GSLazyRecursiveLock;
@@ -271,13 +279,20 @@ static Observation *obsNew(NCTable* t)
 	  unsigned	size;
 
 	  t->numChunks++;
+
 	  size = t->numChunks * sizeof(Observation*);
+#if	GS_WITH_GC
+	  t->chunks = (Observation**)NSReallocateCollectable(
+	    t->chunks, size, NSScannedOption);
+#else
 	  t->chunks = (Observation**)NSZoneRealloc(NSDefaultMallocZone(),
 	    t->chunks, size);
+#endif
+
 	  size = CHUNKSIZE * sizeof(Observation);
 #if	GS_WITH_GC
 	  t->chunks[t->numChunks - 1]
-	    = (Observation*)NSZoneMallocAtomic(NSDefaultMallocZone(), size);
+	    = (Observation*)NSAllocateCollectable(size, 0);
 #else
 	  t->chunks[t->numChunks - 1]
 	    = (Observation*)NSZoneMalloc(NSDefaultMallocZone(), size);
@@ -303,8 +318,12 @@ static GSIMapTable	mapNew(NCTable *t)
     {
       GSIMapTable	m;
 
-      m = NSZoneMalloc(NSDefaultMallocZone(), sizeof(GSIMapTable_t));
-      GSIMapInitWithZoneAndCapacity(m, NSDefaultMallocZone(), 2);
+#if	GS_WITH_GC
+      m = NSAllocateCollectable(sizeof(GSIMapTable_t), NSScannedOption);
+#else
+      m = NSZoneMalloc(_zone, sizeof(GSIMapTable_t));
+#endif
+      GSIMapInitWithZoneAndCapacity(m, _zone, 2);
       return m;
     }
 }
@@ -312,7 +331,9 @@ static GSIMapTable	mapNew(NCTable *t)
 static void	mapFree(NCTable *t, GSIMapTable m)
 {
   if (t->cacheIndex < CACHESIZE)
-    t->cache[t->cacheIndex++] = m;
+    {
+      t->cache[t->cacheIndex++] = m;
+    }
   else
     {
       GSIMapEmptyMap(m);
@@ -390,16 +411,24 @@ static NCTable *newNCTable(void)
 {
   NCTable	*t;
 
-  t = (NCTable*)NSZoneMalloc(NSDefaultMallocZone(), sizeof(NCTable));
+#if	GS_WITH_GC
+  t = (NCTable*)NSAllocateCollectable(sizeof(NCTable), NSScannedOption);
+#else
+  t = (NCTable*)NSZoneMalloc(_zone, sizeof(NCTable));
   memset((void*)t, '\0', sizeof(NCTable));
+#endif
   t->chunkIndex = CHUNKSIZE;
   t->wildcard = ENDOBS;
 
-  t->nameless = NSZoneMalloc(NSDefaultMallocZone(), sizeof(GSIMapTable_t));
-  GSIMapInitWithZoneAndCapacity(t->nameless, NSDefaultMallocZone(), 16);
-
-  t->named = NSZoneMalloc(NSDefaultMallocZone(), sizeof(GSIMapTable_t));
-  GSIMapInitWithZoneAndCapacity(t->named, NSDefaultMallocZone(), 128);
+#if	GS_WITH_GC
+  t->nameless = NSAllocateCollectable(sizeof(GSIMapTable_t), NSScannedOption);
+  t->named = NSAllocateCollectable(sizeof(GSIMapTable_t), NSScannedOption);
+#else
+  t->nameless = NSZoneMalloc(_zone, sizeof(GSIMapTable_t));
+  t->named = NSZoneMalloc(_zone, sizeof(GSIMapTable_t));
+#endif
+  GSIMapInitWithZoneAndCapacity(t->nameless, _zone, 16);
+  GSIMapInitWithZoneAndCapacity(t->named, _zone, 128);
 
   // t->_lock = [GSLazyRecursiveLock new];
   t->_lock = [NSRecursiveLock new];
@@ -425,6 +454,9 @@ static void obsFree(Observation *o)
     {
       NCTable	*t = o->link;
 
+#if	GS_WITH_GC
+      GSAssignZeroingWeakPointer((void**)&o->observer, 0);
+#endif
       o->link = (NCTable*)t->freeList;
       t->freeList = o;
     }
@@ -447,6 +479,8 @@ static void listFree(Observation *list)
  *	we remove to be zero so that, if it currently exists in an array
  *	of observations being posted, the posting code can notice that it
  *	has been removed from its linked list.
+ *
+ *	Also, 
  */
 static Observation *listPurge(Observation *list, id observer)
 {
@@ -526,6 +560,30 @@ purgeMapNode(GSIMapTable map, GSIMapNode node, id observer)
     }
 }
 
+/* purgeCollected() returns a list of observations with any observations for
+ * a collected observer removed.
+ * purgeCollectedFromMapNode() does the same thing but also handles cleanup
+ * of the map node containing the list if necessary.
+ */
+#if	GS_WITH_GC
+#define	purgeCollected(X)	listPurge(X, nil)
+static Observation*
+purgeCollectedFromMapNode(GSIMapTable map, GSIMapNode node)
+{
+  Observation	*o;
+
+  o = node->value.ext = purgeCollected((Observation*)(node->value.ext));
+  if (o == ENDOBS)
+    {
+      GSIMapRemoveKey(map, node->key);
+    }
+  return o;
+}
+#else
+#define	purgeCollected(X)	(X)
+#define purgeCollectedFromMapNode(X, Y) ((Observation*)Y->value.ext)
+#endif
+
 /*
  * In order to hide pointers from garbage collection, we OR in an
  * extra bit.  This should be ok for the objects we deal with
@@ -533,7 +591,7 @@ purgeMapNode(GSIMapTable map, GSIMapNode node, id observer)
  * I know of.
  *
  * We also use this trick to differentiate between map table keys that
- * should be treated as objects (notification names) and thise that
+ * should be treated as objects (notification names) and those that
  * should be treated as pointers (notification objects)
  */
 #define	CHEATGC(X)	(id)(((uintptr_t)X) | 1)
@@ -570,6 +628,16 @@ static NSNotificationCenter *default_center = nil;
 {
   if (self == [NSNotificationCenter class])
     {
+#if	GS_WITH_GC
+      /* We create a typed memory descriptor for map nodes.
+       */
+      GC_word	w[GC_BITMAP_SIZE(GSIMapNode_t)] = {0};
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, key));
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, value));
+      nodeDesc = GC_make_descriptor(w, GC_WORD_LEN(GSIMapNode_t));
+#else
+      _zone = NSDefaultMallocZone();
+#endif
       if (concrete == 0)
 	{
 	  concrete = [GSNotification class];
@@ -607,12 +675,12 @@ static NSNotificationCenter *default_center = nil;
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
 
   [super dealloc];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   if (self == default_center)
     {
@@ -641,7 +709,10 @@ static NSNotificationCenter *default_center = nil;
  *
  * <p>The notification center does not retain observer or object. Therefore,
  * you should always send removeObserver: or removeObserver:name:object: to
- * the notification center before releasing these objects.</p>
+ * the notification center before releasing these objects.<br />
+ * As a convenience, when built with garbage collection, you do not need to
+ * remove any garbage collected observer as the system will do it implicitly.
+ * </p>
  *
  * <p>NB. For MacOS-X compatibility, adding an observer multiple times will
  * register it to receive multiple copies of any matching notification, however
@@ -683,12 +754,18 @@ static NSNotificationCenter *default_center = nil;
   o = obsNew(TABLE);
   o->selector = selector;
   o->method = method;
+#if	GS_WITH_GC
+  GSAssignZeroingWeakPointer((void**)&o->observer, (void*)observer);
+#else
   o->observer = observer;
+#endif
   o->retained = 0;
   o->next = 0;
 
   if (object != nil)
-    object = CHEATGC(object);
+    {
+      object = CHEATGC(object);
+    }
 
   /*
    * Record the Observation in one of the linked lists.
@@ -952,6 +1029,9 @@ static NSNotificationCenter *default_center = nil;
   GSIArrayItem	i[64];
   GSIArray_t	b;
   GSIArray	a = &b;
+#if	GS_WITH_GC
+  NSGarbageCollector	*collector = [NSGarbageCollector defaultCollector];
+#endif
 
   if (name == nil)
     {
@@ -966,19 +1046,26 @@ static NSNotificationCenter *default_center = nil;
     }
 
   /*
-   * Initialise static array to store copies of observers.
-   */
-  GSIArrayInitWithZoneAndStaticCapacity(a, NSDefaultMallocZone(), 64, i);
-
-  /*
-   * Lock the table of observers while we traverse it.
+   * Lock the table of observations while we traverse it.
+   *
+   * The table of observations contains weak pointers which are zeroed when
+   * the observers get garbage collected.  So to avoid consistency problems
+   * we disable gc while we copy all the observations we are interested in.
+   * We use scanned memory in the array in the case where there are more
+   * than the 64 observers we allowed room for on the stack.
    */
+#if	GS_WITH_GC
+  GSIArrayInitWithZoneAndStaticCapacity(a, (NSZone*)1, 64, i);
+  [collector disable];
+#else
+  GSIArrayInitWithZoneAndStaticCapacity(a, _zone, 64, i);
+#endif
   lockNCTable(TABLE);
 
   /*
    * Find all the observers that specified neither NAME nor OBJECT.
    */
-  for (o = WILDCARD; o != ENDOBS; o = o->next)
+  for (o = WILDCARD = purgeCollected(WILDCARD); o != ENDOBS; o = o->next)
     {
       GSIArrayAddItem(a, (GSIArrayItem)o);
     }
@@ -991,7 +1078,7 @@ static NSNotificationCenter *default_center = nil;
       n = GSIMapNodeForSimpleKey(NAMELESS, (GSIMapKey)object);
       if (n != 0)
 	{
-	  o = n->value.ext;
+	  o = purgeCollectedFromMapNode(NAMELESS, n);
 	  while (o != ENDOBS)
 	    {
 	      GSIArrayAddItem(a, (GSIArrayItem)o);
@@ -1023,7 +1110,7 @@ static NSNotificationCenter *default_center = nil;
 	  n = GSIMapNodeForSimpleKey(m, (GSIMapKey)object);
 	  if (n != 0)
 	    {
-	      o = n->value.ext;
+	      o = purgeCollectedFromMapNode(m, n);
 	      while (o != ENDOBS)
 		{
 		  GSIArrayAddItem(a, (GSIArrayItem)o);
@@ -1039,7 +1126,7 @@ static NSNotificationCenter *default_center = nil;
 	      n = GSIMapNodeForSimpleKey(m, (GSIMapKey)nil);
 	      if (n != 0)
 		{
-		  o = n->value.ext;
+	          o = purgeCollectedFromMapNode(m, n);
 		  while (o != ENDOBS)
 		    {
 		      GSIArrayAddItem(a, (GSIArrayItem)o);
@@ -1047,14 +1134,18 @@ static NSNotificationCenter *default_center = nil;
 		    }
 		}
 	    }
-
 	}
     }
 
   /*
-   * Finished with the table ... we can unlock it.
+   * Finished with the table ... we can unlock it and re-enable garbage
+   * collection, safe in the knowledge that the observers we will be
+   * notifying won't get collected prematurely.
    */
   unlockNCTable(TABLE);
+#if	GS_WITH_GC
+  [collector enable];
+#endif
 
   /*
    * Now send all the notifications.
