@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSNotificationQueue class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-18 09:08:13 -0600 (Wed, 18 Mar 2009) $ $Revision: 28093 $
    */
 
 #include "config.h"
@@ -241,7 +241,12 @@ add_to_queue(NSNotificationQueueList *queue, NSNotification *notification,
 {
   NSNotificationQueueRegistration	*item;
 
+#if	GS_WITH_GC
+  item = NSAllocateCollectable(sizeof(NSNotificationQueueRegistration),
+    NSScannedOption);
+#else
   item = NSZoneCalloc(_zone, 1, sizeof(NSNotificationQueueRegistration));
+#endif
   if (item == 0)
     {
       [NSException raise: NSMallocException
@@ -334,8 +339,15 @@ add_to_queue(NSNotificationQueueList *queue, NSNotification *notification,
 
   // init queue
   _center = RETAIN(notificationCenter);
+#if	GS_WITH_GC
+  _asapQueue = NSAllocateCollectable(sizeof(NSNotificationQueueList),
+    NSScannedOption);
+  _idleQueue = NSAllocateCollectable(sizeof(NSNotificationQueueList),
+    NSScannedOption);
+#else
   _asapQueue = NSZoneCalloc(_zone, 1, sizeof(NSNotificationQueueList));
   _idleQueue = NSZoneCalloc(_zone, 1, sizeof(NSNotificationQueueList));
+#endif
   if (_asapQueue == 0 || _idleQueue == 0)
     {
       DESTROY(self);
@@ -360,15 +372,15 @@ add_to_queue(NSNotificationQueueList *queue, NSNotification *notification,
   [NotificationQueueList unregisterQueue: self];
 
   /*
-   * release self from queues
+   * release items from our queues
    */
-  for (item = _asapQueue->head; item; item=item->prev)
+  while ((item = _asapQueue->head) != 0)
     {
       remove_from_queue(_asapQueue, item, _zone);
     }
   NSZoneFree(_zone, _asapQueue);
 
-  for (item = _idleQueue->head; item; item=item->prev)
+  while ((item = _idleQueue->head) != 0)
     {
       remove_from_queue(_idleQueue, item, _zone);
     }
@@ -392,7 +404,7 @@ add_to_queue(NSNotificationQueueList *queue, NSNotification *notification,
  * <em>not</em> posted.
  */
 - (void) dequeueNotificationsMatching: (NSNotification*)notification
-			 coalesceMask: (unsigned int)coalesceMask
+			 coalesceMask: (NSUInteger)coalesceMask
 {
   NSNotificationQueueRegistration	*item;
   NSNotificationQueueRegistration	*next;
@@ -507,7 +519,7 @@ add_to_queue(NSNotificationQueueList *queue, NSNotification *notification,
  */
 - (void) enqueueNotification: (NSNotification*)notification
 		postingStyle: (NSPostingStyle)postingStyle
-		coalesceMask: (unsigned int)coalesceMask
+		coalesceMask: (NSUInteger)coalesceMask
 		    forModes: (NSArray*)modes
 {
   if (coalesceMask != NSNotificationNoCoalescing)
