@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSNumberFormatter class reference</title>
-   $Date: 2008-11-17 13:45:32 +0000 (Mon, 17 Nov 2008) $ $Revision: 27080 $
+   $Date: 2009-01-12 05:48:46 -0700 (Mon, 12 Jan 2009) $ $Revision: 27578 $
    */
 
 #include "Foundation/NSAttributedString.h"
@@ -102,17 +102,17 @@
 {
   NSNumberFormatter	*c = (NSNumberFormatter*) NSCopyObject(self, 0, zone);
 
-  RETAIN(c->_negativeFormat);
-  RETAIN(c->_positiveFormat);
-  RETAIN(c->_attributesForPositiveValues);
-  RETAIN(c->_attributesForNegativeValues);
-  RETAIN(c->_maximum);
-  RETAIN(c->_minimum);
-  RETAIN(c->_roundingBehavior);
-  RETAIN(c->_roundingBehavior);
-  RETAIN(c->_attributedStringForNil);
-  RETAIN(c->_attributedStringForNotANumber);
-  RETAIN(c->_attributedStringForZero);
+  IF_NO_GC(RETAIN(c->_negativeFormat);)
+  IF_NO_GC(RETAIN(c->_positiveFormat);)
+  IF_NO_GC(RETAIN(c->_attributesForPositiveValues);)
+  IF_NO_GC(RETAIN(c->_attributesForNegativeValues);)
+  IF_NO_GC(RETAIN(c->_maximum);)
+  IF_NO_GC(RETAIN(c->_minimum);)
+  IF_NO_GC(RETAIN(c->_roundingBehavior);)
+  IF_NO_GC(RETAIN(c->_roundingBehavior);)
+  IF_NO_GC(RETAIN(c->_attributedStringForNil);)
+  IF_NO_GC(RETAIN(c->_attributedStringForNotANumber);)
+  IF_NO_GC(RETAIN(c->_attributedStringForZero);)
 
   return c;
 }
