@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2008-11-26 09:20:34 +0000 (Wed, 26 Nov 2008) $ $Revision: 27135 $
+   $Date: 2009-05-04 14:07:56 -0600 (Mon, 04 May 2009) $ $Revision: 28268 $
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -92,6 +92,8 @@ static IMP autorelease_imp;
 #include	<gc.h>
 #include	<gc_typed.h>
 
+static SEL finalize_sel;
+static IMP finalize_imp;
 #endif
 
 static Class	NSConstantStringClass;
@@ -111,8 +113,7 @@ static Class	NSConstantStringClass;
 
 /*
  * allocationLock is needed when running multi-threaded for 
- * protecting the map table of zombie information and for retain
- * count protection (when using a map table ... REFCNT_LOCAL is NO)
+ * protecting the map table of zombie information.
  */
 static objc_mutex_t allocationLock = NULL;
 
@@ -121,8 +122,9 @@ BOOL	NSDeallocateZombies = NO;
 
 @class	NSZombie;
 static Class		zombieClass;
-static NSMapTable	zombieMap;
+static NSMapTable	*zombieMap;
 
+#if	!GS_WITH_GC
 static void GSMakeZombie(NSObject *o)
 {
   Class	c = ((id)o)->class_pointer;
@@ -141,6 +143,7 @@ static void GSMakeZombie(NSObject *o)
 	}
     }
 }
+#endif
 
 static void GSLogZombie(id o, SEL sel)
 {
@@ -177,24 +180,12 @@ static void GSLogZombie(id o, SEL sel)
 
 /*
  *	Reference count and memory management
- *
- *	If REFCNT_LOCAL is defined, reference counts for object are stored
- *	with the object, otherwise they are stored in a global map table
- *	that has to be protected by mutexes in a multithreraded environment.
- *	You therefore want REFCNT_LOCAL defined for best performance.
- *
- *	If CACHE_ZONE is defined, the zone in which an object has been
- *	allocated is stored with the object - this makes lookup of the
- *	correct zone to free memory very fast.
+ *	Reference counts for object are stored
+ *	with the object.
+ *	The zone in which an object has been
+ *	allocated is stored with the object.
  */
 
-
-#if	GS_WITH_GC == 0 && !defined(NeXT_RUNTIME)
-#define	REFCNT_LOCAL	1
-#define	CACHE_ZONE	1
-#endif
-
-
 /* Now, if we are on a platform where we know how to do atomic
  * read, increment, and decrement, then we define the GSATOMICREAD
  * macro and macros or functions to increment/decrement.
@@ -209,8 +200,6 @@ static void GSLogZombie(id o, SEL sel)
 #undef	GSATOMICREAD
 #endif
 
-#if	defined(REFCNT_LOCAL)
-
 #if	defined(__MINGW32__)
 #ifndef _WIN64
 #undef InterlockedIncrement
@@ -229,7 +218,14 @@ typedef int32_t volatile *gsatomic_t;
 #define	GSAtomicIncrement(X)	InterlockedIncrement((LONG volatile*)X)
 #define	GSAtomicDecrement(X)	InterlockedDecrement((LONG volatile*)X)
 
-#endif
+
+#elif defined(USE_ATOMIC_BUILDINS) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1))
+/* Use the GCC atomic operations with recent GCC versions */
+
+typedef int32_t volatile *gsatomic_t;
+#define GSATOMICREAD(X) (*(X))
+#define GSAtomicIncrement(X)    __sync_fetch_and_add(X, 1)
+#define GSAtomicDecrement(X)    __sync_fetch_and_sub(X, 1)
 
 
 #elif	defined(__linux__) && (defined(__i386__) || defined(__x86_64__))
@@ -269,11 +265,11 @@ GSAtomicIncrement(gsatomic_t X)
 {
   int tmp;
   __asm__ __volatile__ (
-    "modified:"
+    "incmodified:"
     "lwarx %0,0,%1 \n"
     "addic %0,%0,1 \n"
     "stwcx. %0,0,%1 \n"
-    "bne- modified \n"
+    "bne- incmodified \n"
     :"=&r" (tmp)
     :"r" (X)
     :"cc", "memory");
@@ -285,11 +281,11 @@ GSAtomicDecrement(gsatomic_t X)
 {
   int tmp;
   __asm__ __volatile__ (
-    "modified:"
+    "decmodified:"
     "lwarx %0,0,%1 \n"
     "addic %0,%0,-1 \n"
     "stwcx. %0,0,%1 \n"
-    "bne- modified \n"
+    "bne- decmodified \n"
     :"=&r" (tmp)
     :"r" (X)
     :"cc", "memory");
@@ -298,12 +294,12 @@ GSAtomicDecrement(gsatomic_t X)
 
 #endif
 
-#if	defined(REFCNT_LOCAL) && !defined(GSATOMICREAD)
+#if	!defined(GSATOMICREAD)
 
 /*
  * Having just one allocationLock for all leads to lock contention
  * if there are lots of threads doing lots of retain/release calls.
- * To alleviate this, if using REFCNT_LOCAL, instead of a single
+ * To alleviate this, instead of a single
  * allocationLock for all objects, we divide the object space into
  * chunks, each with its own lock. The chunk is selected by shifting
  * off the low-order ALIGNBITS of the object's pointer (these bits
@@ -320,7 +316,7 @@ static objc_mutex_t allocationLocks[LOCKCOUNT] = { 0 };
 
 static inline objc_mutex_t GSAllocationLockForObject(id p)
 {
-  unsigned i = ((((unsigned)(uintptr_t)p) >> ALIGNBITS) & LOCKMASK);
+  NSUInteger i = ((((NSUInteger)(uintptr_t)p) >> ALIGNBITS) & LOCKMASK);
   return allocationLocks[i];
 }
 
@@ -332,19 +328,13 @@ static inline objc_mutex_t GSAllocationLockForObject(id p)
 #endif
 #define	ALIGN __alignof__(double)
 
-#if	defined(REFCNT_LOCAL) || defined(CACHE_ZONE)
-
 /*
  *	Define a structure to hold information that is held locally
  *	(before the start) in each object.
  */
 typedef struct obj_layout_unpadded {
-#if	defined(REFCNT_LOCAL)
-    unsigned	retained;
-#endif
-#if	defined(CACHE_ZONE)
+    NSUInteger	retained;
     NSZone	*zone;
-#endif
 } unp;
 #define	UNP sizeof(unp)
 
@@ -354,40 +344,12 @@ typedef struct obj_layout_unpadded {
  *	structure correct.
  */
 struct obj_layout {
-#if	defined(REFCNT_LOCAL)
-    unsigned	retained;
-#endif
-#if	defined(CACHE_ZONE)
+    NSUInteger	retained;
     NSZone	*zone;
-#endif
     char	padding[ALIGN - ((UNP % ALIGN) ? (UNP % ALIGN) : ALIGN)];
 };
 typedef	struct obj_layout *obj;
 
-#endif	/* defined(REFCNT_LOCAL) || defined(CACHE_ZONE) */
-
-#if !defined(REFCNT_LOCAL)
-
-/*
- * Set up map table for non-local reference counts.
- */
-
-#define GSI_MAP_EQUAL(M, X, Y)	(X.obj == Y.obj)
-#define GSI_MAP_HASH(M, X)	(X.uint >> 2)
-#define GSI_MAP_RETAIN_KEY(M, X)
-#define GSI_MAP_RELEASE_KEY(M, X)
-#define GSI_MAP_RETAIN_VAL(M, X)
-#define GSI_MAP_RELEASE_VAL(M, X)
-#define GSI_MAP_KTYPES  GSUNION_OBJ|GSUNION_INT
-#define GSI_MAP_VTYPES  GSUNION_INT
-#define	GSI_MAP_NOCLEAN	1
-
-#include "GNUstepBase/GSIMap.h"
-
-static GSIMapTable_t	retain_counts = {0};
-
-#endif	/* !defined(REFCNT_LOCAL) */
-
 
 /**
  * Examines the extra reference count for the object and, if non-zero
@@ -402,14 +364,13 @@ NSDecrementExtraRefCountWasZero(id anObject)
 #if	!GS_WITH_GC
   if (double_release_check_enabled)
     {
-      unsigned release_count;
-      unsigned retain_count = [anObject retainCount];
+      NSUInteger release_count;
+      NSUInteger retain_count = [anObject retainCount];
       release_count = [autorelease_class autoreleaseCountForObject: anObject];
       if (release_count >= retain_count)
         [NSException raise: NSGenericException
 		    format: @"Release would release object too many times."];
     }
-#if	defined(REFCNT_LOCAL)
   if (allocationLock != 0)
     {
 #if	defined(GSATOMICREAD)
@@ -462,49 +423,7 @@ NSDecrementExtraRefCountWasZero(id anObject)
 	  return NO;
 	}
     }
-#else
-  GSIMapNode	node;
-
-  if (allocationLock != 0)
-    {
-      objc_mutex_lock(allocationLock);
-      node = GSIMapNodeForKey(&retain_counts, (GSIMapKey)anObject);
-      if (node == 0)
-	{
-	  objc_mutex_unlock(allocationLock);
-	  return YES;
-	}
-      if (node->value.uint == 0)
-	{
-	  GSIMapRemoveKey((GSIMapTable)&retain_counts, (GSIMapKey)anObject);
-	  objc_mutex_unlock(allocationLock);
-	  return YES;
-	}
-      else
-	{
-	  (node->value.uint)--;
-	}
-      objc_mutex_unlock(allocationLock);
-    }
-  else
-    {
-      node = GSIMapNodeForKey(&retain_counts, (GSIMapKey)anObject);
-      if (node == 0)
-	{
-	  return YES;
-	}
-      if ((node->value.uint) == 0)
-	{
-	  GSIMapRemoveKey((GSIMapTable)&retain_counts, (GSIMapKey)anObject);
-	  return YES;
-	}
-      else
-	{
-	  --(node->value.uint);
-	}
-    }
-#endif
-#endif
+#endif /* !GS_WITH_GC */
   return NO;
 }
 
@@ -513,49 +432,14 @@ NSDecrementExtraRefCountWasZero(id anObject)
  * from 0 to the maximum unsigned integer value minus one).<br />
  * The retain count for an object is this value plus one.
  */
-inline unsigned
+inline NSUInteger
 NSExtraRefCount(id anObject)
 {
 #if	GS_WITH_GC
   return UINT_MAX - 1;
 #else	/* GS_WITH_GC */
-#if	defined(REFCNT_LOCAL)
   return ((obj)anObject)[-1].retained;
-#else
-  GSIMapNode	node;
-  unsigned	ret;
-
-  if (allocationLock != 0)
-    {
-      objc_mutex_t theLock = GSAllocationLockForObject(anObject);
-
-      objc_mutex_lock(theLock);
-      node = GSIMapNodeForKey(&retain_counts, (GSIMapKey)anObject);
-      if (node == 0)
-	{
-	  ret = 0;
-	}
-      else
-	{
-	  ret = node->value.uint;
-	}
-      objc_mutex_unlock(theLock);
-    }
-  else
-    {
-      node = GSIMapNodeForKey(&retain_counts, (GSIMapKey)anObject);
-      if (node == 0)
-	{
-	  ret = 0;
-	}
-      else
-	{
-	  ret = node->value.uint;
-	}
-    }
-  return ret;
-#endif
-#endif
+#endif /* GS_WITH_GC */
 }
 
 /**
@@ -570,7 +454,6 @@ NSIncrementExtraRefCount(id anObject)
 #if	GS_WITH_GC
   return;
 #else	/* GS_WITH_GC */
-#if	defined(REFCNT_LOCAL)
   if (allocationLock != 0)
     {
 #if	defined(GSATOMICREAD)
@@ -607,52 +490,16 @@ NSIncrementExtraRefCount(id anObject)
 	}
       ((obj)anObject)[-1].retained++;
     }
-#else	/* REFCNT_LOCAL */
-  GSIMapNode	node;
-
-  if (allocationLock != 0)
-    {
-      objc_mutex_lock(allocationLock);
-      node = GSIMapNodeForKey(&retain_counts, (GSIMapKey)anObject);
-      if (node != 0)
-	{
-	  if ((node->value.uint) == UINT_MAX - 1)
-	    {
-	      objc_mutex_unlock(allocationLock);
-	      [NSException raise: NSInternalInconsistencyException
-		format:
-		@"NSIncrementExtraRefCount() asked to increment too far"];
-	    }
-	  (node->value.uint)++;
-	}
-      else
-	{
-	  GSIMapAddPair(&retain_counts, (GSIMapKey)anObject, (GSIMapVal)1);
-	}
-      objc_mutex_unlock(allocationLock);
-    }
-  else
-    {
-      node = GSIMapNodeForKey(&retain_counts, (GSIMapKey)anObject);
-      if (node != 0)
-	{
-	  if ((node->value.uint) == UINT_MAX - 1)
-	    {
-	      [NSException raise: NSInternalInconsistencyException
-		format:
-		@"NSIncrementExtraRefCount() asked to increment too far"];
-	    }
-	  (node->value.uint)++;
-	}
-      else
-	{
-	  GSIMapAddPair(&retain_counts, (GSIMapKey)anObject, (GSIMapVal)1);
-	}
-    }
-#endif	/* REFCNT_LOCAL */
 #endif	/* GS_WITH_GC */
 }
 
+#ifndef	NDEBUG
+#define	AADD(c, o) GSDebugAllocationAdd(c, o)
+#define	AREM(c, o) GSDebugAllocationRemove(c, o)
+#else
+#define	AADD(c, o) 
+#define	AREM(c, o) 
+#endif
 
 /*
  *	Now do conditional compilation of memory allocation functions
@@ -664,65 +511,66 @@ NSIncrementExtraRefCount(id anObject)
 inline NSZone *
 GSObjCZone(NSObject *object)
 {
-  return 0;
+  /* MacOS-X 10.5 seems to return the default malloc zone if GC is enabled.
+   */
+  return NSDefaultMallocZone();
 }
 
 static void
 GSFinalize(void* object, void* data)
 {
-  [(id)object gcFinalize];
-#ifndef	NDEBUG
-  GSDebugAllocationRemove(((id)object)->class_pointer, (id)object);
-#endif
+  [(id)object finalize];
+  AREM(((id)object)->class_pointer, (id)object);
   ((id)object)->class_pointer = (void*)0xdeadface;
 }
 
+static BOOL
+GSIsFinalizable(Class c)
+{
+  if (get_imp(c, finalize_sel) != finalize_imp)
+    return YES;
+  return NO;
+}
+
 inline NSObject *
-NSAllocateObject(Class aClass, unsigned extraBytes, NSZone *zone)
+NSAllocateObject(Class aClass, NSUInteger extraBytes, NSZone *zone)
 {
   id	new;
   int	size;
+  GC_descr	gc_type;
 
   NSCAssert((CLS_ISCLASS(aClass)), @"Bad class for new object");
+  gc_type = (GC_descr)aClass->gc_object_type;
   size = aClass->instance_size + extraBytes;
-  if (zone == GSAtomicMallocZone())
+  if (size % sizeof(void*) != 0)
     {
-      new = NSZoneMalloc(zone, size);
+      /* Size must be a multiple of pointer size for the garbage collector
+       * to be able to allocate explicitly typed memory.
+       */
+      size += sizeof(void*) - size % sizeof(void*);
+    }
+
+  if (gc_type == 0)
+    {
+      new = NSZoneCalloc(zone, 1, size);
+      NSLog(@"No garbage collection information for '%s'",
+	GSNameFromClass(aClass));
     }
   else
     {
-      GC_descr	gc_type = (GC_descr)aClass->gc_object_type;
-
-      if (gc_type == 0)
-	{
-	  new = NSZoneMalloc(zone, size);
-	  NSLog(@"No garbage collection information for '%s'",
-	    GSNameFromClass(aClass));
-	}
-      else if ([aClass requiresTypedMemory])
-	{
-	  new = GC_CALLOC_EXPLICTLY_TYPED(1, size, gc_type);
-        }
-      else
-	{
-	  new = NSZoneMalloc(zone, size);
-	}
+      new = GC_calloc_explicitly_typed(1, size, gc_type);
     }
 
   if (new != nil)
     {
-      memset(new, 0, size);
       new->class_pointer = aClass;
-      if (__objc_responds_to(new, @selector(gcFinalize)))
+      if (GSIsFinalizable(aClass))
 	{
-#ifndef	NDEBUG
-	  /*
-	   *	We only do allocation counting for objects that can be
-	   *	finalised - for other objects we have no way of decrementing
-	   *	the count when the object is collected.
+	  /* We only do allocation counting for objects that can be
+	   * finalised - for other objects we have no way of decrementing
+	   * the count when the object is collected.
 	   */
-	  GSDebugAllocationAdd(aClass, new);
-#endif
+	  AADD(aClass, new);
 	  GC_REGISTER_FINALIZER (new, GSFinalize, NULL, NULL, NULL);
 	}
     }
@@ -736,10 +584,6 @@ NSDeallocateObject(NSObject *anObject)
 
 #else	/* GS_WITH_GC */
 
-#if	defined(REFCNT_LOCAL) || defined(CACHE_ZONE)
-
-#if defined(CACHE_ZONE)
-
 inline NSZone *
 GSObjCZone(NSObject *object)
 {
@@ -748,24 +592,9 @@ GSObjCZone(NSObject *object)
   return ((obj)object)[-1].zone;
 }
 
-#else	/* defined(CACHE_ZONE)	*/
-
-inline NSZone *
-GSObjCZone(NSObject *object)
-{
-  if (GSObjCClass(object) == NSConstantStringClass)
-    return NSDefaultMallocZone();
-  return NSZoneFromPointer(&((obj)object)[-1]);
-}
-
-#endif	/* defined(CACHE_ZONE)	*/
-
 inline NSObject *
-NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
+NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
 {
-#ifndef	NDEBUG
-  extern void GSDebugAllocationAdd(Class c, id o);
-#endif
   id	new;
   int	size;
 
@@ -779,14 +608,10 @@ NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
   if (new != nil)
     {
       memset (new, 0, size);
-#if	defined(CACHE_ZONE)
       ((obj)new)->zone = zone;
-#endif
       new = (id)&((obj)new)[1];
       new->class_pointer = aClass;
-#ifndef	NDEBUG
-      GSDebugAllocationAdd(aClass, new);
-#endif
+      AADD(aClass, new);
     }
   return new;
 }
@@ -794,17 +619,12 @@ NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
 inline void
 NSDeallocateObject(NSObject *anObject)
 {
-#ifndef	NDEBUG
-  extern void GSDebugAllocationRemove(Class c, id o);
-#endif
   if ((anObject!=nil) && CLS_ISCLASS(((id)anObject)->class_pointer))
     {
       obj	o = &((obj)anObject)[-1];
       NSZone	*z = GSObjCZone(anObject);
 
-#ifndef	NDEBUG
-      GSDebugAllocationRemove(((id)anObject)->class_pointer, (id)anObject);
-#endif
+      AREM(((id)anObject)->class_pointer, (id)anObject);
       if (NSZombieEnabled == YES)
 	{
 	  GSMakeZombie(anObject);
@@ -822,67 +642,41 @@ NSDeallocateObject(NSObject *anObject)
   return;
 }
 
-#else
-
-inline NSZone *
-GSObjCZone(NSObject *object)
-{
-  if (GSObjCClass(object) == NSConstantStringClass)
-    return NSDefaultMallocZone();
-  return NSZoneFromPointer(object);
-}
-
-inline NSObject *
-NSAllocateObject (Class aClass, unsigned extraBytes, NSZone *zone)
-{
-  id	new;
-  int	size;
-
-  NSCAssert((CLS_ISCLASS(aClass)), @"Bad class for new object");
-  size = aClass->instance_size + extraBytes;
-  new = NSZoneMalloc (zone, size);
-  if (new != nil)
-    {
-      memset (new, 0, size);
-      new->class_pointer = aClass;
-#ifndef	NDEBUG
-      GSDebugAllocationAdd(aClass, new);
-#endif
-    }
-  return new;
-}
-
-inline void
-NSDeallocateObject(NSObject *anObject)
-{
-  if ((anObject!=nil) && CLS_ISCLASS(((id)anObject)->class_pointer))
-    {
-      NSZone	*z = [anObject zone];
-
-#ifndef	NDEBUG
-      GSDebugAllocationRemove(((id)anObject)->class_pointer, (id)anObject);
-#endif
-      if (NSZombieEnabled == YES)
-	{
-	  GSMakeZombie(anObject);
-	  if (NSDeallocateZombies == YES)
-	    {
-	      NSZoneFree(z, anObject);
-	    }
-	}
-      else
-	{
-	  ((id)anObject)->class_pointer = (void*) 0xdeadface;
-	  NSZoneFree(z, anObject);
-	}
-    }
-  return;
-}
-
-#endif	/* defined(REFCNT_LOCAL) || defined(CACHE_ZONE) */
-
 #endif	/* GS_WITH_GC */
 
+
+void
+GSPrivateSwizzle(id o, Class c)
+{
+  if (o->class_pointer != c)
+    {
+#if	GS_WITH_GC
+      /* We only do allocation counting for objects that can be
+       * finalised - for other objects we have no way of decrementing
+       * the count when the object is collected.
+       */
+      if (GSIsFinalizable(o->class_pointer))
+	{
+	  /* Already finalizable, so we just need to do any allocation
+	   * accounting.
+	   */
+          AREM(o->class_pointer, o);
+          AADD(c, o);
+	}
+      else if (GSIsFinalizable(c))
+	{
+	  /* New clas is finalizable, so we must register the instance
+	   * for finalisation and do allocation acounting for it.
+	   */
+	  AADD(c, o);
+	  GC_REGISTER_FINALIZER (o, GSFinalize, NULL, NULL, NULL);
+	}
+#endif	/* GS_WITH_GC */
+      o->class_pointer = c;
+    }
+}
+
+
 BOOL
 NSShouldRetainWithZone (NSObject *anObject, NSZone *requestedZone)
 {
@@ -1046,8 +840,8 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 {
   if (allocationLock == 0)
     {
-#if defined(REFCNT_LOCAL) && !defined(GSATOMICREAD)
-      unsigned	i;
+#if !defined(GSATOMICREAD)
+      NSUInteger	i;
 
       for (i = 0; i < LOCKCOUNT; i++)
         {
@@ -1059,12 +853,16 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 }
 
 #if	GS_WITH_GC
-/**
- * A utility method used when garbage collection is enabled.  Can be ignored.
+/* Function to log Boehm GC warnings
+ * NB. This must not allocate any collectable memory as it may result
+ * in a deadlock in the garbage collecting library.
  */
-+ (BOOL) requiresTypedMemory
+static void
+GSGarbageCollectorLog(char *msg, GC_word arg)
 {
-  return NO;
+  char	buf[strlen(msg)+1024];
+  sprintf(buf, msg, (unsigned long)arg);
+  fprintf(stderr, "Garbage collector: %s", buf);
 }
 #endif
 
@@ -1079,11 +877,21 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 {
   if (self == [NSObject class])
     {
+#if	GS_WITH_GC
+      /* Make sure that the garbage collection library is initialised.
+       * This is not necessary on most platforms, but is good practice.
+       */
+      GC_init();
+      GC_set_warn_proc(GSGarbageCollectorLog);
+#endif
+
 #ifdef __MINGW32__
-      // See libgnustep-base-entry.m
-      extern void gnustep_base_socket_init(void);	
-      gnustep_base_socket_init();	
-#else
+      {
+        // See libgnustep-base-entry.m
+        extern void gnustep_base_socket_init(void);	
+        gnustep_base_socket_init();	
+      }
+#else /* __MINGW32__ */
 
 #ifdef	SIGPIPE
     /*
@@ -1113,9 +921,9 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 	    fprintf(stderr, "Unable to retrieve information about SIGPIPE\n");
 	  }
       }
-#else
+#else /* HAVE_SIGACTION */
       {
-	void	(*handler)(int);
+	void	(*handler)(NSInteger);
 
 	handler = signal(SIGPIPE, SIG_IGN);
 	if (handler != SIG_DFL)
@@ -1123,8 +931,13 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 	    signal(SIGPIPE, handler);
 	  }
       }
-#endif
-#endif
+#endif /* HAVE_SIGACTION */
+#endif /* SIGPIPE */
+#endif /* __MINGW32__ */
+
+#if	GS_WITH_GC
+      finalize_sel = @selector(finalize);
+      finalize_imp = get_imp(self, finalize_sel);
 #endif
 
 #if defined(__FreeBSD__) && defined(__i386__)
@@ -1159,12 +972,6 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
       autorelease_class = [NSAutoreleasePool class];
       autorelease_sel = @selector(addObject:);
       autorelease_imp = [autorelease_class methodForSelector: autorelease_sel];
-#if	GS_WITH_GC == 0
-#if	!defined(REFCNT_LOCAL)
-      GSIMapInitWithZoneAndCapacity(&retain_counts,
-	NSDefaultMallocZone(), 1024);
-#endif
-#endif
       NSConstantStringClass = [NSString constantStringClass];
       GSPrivateBuildStrings();
       [[NSNotificationCenter defaultCenter]
@@ -1424,6 +1231,11 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
   NSDeallocateObject (self);
 }
 
+- (void) finalize
+{
+  return;
+}
+
 /**
  *  This method is an anachronism.  Do not use it.
  */
@@ -1505,7 +1317,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
   for (proto_list = ((struct objc_class*)self)->protocols;
        proto_list; proto_list = proto_list->next)
     {
-      unsigned int i;
+      NSUInteger i;
 
       for (i = 0; i < proto_list->count; i++)
 	{
@@ -1635,7 +1447,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 
       while (found == NO && protocols != 0)
 	{
-	  unsigned	i = 0;
+	  NSUInteger	i = 0;
 
 	  while (found == NO && i < protocols->count)
 	    {
@@ -1821,8 +1633,8 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 #if	GS_WITH_GC == 0
   if (double_release_check_enabled)
     {
-      unsigned release_count;
-      unsigned retain_count = [self retainCount];
+      NSUInteger release_count;
+      NSUInteger retain_count = [self retainCount];
       release_count = [autorelease_class autoreleaseCountForObject:self];
       if (release_count > retain_count)
         [NSException
@@ -1860,7 +1672,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
  * The default implementation returns a value based on the address
  * of the instance.
  */
-- (unsigned) hash
+- (NSUInteger) hash
 {
   /*
    * Ideally we would shift left to lose any zero bits produced by the
@@ -1869,7 +1681,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
    * In the absence of detailed information, pick a reasonable value
    * assuming the object will be aligned to an eight byte boundary.
    */
-  return (unsigned)(uintptr_t)self >> 3;
+  return (NSUInteger)(uintptr_t)self >> 3;
 }
 
 /**
@@ -2105,7 +1917,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
  * By convention, objects which should (or can) never be deallocated
  * return the maximum unsigned integer value.
  */
-- (unsigned) retainCount
+- (NSUInteger) retainCount
 {
 #if	GS_WITH_GC
   return UINT_MAX;
@@ -2119,7 +1931,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
  * the maximum unsigned integer value, as classes can not be deallocated
  * the retain count mechanism is a dummy system for them.
  */
-+ (unsigned) retainCount
++ (NSUInteger) retainCount
 {
   return UINT_MAX;
 }
@@ -2165,7 +1977,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
  *  a number assigned by the Objective C compiler if [NSObject -setVersion] has
  *  not been called.
  */
-+ (int) version
++ (NSInteger) version
 {
   return class_get_version(self);
 }
@@ -2173,7 +1985,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 /**
  * Sets the version number of the receiving class.  Should be nonnegative.
  */
-+ (id) setVersion: (int)aVersion
++ (id) setVersion: (NSInteger)aVersion
 {
   if (aVersion < 0)
     [NSException raise: NSInvalidArgumentException
@@ -2384,13 +2196,13 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
  * level information.
  */
 - (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
-			     indent: (unsigned)level
+			     indent: (NSUInteger)level
 {
   return [self descriptionWithLocale: aLocale];
 }
 
 + (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
-			     indent: (unsigned)level
+			     indent: (NSUInteger)level
 {
   return [self descriptionWithLocale: aLocale];
 }
@@ -2486,7 +2298,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
   return 0;
 }
 
-+ (int) streamVersion: (TypedStream*)aStream
++ (NSInteger) streamVersion: (TypedStream*)aStream
 {
 #ifndef NeXT_RUNTIME
   if (aStream->mode == OBJC_READONLY)
@@ -2608,7 +2420,7 @@ GSDescriptionForClassMethod(pcl self, SEL aSel)
 }
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
-  unsigned	size = [[anInvocation methodSignature] methodReturnLength];
+  NSUInteger	size = [[anInvocation methodSignature] methodReturnLength];
   unsigned char	v[size];
 
   memset(v, '\0', size);
