@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPathUtilities function reference</title>
-   $Date: 2008-12-19 11:50:32 +0000 (Fri, 19 Dec 2008) $ $Revision: 27342 $
+   $Date: 2009-03-18 04:02:49 -0600 (Wed, 18 Mar 2009) $ $Revision: 28092 $
    */
 
 /**
@@ -78,6 +78,8 @@
 
 #include "GSPrivate.h"
 
+#include "pathconfig/pathconfig.h"
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>		// for getuid()
 #endif
@@ -720,13 +722,13 @@ GNUstepConfig(NSDictionary *newConfig)
                             "The file '%S' is writable by someone other than"
                             " its owner (permissions 0%lo).\nIgnoring it.\n",
                             [defs fileSystemRepresentation],
-                            [attributes filePosixPermissions]);
+                            (long)[attributes filePosixPermissions]);
 #else
                           fprintf(stderr,
                             "The file '%s' is writable by someone other than"
                             " its owner (permissions 0%lo).\nIgnoring it.\n",
                             [defs fileSystemRepresentation],
-                            [attributes filePosixPermissions]);
+                            (long)[attributes filePosixPermissions]);
 #endif
                           d = nil;
                         }
@@ -1032,12 +1034,12 @@ ParseConfigurationFile(NSString *fileName, NSMutableDictionary *dict,
       fprintf(stderr, "The file '%S' is writable by someone other than"
 	" its owner (permissions 0%lo).\nIgnoring it.\n",
 	[fileName fileSystemRepresentation],
-        [attributes filePosixPermissions]);
+        (long)[attributes filePosixPermissions]);
 #else
       fprintf(stderr, "The file '%s' is writable by someone other than"
 	" its owner (permissions 0%lo).\nIgnoring it.\n",
 	[fileName fileSystemRepresentation],
-        [attributes filePosixPermissions]);
+        (long)[attributes filePosixPermissions]);
 #endif
       return NO;
     }
@@ -2114,6 +2116,6 @@ if (domainMask & mask) \
         }
     }
 
-  AUTORELEASE (paths);
+  IF_NO_GC(AUTORELEASE (paths);)
   return paths;
 }
