@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPort class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-08 08:34:16 -0600 (Sun, 08 Mar 2009) $ $Revision: 28051 $
    */
 
 #include "config.h"
@@ -81,6 +81,8 @@ static Class	NSPort_concrete_class;
     {
       NSUserDefaults	*defs;
 
+      GSMakeWeakPointer(self, "delegate");
+
       NSPort_abstract_class = self;
       NSPort_concrete_class = [NSMessagePort class];
 
@@ -101,7 +103,7 @@ static Class	NSPort_concrete_class;
     return AUTORELEASE([self new]);
 }
 
-+ (NSPort*) portWithMachPort: (int)machPort
++ (NSPort*) portWithMachPort: (NSInteger)machPort
 {
   return AUTORELEASE([[self alloc] initWithMachPort: machPort]);
 }
@@ -139,7 +141,7 @@ static Class	NSPort_concrete_class;
   return self;
 }
 
-- (id) initWithMachPort: (int)machPort
+- (id) initWithMachPort: (NSInteger)machPort
 {
   [self shouldNotImplement: _cmd];
   return nil;
@@ -165,7 +167,7 @@ static Class	NSPort_concrete_class;
   return _is_valid;
 }
 
-- (int) machPort
+- (NSInteger) machPort
 {
   [self shouldNotImplement: _cmd];
   return 0;
@@ -219,7 +221,7 @@ static Class	NSPort_concrete_class;
   [aLoop removePort: self forMode: aMode];
 }
 
-- (unsigned) reservedSpaceLength
+- (NSUInteger) reservedSpaceLength
 {
   [self subclassResponsibility: _cmd];
   return 0;
@@ -228,7 +230,7 @@ static Class	NSPort_concrete_class;
 - (BOOL) sendBeforeDate: (NSDate*)when
              components: (NSMutableArray*)components
                    from: (NSPort*)receivingPort
-               reserved: (unsigned) length
+               reserved: (NSUInteger)length
 {
   return [self sendBeforeDate: when
 			msgid: 0
@@ -238,10 +240,10 @@ static Class	NSPort_concrete_class;
 }
 
 - (BOOL) sendBeforeDate: (NSDate*)when
-		  msgid: (int)msgid
+		  msgid: (NSInteger)msgid
              components: (NSMutableArray*)components
                    from: (NSPort*)receivingPort
-               reserved: (unsigned)length
+               reserved: (NSUInteger)length
 {
   [self subclassResponsibility: _cmd];
   return YES;
