@@ -30,7 +30,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPortCoder class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-18 04:02:49 -0600 (Wed, 18 Mar 2009) $ $Revision: 28092 $
    */
 
 #include "config.h"
@@ -61,6 +61,13 @@
 #define	GSI_MAP_EQUAL(M, X,Y)	((X).ptr == (Y).ptr)
 #define	GSI_MAP_NOCLEAN	1
 
+#if	GS_WITH_GC
+#include	<gc_typed.h>
+static GC_descr	nodeDesc;	// Type descriptor for map node.
+#define	GSI_MAP_NODES(M, X) \
+(GSIMapNode)GC_calloc_explicitly_typed(X, sizeof(GSIMapNode_t), nodeDesc)
+#endif
+
 #include "GNUstepBase/GSIMap.h"
 
 /*
@@ -260,6 +267,12 @@ typeCheck(char t1, char t2)
 #endif
 	)) return;
 
+/* HACK also allow float and double to be used interchangably as MacOS-X
+ * intorduced CGFloat, which may be aither a float or a double.
+ */
+      if ((c == _C_FLT || c == _C_DBL) && (t1 == _C_FLT || t1 == _C_DBL))
+	return;
+
       [NSException raise: NSInternalInconsistencyException
 		  format: @"expected %s and got %s",
 		    typeToName1(t1), typeToName2(t2)];
@@ -354,6 +367,14 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
       _eTagImp = [mutableDataClass instanceMethodForSelector: eTagSel];
       _xRefImp = [mutableDataClass instanceMethodForSelector: xRefSel];
       mutableDictionaryClass = [NSMutableDictionary class];
+#if	GS_WITH_GC
+      /* We create a typed memory descriptor for map nodes.
+       */
+      GC_word	w[GC_BITMAP_SIZE(GSIMapNode_t)] = {0};
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, key));
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, value));
+      nodeDesc = GC_make_descriptor(w, GC_WORD_LEN(GSIMapNode_t));
+#endif
     }
 }
 
@@ -365,7 +386,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 
   coder = [self allocWithZone: NSDefaultMallocZone()];
   coder = [coder initWithReceivePort: recv sendPort: send components: comp];
-  AUTORELEASE(coder);
+  IF_NO_GC(AUTORELEASE(coder);)
   return coder;
 }
 
@@ -923,12 +944,36 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 #endif
       case _GSC_FLT:
 	typeCheck(*type, _GSC_FLT);
-	(*_dDesImp)(_src, dDesSel, address, type, &_cursor, nil);
+	if (*type == _C_FLT)
+	  {
+	    (*_dDesImp)(_src, dDesSel, address, type, &_cursor, nil);
+	  }
+	else
+	  {
+	    float	val;
+
+	    /* We found a float when expecting a double ... handle it.
+	     */
+	    (*_dDesImp)(_src, dDesSel, &val, @encode(float), &_cursor, nil);
+	    *(double*)address = (double)val;
+	  }
 	return;
 
       case _GSC_DBL:
 	typeCheck(*type, _GSC_DBL);
-	(*_dDesImp)(_src, dDesSel, address, type, &_cursor, nil);
+	if (*type == _C_DBL)
+	  {
+	    (*_dDesImp)(_src, dDesSel, address, type, &_cursor, nil);
+	  }
+	else
+	  {
+	    double	val;
+
+	    /* We found a double when expecting a float ... handle it.
+	     */
+	    (*_dDesImp)(_src, dDesSel, &val, @encode(double), &_cursor, nil);
+	    *(float*)address = (float)val;
+	  }
 	return;
 
       default:
@@ -1712,8 +1757,14 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	      /*
 	       *	Set up map tables.
 	       */
+#if	GS_WITH_GC
+	      _clsMap
+		= (GSIMapTable)NSAllocateCollectable(sizeof(GSIMapTable_t)*4,
+		NSScannedOption);
+#else
 	      _clsMap
 		= (GSIMapTable)NSZoneMalloc(_zone, sizeof(GSIMapTable_t)*4);
+#endif
 	      _cIdMap = &_clsMap[1];
 	      _uIdMap = &_clsMap[2];
 	      _ptrMap = &_clsMap[3];
@@ -1799,7 +1850,12 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	   */
 	  if (firstTime == YES)
 	    {
+#if	GS_WITH_GC
+	      _clsAry
+		= NSAllocateCollectable(sizeof(GSIArray_t)*3, NSScannedOption);
+#else
 	      _clsAry = NSZoneMalloc(_zone, sizeof(GSIArray_t)*3);
+#endif
 	      _objAry = &_clsAry[1];
 	      _ptrAry = &_clsAry[2];
 	      GSIArrayInitWithZoneAndCapacity(_clsAry, _zone, sizeC);
@@ -1858,7 +1914,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
   return _version;
 }
 
-- (unsigned) versionForClassName: (NSString*)className
+- (NSInteger) versionForClassName: (NSString*)className
 {
   GSClassInfo	*info = nil;
   unsigned	version = NSNotFound;
