@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProcessInfo class reference</title>
-   $Date: 2008-11-18 09:41:38 +0000 (Tue, 18 Nov 2008) $ $Revision: 27088 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
 */
 
 /*************************************************************************
@@ -572,7 +572,7 @@ static char	**_gnu_noobjc_env = NULL;
   
   // Read commandline
   proc_file_name = (char*)objc_malloc(sizeof(char) * 2048);
-  sprintf(proc_file_name, "/proc/%d/psinfo", (int) getpid());
+  sprintf(proc_file_name, "/proc/%d/psinfo", (int)getpid());
   
   ifp = fopen(proc_file_name, "r");
   if (ifp == NULL)
@@ -681,7 +681,7 @@ static char	**_gnu_noobjc_env = NULL;
 
   // Read commandline
   proc_file_name = (char *)objc_malloc(sizeof(char) * 2048);
-  sprintf(proc_file_name, "/proc/%d/cmdline", (int) getpid());
+  sprintf(proc_file_name, "/proc/%d/cmdline", (int)getpid());
 
   /*
    * We read the /proc file thrice.
@@ -991,7 +991,7 @@ int main(int argc, char *argv[], char *env[])
       pid = [self processIdentifier];
       start = (unsigned long)GSTimeNow();
       host = [[self hostName] stringByReplacingString: @"." withString: @"_"];
-      RETAIN(host);
+      IF_NO_GC(RETAIN(host);)
     }
   count = counter++;
   [gnustep_global_lock unlock];
@@ -1147,7 +1147,7 @@ static void determineOperatingSystem()
     }
 }
 
-- (unsigned int) operatingSystem
+- (NSUInteger) operatingSystem
 {
   if (_operatingSystem == 0)
     {
