@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProtocolChecker class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-01-12 05:48:46 -0700 (Mon, 12 Jan 2009) $ $Revision: 27578 $
    */
 
 #include "config.h"
@@ -203,7 +203,7 @@
       if (sig == nil)
 	{
 	  sig = [NSMethodSignature signatureWithObjCTypes: "@@::"];
-	  RETAIN(sig);
+	  IF_NO_GC(RETAIN(sig);)
 	}
       return sig;
     }
