@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSRunLoop class reference</title>
-   $Date: 2008-11-17 13:45:32 +0000 (Mon, 17 Nov 2008) $ $Revision: 27080 $
+   $Date: 2009-03-10 05:30:16 -0600 (Tue, 10 Mar 2009) $ $Revision: 28062 $
 */
 
 #include "config.h"
@@ -72,7 +72,7 @@ NSString * const NSDefaultRunLoopMode = @"NSDefaultRunLoopMode";
 static NSDate	*theFuture = nil;
 
 @interface NSObject (OptionalPortRunLoop)
-- (void) getFds: (int*)fds count: (int*)count;
+- (void) getFds: (NSInteger*)fds count: (NSInteger*)count;
 @end
 
 
@@ -95,7 +95,7 @@ static NSDate	*theFuture = nil;
 - (id) initWithSelector: (SEL)aSelector
 		 target: (id)target
 	       argument: (id)argument
-		  order: (unsigned int)order;
+		  order: (NSUInteger)order;
 @end
 
 @implementation GSRunLoopPerformer
@@ -115,7 +115,7 @@ static NSDate	*theFuture = nil;
 - (id) initWithSelector: (SEL)aSelector
 		 target: (id)aTarget
 	       argument: (id)anArgument
-		  order: (unsigned int)theOrder
+		  order: (NSUInteger)theOrder
 {
   self = [super init];
   if (self)
@@ -147,7 +147,7 @@ static NSDate	*theFuture = nil;
  * The GSTimedPerformer class is used to hold information about
  * messages which are due to be sent to objects at a particular time.
  */
-@interface GSTimedPerformer: NSObject <GCFinalization>
+@interface GSTimedPerformer: NSObject
 {
 @public
   SEL		selector;
@@ -168,7 +168,7 @@ static NSDate	*theFuture = nil;
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   TEST_RELEASE(timer);
   RELEASE(target);
   RELEASE(argument);
@@ -183,7 +183,7 @@ static NSDate	*theFuture = nil;
     removeObjectIdenticalTo: self];
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   [self invalidate];
 }
@@ -746,29 +746,26 @@ static inline BOOL timerInvalidated(NSTimer *t)
 					 NSObjectMapValueCallBacks, 0);
       _timedPerformers = [[NSMutableArray alloc] initWithCapacity: 8];
 #ifdef	HAVE_POLL_F
-      _extra = objc_malloc(sizeof(pollextra));
+#if	GS_WITH_GC
+      _extra = NSAllocateCollectable(sizeof(pollextra), NSScannedOption);
+#else
+      _extra = NSZoneMalloc(NSDefaultMallocZone(), sizeof(pollextra));
       memset(_extra, '\0', sizeof(pollextra));
+#endif
 #endif
     }
   return self;
 }
 
 - (void) dealloc
-{
-  [self gcFinalize];
-  [super dealloc];
-}
-
-- (void) gcFinalize
 {
 #ifdef	HAVE_POLL_F
   if (_extra != 0)
     {
       pollextra	*e = (pollextra*)_extra;
-
       if (e->index != 0)
-	objc_free(e->index);
-      objc_free(e);
+	NSZoneFree(NSDefaultMallocZone(), e->index);
+      NSZoneFree(NSDefaultMallocZone(), e);
     }
 #endif
   RELEASE(_contextStack);
@@ -777,6 +774,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
       NSFreeMapTable(_contextMap);
     }
   RELEASE(_timedPerformers);
+  [super dealloc];
 }
 
 /**
@@ -1047,11 +1045,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
 
       RELEASE(arp);
 
-      if (when != nil)
-	{
-	  AUTORELEASE(when);
-	}
-      else
+      if (when == nil)
         {
 	  GSIArray		watchers = context->watchers;
 	  unsigned		i = GSIArrayCount(watchers);
@@ -1070,6 +1064,12 @@ static inline BOOL timerInvalidated(NSTimer *t)
 	      when = theFuture;
 	    }
 	}
+#if	!GS_WITH_GC
+      else
+	{
+	  AUTORELEASE(when);
+	}
+#endif
 
       NSDebugMLLog(@"NSRunLoop", @"limit date %f in %@",
 	[when timeIntervalSinceReferenceDate], mode);
@@ -1083,7 +1083,8 @@ static inline BOOL timerInvalidated(NSTimer *t)
  * just poll inputs and return,
  * otherwise block until input is available or until the
  * earliest limit date has passed (whichever comes first).<br />
- * If the supplied mode is nil, uses NSDefaultRunLoopMode.
+ * If the supplied mode is nil, uses NSDefaultRunLoopMode.<br />
+ * If there are no input sources in the mode, returns immediately.
  */
 - (void) acceptInputForMode: (NSString*)mode
 		 beforeDate: (NSDate*)limit_date
@@ -1106,9 +1107,6 @@ static inline BOOL timerInvalidated(NSTimer *t)
 
   NS_DURING
     {
-      GSIArray		watchers;
-      unsigned		i;
-
       /*
        * If we have a housekeeping timer, and it is earlier than the
        * limit date we have been given, we use the date of the housekeeper
@@ -1121,17 +1119,15 @@ static inline BOOL timerInvalidated(NSTimer *t)
 	  limit_date = timerDate(context->housekeeper);
 	}
 
-      if ((context == nil || (watchers = context->watchers) == 0
-	|| (i = GSIArrayCount(watchers)) == 0))
+      if (context == nil || GSIArrayCount(context->watchers) == 0)
 	{
 	  NSDebugMLLog(@"NSRunLoop", @"no inputs in mode %@", mode);
 	  GSPrivateNotifyASAP();
 	  GSPrivateNotifyIdle();
-	  /*
-	   * Pause for as long as possible (up to the limit date)
+	  /* Pause until the limit date or until we might have
+	   * a method to perform in this thread.
 	   */
-	  [NSThread sleepUntilDate: limit_date];
-	  ti = [limit_date timeIntervalSinceNow];
+          [GSRunLoopCtxt awakenedBefore: nil];
 	  GSPrivateCheckTasks();
 	  if (context != nil)
 	    {
@@ -1214,6 +1210,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
 - (BOOL) runMode: (NSString*)mode beforeDate: (NSDate*)date
 {
   CREATE_AUTORELEASE_POOL(arp);
+  GSRunLoopCtxt	*context;
   NSDate	*d;
 
   NSAssert(mode != nil, NSInvalidArgumentException);
@@ -1222,7 +1219,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
   d = [self limitDateForMode: mode];
   if (d == nil)
     {
-      NSDebugMLLog(@"NSRunLoop", @"run mode with nothing to don %@", mode);
+      NSDebugMLLog(@"NSRunLoop", @"run mode with nothing to do %@", mode);
       /*
        * Notify if any tasks have completed.
        */
@@ -1234,6 +1231,15 @@ static inline BOOL timerInvalidated(NSTimer *t)
       return NO;
     }
 
+  /* Check to see if we have any input sources.
+   */
+  context = NSMapGet(_contextMap, mode);
+  if (context == nil || GSIArrayCount(context->watchers) == 0)
+    {
+      RELEASE(arp);
+      return NO;
+    }
+
   /*
    * Use the earlier of the two dates we have.
    * Retain the date in case the firing of a timer (or some other event)
@@ -1400,7 +1406,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
 - (void) performSelector: (SEL)aSelector
 		  target: (id)target
 		argument: (id)argument
-		   order: (unsigned int)order
+		   order: (NSUInteger)order
 		   modes: (NSArray*)modes
 {
   unsigned		count = [modes count];
