@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSSerializer class reference</title>
-   $Date: 2008-07-06 10:18:30 +0100 (Sun, 06 Jul 2008) $ $Revision: 26747 $
+   $Date: 2009-03-09 09:11:51 -0600 (Mon, 09 Mar 2009) $ $Revision: 28054 $
    */
 
 #include "config.h"
@@ -63,6 +63,11 @@
 #define	GSI_MAP_EQUAL(M, X,Y)	[(X).obj isEqualToString: (Y).obj]
 #define	GSI_MAP_NOCLEAN	1
 
+#if	GS_WITH_GC
+#define	GSI_MAP_NODES(M, X) \
+(GSIMapNode)NSAllocateCollectable(X * sizeof(GSIMapNode_t), 0)
+#endif
+
 #include "GNUstepBase/GSIMap.h"
 
 /*
@@ -555,7 +560,11 @@ deserializeFromInfo(_NSDeserializerInfo* info)
 	  char		*b;
 	
 	  size = (*info->deiImp)(info->data, deiSel, info->cursor);
+#if	GS_WITH_GC
+	  b = NSAllocateCollectable(size, 0);
+#else
 	  b = NSZoneMalloc(NSDefaultMallocZone(), size);
+#endif
 	  (*info->debImp)(info->data, debSel, b, size, info->cursor);
 	  s = [[StringClass alloc] initWithBytesNoCopy: b
 						length: size - 1
@@ -586,7 +595,11 @@ deserializeFromInfo(_NSDeserializerInfo* info)
 	  unichar	*b;
 	
 	  size = (*info->deiImp)(info->data, deiSel, info->cursor);
+#if	GS_WITH_GC
+	  b = NSAllocateCollectable(size*sizeof(unichar), 0);
+#else
 	  b = NSZoneMalloc(NSDefaultMallocZone(), size*sizeof(unichar));
+#endif
 	  (*info->debImp)(info->data, debSel, b, size*sizeof(unichar),
 	    info->cursor);
 	  s = [[StringClass alloc] initWithBytesNoCopy: b
@@ -684,7 +697,7 @@ deserializeFromInfo(_NSDeserializerInfo* info)
 			   * rather than simply releasing as the key may
 			   * be referred to by a cross-reference later.
 			   */
-			  AUTORELEASE(k);
+			  IF_NO_GC(AUTORELEASE(k);)
 			  RELEASE(o);
 			}
 		    }
