@@ -188,7 +188,7 @@ typedef enum {
   GS_H_CONNECTED	// Currently connected.
 } GSHandleState;
 
-@interface GSTcpHandle : NSObject <GCFinalization, RunLoopEvents>
+@interface GSTcpHandle : NSObject <RunLoopEvents>
 {
   SOCKET		desc;		/* File descriptor for I/O.	*/
   unsigned		wItem;		/* Index of item being written.	*/
@@ -676,7 +676,7 @@ static Class	runLoopClass;
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   DESTROY(defaultAddress);
   DESTROY(rData);
   DESTROY(rItems);
@@ -703,7 +703,7 @@ static Class	runLoopClass;
 }
 #endif
 
-- (void) gcFinalize
+- (void) finalize
 {
   [self invalidate];
   (void)close(desc);
@@ -1076,7 +1076,7 @@ static Class	runLoopClass;
           DESTROY(rItems);
           NSDebugMLLog(@"GSTcpHandle",
         	@"got message %@ on 0x%x", pm, self);
-          RETAIN(rp);
+          IF_NO_GC(RETAIN(rp);)
           M_UNLOCK(myLock);
           NS_DURING
             {
@@ -1227,9 +1227,8 @@ static Class	runLoopClass;
 {
 #if	defined(__MINGW32__)
   WSANETWORKEVENTS ocurredEvents;
+#else
 #endif
-  NSDebugMLLog(@"GSTcpHandle", @"received %s event on 0x%x",
-    type == ET_RPORT ? "read" : "write", self);
   /*
    * If we have been invalidated (desc < 0) then we should ignore this
    * event and remove ourself from the runloop.
@@ -1326,7 +1325,7 @@ static Class	runLoopClass;
       abort();      
     }
 #else
-  if (type == ET_RPORT)
+  if (type != ET_WDESC)
     {
       [self receivedEventRead];
     }
@@ -1353,7 +1352,7 @@ static Class	runLoopClass;
 
   l = [runLoopClass currentRunLoop];
 
-  RETAIN(self);
+  IF_NO_GC(RETAIN(self);)
 
 #if	defined(__MINGW32__)
   NSAssert(event != WSA_INVALID_EVENT, @"Socket without win32 event!");
@@ -1754,7 +1753,7 @@ static Class		tcpPortClass;
     }
   else
     {
-      RETAIN(port);
+      IF_NO_GC(RETAIN(port);)
       NSDebugMLLog(@"NSPort", @"Using pre-existing port: %@", port);
     }
   IF_NO_GC(AUTORELEASE(port));
@@ -1797,7 +1796,7 @@ static Class		tcpPortClass;
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   [super dealloc];
 }
 
@@ -1819,7 +1818,7 @@ static Class		tcpPortClass;
   return desc;
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   NSDebugMLLog(@"NSPort", @"NSSocketPort 0x%x finalized", self);
   [self invalidate];
@@ -1938,7 +1937,7 @@ static Class		tcpPortClass;
     {
       if ((NSPort*) [handle recvPort] == recvPort)
 	{
-	  RETAIN(handle);
+	  IF_NO_GC(RETAIN(handle);)
 	  NSEndMapTableEnumeration(&me);
 	  M_UNLOCK(myLock);
 	  return AUTORELEASE(handle);
@@ -2060,7 +2059,7 @@ static Class		tcpPortClass;
 {
   if ([self isValid] == YES)
     {
-      RETAIN(self);
+      IF_NO_GC(RETAIN(self);)
       M_LOCK(myLock);
 
       if ([self isValid] == YES)
@@ -2254,7 +2253,7 @@ static Class		tcpPortClass;
  */
 - (void) removeHandle: (GSTcpHandle*)handle
 {
-  RETAIN(self);
+  IF_NO_GC(RETAIN(self);)
   M_LOCK(myLock);
   if ([handle sendPort] == self)
     {
@@ -2267,7 +2266,7 @@ static Class		tcpPortClass;
 	   * been retained - we must therefore release this port since the
 	   * handle no longer uses it.
 	   */
-	  AUTORELEASE(self);
+	  IF_NO_GC(AUTORELEASE(self);)
 	}
       handle->sendPort = nil;
     }
