@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2008-12-07 06:02:02 +0000 (Sun, 07 Dec 2008) $ $Revision: 27240 $
+   $Date: 2009-01-12 05:48:46 -0700 (Mon, 12 Jan 2009) $ $Revision: 27578 $
    */
 
 #import "config.h"
@@ -1195,7 +1195,7 @@ typedef enum {
       NSMutableSet	*known = (NSMutableSet*)NSMapGet(_portMap, port);
       NSString		*name;
 
-      RETAIN(known);
+      IF_NO_GC(RETAIN(known);)
       while ((name = [known anyObject]) != nil)
 	{
 	  if ([self removePortForName: name] == NO)
