@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTask class reference</title>
-   $Date: 2008-11-17 13:45:32 +0000 (Mon, 17 Nov 2008) $ $Revision: 27080 $
+   $Date: 2009-01-12 11:36:37 -0700 (Mon, 12 Jan 2009) $ $Revision: 27581 $
    */
 
 #include "config.h"
@@ -285,7 +285,7 @@ pty_slave(const char* name)
   return AUTORELEASE(task);
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
   [tasksLock lock];
   NSMapRemove(activeTasks, (void*)(intptr_t)_taskId);
@@ -294,7 +294,7 @@ pty_slave(const char* name)
 
 - (void) dealloc
 {
-  [self gcFinalize];
+  [self finalize];
   RELEASE(_arguments);
   RELEASE(_environment);
   RELEASE(_launchPath);
@@ -954,9 +954,9 @@ GSPrivateCheckTasks()
   return found;
 }
 
-- (void) gcFinalize
+- (void) finalize
 {
-  [super gcFinalize];
+  [super finalize];
   if (wThread != NULL)
     {
       CloseHandle(wThread);
@@ -1278,7 +1278,7 @@ GSPrivateCheckTasks()
 
 	      [tasksLock lock];
 	      t = (NSTask*)NSMapGet(activeTasks, (void*)(intptr_t)result);
-	      AUTORELEASE(RETAIN(t));
+	      IF_NO_GC(AUTORELEASE(RETAIN(t));)
 	      [tasksLock unlock];
 	      if (t != nil)
 		{
