@@ -28,7 +28,7 @@
    Boston, MA 02111 USA.
 
    <title>NSThread class reference</title>
-   $Date: 2008-09-23 09:10:59 +0100 (Tue, 23 Sep 2008) $ $Revision: 26864 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
 */
 
 #include "config.h"
@@ -80,6 +80,10 @@
 #include "GSPrivate.h"
 #include "GSRunLoopCtxt.h"
 
+#if	GS_WITH_GC
+#include	<gc.h>
+#endif
+
 @interface NSAutoreleasePool (NSThread)
 + (void) _endThread: (NSThread*)thread;
 @end
@@ -203,12 +207,12 @@ GSSleepUntilIntervalSinceReferenceDate(NSTimeInterval when)
   while (delay > 0)
     {
 #ifdef	HAVE_USLEEP
-      usleep ((int)(delay*1000000));
+      usleep ((NSInteger)(delay*1000000));
 #else
 #if defined(__MINGW32__)
       Sleep (delay*1000);
 #else
-      sleep ((int)delay);
+      sleep ((NSInteger)delay);
 #endif
 #endif
       delay = when - GSTimeNow();
@@ -389,6 +393,13 @@ gnustep_base_thread_callback(void)
 	   * threaded BEFORE sending the notifications.
 	   */
 	  entered_multi_threaded_state = YES;
+#if	GS_WITH_GC && defined(HAVE_GC_ALLOW_REGISTER_THREADS)
+	  /* This function needs to be called before going multi-threaded
+	   * so that the garbage collection library knows to support
+	   * registration of new threads.
+	   */
+	  GS_allow_register_threads();
+#endif
 	  NS_DURING
 	    {
 	      [GSPerformHolder class];	// Force initialization
@@ -506,6 +517,9 @@ gnustep_base_thread_callback(void)
 
       objc_thread_set_data (NULL);
 
+#if	GS_WITH_GC && defined(HAVE_GC_REGISTER_MY_THREAD)
+      GC_unregister_my_thread();
+#endif
       /*
        * Tell the runtime to exit the thread
        */
@@ -734,6 +748,27 @@ gnustep_base_thread_callback(void)
         NSStringFromSelector(_cmd)];
     }
 
+#if	GS_WITH_GC && defined(HAVE_GC_REGISTER_MY_THREAD)
+  {
+    struct GC_stack_base	base;
+
+    if (GC_get_stack_base(&base) == GC_SUCCESS)
+      {
+	int	result;
+
+	result = GC_register_my_thread(&base);
+	if (result != GC_SUCCESS && result != GC_DUPLICATE)
+	  {
+	    fprintf(stderr, "Argh ... no thread support in garbage collection library\n");
+	  }
+      }
+    else
+      {
+	fprintf(stderr, "Unable to determine stack base to register new thread for garbage collection\n");
+      }
+  }
+#endif
+
 #if     defined(HAVE_SETRLIMIT) && defined(RLIMIT_STACK)
   if (_stackSize > 0)
     {
@@ -791,7 +826,7 @@ pthread_detach(pthread_self());
   ASSIGN(_name, aName);
 }
 
-- (void) setStackSize: (unsigned)stackSize
+- (void) setStackSize: (NSUInteger)stackSize
 {
   _stackSize = stackSize;
 #if     !defined(HAVE_SETRLIMIT) || !defined(RLIMIT_STACK)
@@ -799,7 +834,7 @@ pthread_detach(pthread_self());
 #endif
 }
 
-- (unsigned) stackSize
+- (NSUInteger) stackSize
 {
   return _stackSize;
 }
@@ -834,7 +869,7 @@ pthread_detach(pthread_self());
 
   /* The thread must persist until it finishes executing.
    */
-  RETAIN(self);
+  IF_NO_GC(RETAIN(self);)
 
   /* Mark the thread as active whiul it's running.
    */
