@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSURL class reference</title>
-   $Date: 2008-12-14 19:12:46 +0000 (Sun, 14 Dec 2008) $ $Revision: 27291 $
+   $Date: 2009-04-27 14:58:17 -0600 (Mon, 27 Apr 2009) $ $Revision: 28259 $
 */
 
 /*
@@ -46,6 +46,7 @@ function may be incorrect
 #import "Foundation/NSLock.h"
 #import "Foundation/NSMapTable.h"
 #import "Foundation/NSObject.h"
+#import "Foundation/NSPortCoder.h"
 #import "Foundation/NSRunLoop.h"
 #import "Foundation/NSString.h"
 #import "Foundation/NSURL.h"
@@ -142,7 +143,11 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
       len += strlen(rel->fragment) + 1;		// #fragment
     }
 
-  ptr = buf = (char*)NSZoneMalloc(GSAtomicMallocZone(), len);
+#if	GS_WITH_GC
+  ptr = buf = (char*)NSAllocateCollectable(len, 0);
+#else
+  ptr = buf = (char*)NSZoneMalloc(NSDefaultMallocZone(), len);
+#endif
 
   if (rel->scheme != 0)
     {
@@ -620,10 +625,15 @@ static unsigned	urlAlign;
  */
 - (id) initFileURLWithPath: (NSString*)aPath
 {
-  BOOL	flag = NO;
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  BOOL		flag = NO;
 
-  if ([[NSFileManager defaultManager] fileExistsAtPath: aPath
-    isDirectory: &flag] == YES)
+  if ([aPath isAbsolutePath] == NO)
+    {
+      aPath = [[mgr currentDirectoryPath]
+	stringByAppendingPathComponent: aPath];
+    }
+  if ([mgr fileExistsAtPath: aPath isDirectory: &flag] == YES)
     {
       if ([aPath isAbsolutePath] == NO)
 	{
@@ -635,7 +645,7 @@ static unsigned	urlAlign;
 	}
     }
   self = [self initWithScheme: NSURLFileScheme
-			 host: nil
+			 host: @"localhost"
 			 path: aPath];
   return self;
 }
@@ -693,7 +703,11 @@ static unsigned	urlAlign;
       BOOL	canBeGeneric = YES;
 
       size += sizeof(parsedURL) + urlAlign + 1;
-      buf = _data = (parsedURL*)NSZoneMalloc(GSAtomicMallocZone(), size);
+#if	GS_WITH_GC
+      buf = _data = (parsedURL*)NSAllocateCollectable(size, 0);
+#else
+      buf = _data = (parsedURL*)NSZoneMalloc(NSDefaultMallocZone(), size);
+#endif
       memset(buf, '\0', size);
       start = end = ptr = (char*)&buf[1];
       [_urlString getCString: start
@@ -1139,7 +1153,7 @@ static unsigned	urlAlign;
   [aCoder encodeObject: _baseURL];
 }
 
-- (unsigned int) hash
+- (NSUInteger) hash
 {
   return [[self absoluteString] hash];
 }
@@ -1436,6 +1450,13 @@ static unsigned	urlAlign;
 	}
 
       unescape(buf, buf);
+      /* Remove any trailing '/' from the path for MacOS-X compatibility.
+       */
+      tmp = buf + strlen(buf) - 1;
+      if (tmp > buf && *tmp == '/')
+	{
+	  *tmp = '\0';
+	}
       path = [NSString stringWithUTF8String: buf];
     }
   return path;
@@ -1515,6 +1536,15 @@ static unsigned	urlAlign;
   return _urlString;
 }
 
+/* Encode bycopy unless explicitly requested otherwise.
+ */
+- (id) replacementObjectForPortCoder: (NSPortCoder*)aCoder
+{
+  if ([aCoder isByref] == NO)
+    return self;
+  return [super replacementObjectForPortCoder: aCoder];
+}
+
 /**
  * Loads the resource data for the represented URL and returns the result.
  * The shouldUseCache flag determines whether data previously retrieved by
@@ -1651,7 +1681,7 @@ static unsigned	urlAlign;
       if (c != 0)
 	{
 	  handle = [[c alloc] initWithURL: self cached: shouldUseCache];
-	  AUTORELEASE(handle);
+	  IF_NO_GC([handle autorelease];)
 	}
     }
   return handle;
@@ -1734,7 +1764,7 @@ static unsigned	urlAlign;
 {
   id	c = clientForHandle(_clients, sender);
 
-  RETAIN(self);
+  IF_NO_GC([self retain];)
   [sender removeClient: self];
   if (c != nil)
     {
