@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSURLHandle class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-03-03 12:59:58 -0700 (Tue, 03 Mar 2009) $ $Revision: 28029 $
 */
 
 #import "GSURLPrivate.h"
@@ -252,10 +252,10 @@ static Class		NSURLHandleClass = 0;
 {
   id	o = client;
 
-  RETAIN(o);
+  IF_NO_GC([o retain];)
   [_clients removeObjectIdenticalTo: o];
   [_clients addObject: o];
-  RELEASE(o);
+  IF_NO_GC([o release];)
 }
 
 /**
@@ -312,12 +312,12 @@ static Class		NSURLHandleClass = 0;
  */
 - (void) cancelLoadInBackground
 {
-  RETAIN(self);
+  IF_NO_GC([self retain];)
   [_clients makeObjectsPerformSelector:
     @selector(URLHandleResourceDidCancelLoading:)
     withObject: self];
   [self endLoadInBackground];
-  RELEASE(self);
+  IF_NO_GC([self release];)
 }
 
 - (void) dealloc
@@ -631,7 +631,7 @@ static NSLock			*fileLock = nil;
       NS_DURING
 	{
 	  obj = [fileCache objectForKey: path];
-	  AUTORELEASE(RETAIN(obj));
+	  IF_NO_GC([[obj retain] autorelease];)
 	}
       NS_HANDLER
 	{
@@ -723,7 +723,7 @@ static NSLock			*fileLock = nil;
 	  if (obj != nil)
 	    {
 	      DESTROY(self);
-	      RETAIN(obj);
+	      IF_NO_GC([obj retain];)
 	    }
 	}
       NS_HANDLER
