@@ -1,8 +1,9 @@
 /* Implementation for NSURLProtocol for GNUstep
    Copyright (C) 2006 Software Foundation, Inc.
 
-   Written by:  Richard Frith-Macdonald <frm@gnu.org>
+   Written by:  Richard Frith-Macdonald <rfm@gnu.org>
    Date: 2006
+   Parts (FTP and About in particular) based on later code by Nikolaus Schaller
    
    This file is part of the GNUstep Base Library.
 
@@ -27,6 +28,7 @@
 #import <Foundation/NSNotification.h>
 #import <Foundation/NSRunLoop.h>
 #import <Foundation/NSValue.h>
+#import <Foundation/NSDebug.h>
 
 #import "GNUstepBase/GSMime.h"
 
@@ -128,10 +130,18 @@ static NSLock		*pairLock = nil;
 
 - (void) dealloc
 {
-  [ip release];
-  [op release];
-  [host release];
-  [expires release];
+  [ip setDelegate: nil];
+  [op setDelegate: nil];
+  [ip removeFromRunLoop: [NSRunLoop currentRunLoop]
+		forMode: NSDefaultRunLoopMode];
+  [op removeFromRunLoop: [NSRunLoop currentRunLoop]
+		forMode: NSDefaultRunLoopMode];
+  [ip close];
+  [op close];
+  DESTROY(ip);
+  DESTROY(op);
+  DESTROY(host);
+  DESTROY(expires);
   [super dealloc];
 }
 
@@ -241,6 +251,7 @@ static NSLock		*pairLock = nil;
   NSURLCredential		*_credential;
   NSHTTPURLResponse		*_response;
 }
+- (void) setDebug: (BOOL)flag;
 @end
 
 @interface _NSHTTPSURLProtocol : _NSHTTPURLProtocol
@@ -330,6 +341,28 @@ static NSURLProtocol	*placeholder = nil;
   return NO;
 }
 
++(Class) _classToHandleRequest:(NSURLRequest *)request
+{
+  Class protoClass = nil;
+  int count;
+  [regLock lock];
+
+  count = [registered count];
+  while (count-- > 0)
+    {
+      Class	proto = [registered objectAtIndex: count];
+
+      if ([proto canInitWithRequest: request] == YES)
+	{
+	  protoClass = proto;
+	  break;
+	}
+    }
+  [regLock unlock];
+  return protoClass;
+}
+
+
 + (void) setProperty: (id)value
 	      forKey: (NSString *)key
 	   inRequest: (NSMutableURLRequest *)request
@@ -364,10 +397,21 @@ static NSURLProtocol	*placeholder = nil;
   if (this != 0)
     {
       [self stopLoading];
-      RELEASE(this->input);
-      RELEASE(this->output);
-      RELEASE(this->cachedResponse);
-      RELEASE(this->request);
+      if (this->input != nil)
+	{
+	  [this->input setDelegate: nil];
+	  [this->output setDelegate: nil];
+	  [this->input removeFromRunLoop: [NSRunLoop currentRunLoop]
+				 forMode: NSDefaultRunLoopMode];
+	  [this->output removeFromRunLoop: [NSRunLoop currentRunLoop]
+				  forMode: NSDefaultRunLoopMode];
+          [this->input close];
+          [this->output close];
+          DESTROY(this->input);
+          DESTROY(this->output);
+	}
+      DESTROY(this->cachedResponse);
+      DESTROY(this->request);
 #if	USE_ZLIB
       if (this->compressing == YES)
 	{
@@ -377,7 +421,7 @@ static NSURLProtocol	*placeholder = nil;
 	{
 	  inflateEnd(&this->z);
 	}
-      RELEASE(this->compressed);
+      DESTROY(this->compressed);
 #endif
       NSZoneFree([self zone], this);
       _NSURLProtocolInternal = 0;
@@ -524,12 +568,9 @@ static NSURLProtocol	*placeholder = nil;
   [super dealloc];
 }
 
-- (void) _schedule
+- (void) setDebug: (BOOL)flag
 {
-  [this->input scheduleInRunLoop: [NSRunLoop currentRunLoop]
-			 forMode: NSDefaultRunLoopMode];
-  [this->output scheduleInRunLoop: [NSRunLoop currentRunLoop]
-			  forMode: NSDefaultRunLoopMode];
+  _debug = flag;
 }
 
 - (void) startLoading
@@ -568,7 +609,7 @@ static NSURLProtocol	*placeholder = nil;
   _statusCode = 0;	/* No status returned yet.	*/
   _isLoading = YES;
   _complete = NO;
-  _debug = NO;
+  _debug = GSDebugSet(@"NSHTTPURLProtocol");
 
   /* Perform a redirect if the path is empty.
    * As per MacOs-X documentation.
@@ -652,7 +693,8 @@ static NSURLProtocol	*placeholder = nil;
 	{
 	  if (_debug == YES)
 	    {
-	      NSLog(@"did not create streams for %@:%@", host, [url port]);
+	      NSLog(@"%@ did not create streams for %@:%@",
+		self, host, [url port]);
 	    }
 	  [self stopLoading];
 	  [this->client URLProtocol: self didFailWithError:
@@ -664,8 +706,10 @@ static NSURLProtocol	*placeholder = nil;
 		nil]]];
 	  return;
 	}
-      RETAIN(this->input);
-      RETAIN(this->output);
+#if	!GS_WITH_GC
+      [this->input retain];
+      [this->output retain];
+#endif
       if ([[url scheme] isEqualToString: @"https"] == YES)
         {
           [this->input setProperty: NSStreamSocketSecurityLevelNegotiatedSSL
@@ -675,31 +719,31 @@ static NSURLProtocol	*placeholder = nil;
         }
       [this->input setDelegate: self];
       [this->output setDelegate: self];
-      [self _schedule];
+      [this->input scheduleInRunLoop: [NSRunLoop currentRunLoop]
+			     forMode: NSDefaultRunLoopMode];
+      [this->output scheduleInRunLoop: [NSRunLoop currentRunLoop]
+			      forMode: NSDefaultRunLoopMode];
       [this->input open];
       [this->output open];
     }
 }
 
-- (void) _unschedule
-{
-  [this->input removeFromRunLoop: [NSRunLoop currentRunLoop]
-			 forMode: NSDefaultRunLoopMode];
-  [this->output removeFromRunLoop: [NSRunLoop currentRunLoop]
-			  forMode: NSDefaultRunLoopMode];
-}
-
 - (void) stopLoading
 {
   if (_debug == YES)
     {
-      NSLog(@"stopLoading: %@", self);
+      NSLog(@"%@ stopLoading", self);
     }
   _isLoading = NO;
   DESTROY(_writeData);
   if (this->input != nil)
     {
-      [self _unschedule];
+      [this->input setDelegate: nil];
+      [this->output setDelegate: nil];
+      [this->input removeFromRunLoop: [NSRunLoop currentRunLoop]
+			     forMode: NSDefaultRunLoopMode];
+      [this->output removeFromRunLoop: [NSRunLoop currentRunLoop]
+			      forMode: NSDefaultRunLoopMode];
       [this->input close];
       [this->output close];
       DESTROY(this->input);
@@ -729,7 +773,7 @@ static NSURLProtocol	*placeholder = nil;
 	  e = [stream streamError];
 	  if (_debug)
 	    {
-	      NSLog(@"receive error %@", e);
+	      NSLog(@"%@ receive error %@", self, e);
 	    }
 	  [self stopLoading];
 	  [this->client URLProtocol: self didFailWithError: e];
@@ -738,7 +782,8 @@ static NSURLProtocol	*placeholder = nil;
     }
   if (_debug)
     {
-      NSLog(@"Read %d bytes: '%*.*s'", readCount, readCount, readCount, buffer);
+      NSLog(@"%@ read %d bytes: '%*.*s'",
+	self, readCount, readCount, readCount, buffer);
     }
 
   if (_parser == nil)
@@ -752,7 +797,7 @@ static NSURLProtocol	*placeholder = nil;
     {
       if (_debug == YES)
 	{
-	  NSLog(@"HTTP parse failure - %@", _parser);
+	  NSLog(@"%@ HTTP parse failure - %@", self, _parser);
 	}
       e = [NSError errorWithDomain: @"parse error"
 			      code: 0
@@ -1063,15 +1108,23 @@ static NSURLProtocol	*placeholder = nil;
 		      [request setValue: auth
 			forHTTPHeaderField: @"Authorization"];
 		      [self stopLoading];
+		      [this->request release];
+		      this->request = request;
 		      DESTROY(this->cachedResponse);
 		      [self startLoading];
+		      return;
 		    }
 		}
 	    }
 
-	  [self _unschedule];
+	  [this->input removeFromRunLoop: [NSRunLoop currentRunLoop]
+				 forMode: NSDefaultRunLoopMode];
+	  [this->output removeFromRunLoop: [NSRunLoop currentRunLoop]
+				  forMode: NSDefaultRunLoopMode];
 	  if (_shouldClose == YES)
 	    {
+	      [this->input setDelegate: nil];
+	      [this->output setDelegate: nil];
 	      [this->input close];
 	      [this->output close];
 	      DESTROY(this->input);
@@ -1138,7 +1191,7 @@ static NSURLProtocol	*placeholder = nil;
 	   */
 	  if (_debug == YES)
 	    {
-	      NSLog(@"HTTP response not received - %@", _parser);
+	      NSLog(@"%@ HTTP response not received - %@", self, _parser);
 	    }
 	  [self stopLoading];
 	  [this->client URLProtocol: self didFailWithError:
@@ -1153,7 +1206,7 @@ static NSURLProtocol	*placeholder = nil;
 {
   /* Make sure no action triggered by anything else destroys us prematurely.
    */
-  AUTORELEASE(RETAIN(self));
+  IF_NO_GC([[self retain] autorelease];)
 
 #if 0
   NSLog(@"stream: %@ handleEvent: %x for: %@", stream, event, self);
@@ -1171,7 +1224,7 @@ static NSURLProtocol	*placeholder = nil;
 	  case NSStreamEventOpenCompleted: 
 	    if (_debug == YES)
 	      {
-		NSLog(@"HTTP input stream opened");
+		NSLog(@"%@ HTTP input stream opened", self);
 	      }
 	    return;
 
@@ -1194,7 +1247,7 @@ static NSURLProtocol	*placeholder = nil;
 
 	      if (_debug == YES)
 	        {
-	          NSLog(@"HTTP output stream opened");
+	          NSLog(@"%@ HTTP output stream opened", self);
 	        }
 	      DESTROY(_writeData);
 	      _writeOffset = 0;
@@ -1281,7 +1334,7 @@ static NSURLProtocol	*placeholder = nil;
 		    {
 		      if (_debug == YES)
 		        {
-			  NSLog(@"Wrote %d bytes: '%*.*s'", written,
+			  NSLog(@"%@ wrote %d bytes: '%*.*s'", self, written,
 			    written, written, bytes + _writeOffset);
 			}
 		      _writeOffset += written;
@@ -1322,8 +1375,8 @@ static NSURLProtocol	*placeholder = nil;
 			{
 			  if (_debug == YES)
 			    {
-			      NSLog(@"error reading from HTTPBody stream %@",
-				[NSError _last]);
+			      NSLog(@"%@ error reading from HTTPBody stream %@",
+				self, [NSError _last]);
 			    }
 			  [self stopLoading];
 			  [this->client URLProtocol: self didFailWithError:
@@ -1339,8 +1392,8 @@ static NSURLProtocol	*placeholder = nil;
 			    {
 			      if (_debug == YES)
 				{
-				  NSLog(@"Wrote %d bytes: '%*.*s'", written,
-				    written, written, buffer);
+				  NSLog(@"%@ wrote %d bytes: '%*.*s'", self,
+				    written, written, written, buffer);
 				}
 			      len -= written;
 			      if (len > 0)
@@ -1372,7 +1425,7 @@ static NSURLProtocol	*placeholder = nil;
 		{
 		  if (_debug)
 		    {
-		      NSLog(@"request sent");
+		      NSLog(@"%@ request sent", self);
 		    }
 		  if (_shouldClose == YES)
 		    {
@@ -1389,9 +1442,23 @@ static NSURLProtocol	*placeholder = nil;
 	    break;
 	}
     }
-  NSLog(@"An error %@ occurred on the event %08x of stream %@ of %@", [stream streamError], event, stream, self);
-  [self stopLoading];
-  [this->client URLProtocol: self didFailWithError: [stream streamError]];
+  else
+    {
+      NSLog(@"Unexpected event %d occurred on stream %@ not being used by %@",
+	event, stream, self);
+    }
+  if (event == NSStreamEventErrorOccurred)
+    {
+      NSLog(@"An error %@ occurred on stream %@ of %@",
+	[stream streamError], stream, self);
+      [self stopLoading];
+      [this->client URLProtocol: self didFailWithError: [stream streamError]];
+    }
+  else
+    {
+      NSLog(@"Unexpected event %d ignored on stream %@ of %@",
+	event, stream, self);
+    }
 }
 
 - (void) useCredential: (NSURLCredential*)credential
@@ -1451,8 +1518,10 @@ static NSURLProtocol	*placeholder = nil;
 			    userInfo: nil]];
 	  return;
 	}
-      RETAIN(this->input);
-      RETAIN(this->output);
+#if	!GS_WITH_GC
+      [this->input retain];
+      [this->output retain];
+#endif
       if ([[url scheme] isEqualToString: @"https"] == YES)
         {
           [this->input setProperty: NSStreamSocketSecurityLevelNegotiatedSSL
@@ -1476,6 +1545,8 @@ static NSURLProtocol	*placeholder = nil;
 {
   if (this->input)
     {
+      [this->input setDelegate: nil];
+      [this->output setDelegate: nil];
       [this->input removeFromRunLoop: [NSRunLoop currentRunLoop]
 			     forMode: NSDefaultRunLoopMode];
       [this->output removeFromRunLoop: [NSRunLoop currentRunLoop]
@@ -1517,9 +1588,23 @@ static NSURLProtocol	*placeholder = nil;
       NSLog(@"An event occurred on the output stream.");
   	// if successfully opened, send out FTP request header
     }
-  NSLog(@"An error %@ occurred on the event %08x of stream %@ of %@",
-    [stream streamError], event, stream, self);
-  [this->client URLProtocol: self didFailWithError: [stream streamError]];
+  else
+    {
+      NSLog(@"Unexpected event %d occurred on stream %@ not being used by %@",
+	event, stream, self);
+    }
+  if (event == NSStreamEventErrorOccurred)
+    {
+      NSLog(@"An error %@ occurred on stream %@ of %@",
+	[stream streamError], stream, self);
+      [self stopLoading];
+      [this->client URLProtocol: self didFailWithError: [stream streamError]];
+    }
+  else
+    {
+      NSLog(@"Unexpected event %d ignored on stream %@ of %@",
+	event, stream, self);
+    }
 }
 
 @end
