@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSUnarchiver class reference</title>
-   $Date: 2008-12-08 09:08:05 +0000 (Mon, 08 Dec 2008) $ $Revision: 27257 $
+   $Date: 2009-02-22 10:07:36 -0700 (Sun, 22 Feb 2009) $ $Revision: 27949 $
    */
 
 #import "config.h"
@@ -220,6 +220,11 @@ typeCheck(char t1, char t2)
 	|| t1 == _C_ULNG_LNG
 #endif
 	)) return;
+/* HACK also allow float and double to be used interchangably as MacOS-X
+ * intorduced CGFloat, which may be aither a float or a double.
+ */
+      if ((c == _C_FLT || c == _C_DBL) && (t1 == _C_FLT || t1 == _C_DBL))
+	return;
 
       [NSException raise: NSInternalInconsistencyException
 		  format: @"expected %s and got %s",
@@ -756,8 +761,7 @@ static Class NSDataMallocClass;
 		   */
 		  if (c == nil)
 		    {
-		      NSLog(@"Got nil when trying to unarchive class %s",
-			className);
+		      NSLog(@"Unable to find class named '%@'", className);
 		    }
 		  [classInfo mapToClass: c withName: className];
 		  [objDict setObject: classInfo forKey: className];
@@ -991,12 +995,36 @@ static Class NSDataMallocClass;
 #endif
       case _GSC_FLT:
 	typeCheck(*type, _GSC_FLT);
-	(*desImp)(src, desSel, address, type, &cursor, nil);
+	if (*type == _C_FLT)
+	  {
+	    (*desImp)(src, desSel, address, type, &cursor, nil);
+	  }
+	else
+	  {
+	    float	val;
+
+	    /* We found a float when expecting a double ... handle it.
+	     */
+	    (*desImp)(src, desSel, &val, @encode(float), &cursor, nil);
+	    *(double*)address = (double)val;
+	  }
 	return;
 
       case _GSC_DBL:
 	typeCheck(*type, _GSC_DBL);
-	(*desImp)(src, desSel, address, type, &cursor, nil);
+	if (*type == _C_DBL)
+	  {
+	    (*desImp)(src, desSel, address, type, &cursor, nil);
+	  }
+	else
+	  {
+	    double	val;
+
+	    /* We found a double when expecting a float ... handle it.
+	     */
+	    (*desImp)(src, desSel, &val, @encode(double), &cursor, nil);
+	    *(float*)address = (float)val;
+	  }
 	return;
 
       default:
@@ -1117,14 +1145,12 @@ static Class NSDataMallocClass;
 	{
 	  void		*b;
 	  NSData	*d;
-	  NSZone	*z;
 
 #if	GS_WITH_GC
-	  z = GSAtomicMallocZone();
+	  b = NSAllocateCollectable(l, 0);
 #else
-	  z = zone;
+	  b = NSZoneMalloc(zone, l);
 #endif
-	  b = NSZoneMalloc(z, l);
 	  [self decodeArrayOfObjCType: @encode(unsigned char)
 				count: l
 				   at: b];
