@@ -21,7 +21,7 @@
    Boston, MA 02111 USA.
 
    <title>NSUndoManager class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-02-25 08:56:51 -0700 (Wed, 25 Feb 2009) $ $Revision: 27980 $
 */
 
 #include "config.h"
@@ -156,10 +156,39 @@
  *	Private category for the method used to handle default grouping
  */
 @interface NSUndoManager (Private)
+- (void) _begin;
 - (void) _loop: (id)arg;
 @end
 
 @implementation NSUndoManager (Private)
+/* This method is used to begin undo grouping internally.
+ * It's necessary to have a different mechanism from the -beginUndoGroup
+ * because it seems that in MacOS-X 10.5 a call to -beginUndoGroup when
+ * at the top level will actually create two undo groups.
+ */
+- (void) _begin
+{
+  PrivateUndoGroup	*parent;
+
+  parent = (PrivateUndoGroup*)_group;
+  _group = [[PrivateUndoGroup alloc] initWithParent: parent];
+  if (_group == nil)
+    {
+      _group = parent;
+      [NSException raise: NSInternalInconsistencyException
+		  format: @"beginUndoGrouping failed to greate group"];
+    }
+  else
+    {
+      RELEASE(parent);
+
+      if (_isUndoing == NO && _isRedoing == NO)
+        [[NSNotificationCenter defaultCenter]
+	    postNotificationName: NSUndoManagerDidOpenUndoGroupNotification
+			  object: self];
+    }
+}
+
 - (void) _loop: (id)arg
 {
   if (_groupsByEvent && _group != nil)
@@ -187,38 +216,35 @@
 
 /**
  * Starts a new grouping of undo actions which can be
- * atomically undone by an [-undo] invocation.
- * This method posts an NSUndoManagerCheckpointNotification
- * unless an undo is currently in progress.  It posts an
- * NSUndoManagerDidOpenUndoGroupNotification upon creating the grouping.
+ * atomically undone by an [-undo] invocation.<br />
+ * This method posts an NSUndoManagerDidOpenUndoGroupNotification
+ * upon creating the grouping.<br />
+ * It first posts an NSUndoManagerCheckpointNotification
+ * unless an undo is currently in progress.<br />
+ * The order of these notifications is undefined, but the GNUstep
+ * implementation currently mimics the observed order in MacOS-X 10.5
  */
 - (void) beginUndoGrouping
 {
-  PrivateUndoGroup	*parent;
+  /* It seems that MacOS-X 10.5 implicitly creates a top-level group
+   * if this method is called when groupsByEvent is set and there is
+   * no existing top level group.  There is no checkpoint notification
+   * posted for the implicit group creation.
+   */
+  if (_group == nil && [self groupsByEvent])
+    {
+      [self _begin];	// Start top level group
+    }
 
+  /* Post the checkpoint notification and THEN create the group.
+   */
   if (_isUndoing == NO)
     {
       [[NSNotificationCenter defaultCenter]
 	  postNotificationName: NSUndoManagerCheckpointNotification
 			object: self];
     }
-  parent = (PrivateUndoGroup*)_group;
-  _group = [[PrivateUndoGroup alloc] initWithParent: parent];
-  if (_group == nil)
-    {
-      _group = parent;
-      [NSException raise: NSInternalInconsistencyException
-		  format: @"beginUndoGrouping failed to greate group"];
-    }
-  else
-    {
-      RELEASE(parent);
-
-      if (_isUndoing == NO && _isRedoing == NO)
-        [[NSNotificationCenter defaultCenter]
-	    postNotificationName: NSUndoManagerDidOpenUndoGroupNotification
-			  object: self];
-    }
+  [self _begin];	// Start a new group
 }
 
 /**
@@ -420,7 +446,7 @@
 	{
 	  if ([self groupsByEvent])
 	    {
-	      [self beginUndoGrouping];
+	      [self _begin];
 	    }
 	  else
 	    {
@@ -475,7 +501,7 @@
  * groupings which can be nested, not the number of of groups on either
  * the undo or redo stack.
  */
-- (int) groupingLevel
+- (NSInteger) groupingLevel
 {
   PrivateUndoGroup	*g = (PrivateUndoGroup*)_group;
   int			level = 0;
@@ -551,7 +577,7 @@
  * The default value is 0 meaning the number is only limited by
  * memory availability.
  */
-- (unsigned int) levelsOfUndo
+- (NSUInteger) levelsOfUndo
 {
   return _levelsOfUndo;
 }
@@ -719,7 +745,7 @@
 	{
 	  if ([self groupsByEvent])
 	    {
-	      [self beginUndoGrouping];
+	      [self _begin];
 	    }
 	  else
 	    {
@@ -852,7 +878,7 @@
  * When set to 0 the stack size is limited by the range of a unsigned int,
  * available memory.
  */
-- (void) setLevelsOfUndo: (unsigned)num
+- (void) setLevelsOfUndo: (NSUInteger)num
 {
   _levelsOfUndo = num;
   if (num > 0)
@@ -881,17 +907,17 @@
     {
       ASSIGN(_modes, newModes);
       if (_runLoopGroupingPending)
-      {
-	NSRunLoop *runLoop = [NSRunLoop currentRunLoop];
-        [runLoop cancelPerformSelector: @selector(_loop:)
-				target: self
-			      argument: nil];
-        [runLoop performSelector: @selector(_loop:)
-			  target: self
-			argument: nil
-			   order: NSUndoCloseGroupingRunLoopOrdering
-			   modes: _modes];
-      }
+	{
+	  NSRunLoop *runLoop = [NSRunLoop currentRunLoop];
+	  [runLoop cancelPerformSelector: @selector(_loop:)
+				  target: self
+				argument: nil];
+	  [runLoop performSelector: @selector(_loop:)
+			    target: self
+			  argument: nil
+			     order: NSUndoCloseGroupingRunLoopOrdering
+			     modes: _modes];
+	}
     }
 }
 
