@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSUserDefaults class reference</title>
-   $Date: 2008-11-11 06:59:11 +0000 (Tue, 11 Nov 2008) $ $Revision: 27022 $
+   $Date: 2009-02-23 13:42:32 -0700 (Mon, 23 Feb 2009) $ $Revision: 27962 $
 */
 
 #include "config.h"
@@ -449,7 +449,7 @@ static BOOL setSharedDefaults = NO;     /* Flag to prevent infinite recursion */
    */
   if (setSharedDefaults == YES)
     {
-      RETAIN(sharedDefaults);
+      IF_NO_GC([sharedDefaults retain];)
       [classLock unlock];
       return AUTORELEASE(sharedDefaults);
     }
@@ -628,7 +628,7 @@ static BOOL setSharedDefaults = NO;     /* Flag to prevent infinite recursion */
       /* FIXME - should we set this as volatile domain for English ? */
       [sharedDefaults registerDefaults: [self _unlocalizedDefaults]];
     }
-  RETAIN(sharedDefaults);
+  IF_NO_GC([sharedDefaults retain];)
   updateCache(sharedDefaults);
   [classLock unlock];
   return AUTORELEASE(sharedDefaults);
@@ -985,14 +985,14 @@ static BOOL setSharedDefaults = NO;     /* Flag to prevent infinite recursion */
   return 0.0;
 }
 
-- (int) integerForKey: (NSString*)defaultName
+- (NSInteger) integerForKey: (NSString*)defaultName
 {
   id	obj = [self objectForKey: defaultName];
 
   if (obj != nil && ([obj isKindOfClass: NSStringClass]
     || [obj isKindOfClass: NSNumberClass]))
     {
-      return [obj intValue];
+      return [obj integerValue];
     }
   return 0;
 }
@@ -1024,7 +1024,7 @@ static BOOL setSharedDefaults = NO;     /* Flag to prevent infinite recursion */
       if (dict != nil && (object = [dict objectForKey: defaultName]))
 	break;
     }
-  RETAIN(object);
+  IF_NO_GC([object retain];)
   [_lock unlock];
   return AUTORELEASE(object);
 }
@@ -1075,9 +1075,9 @@ static BOOL setSharedDefaults = NO;     /* Flag to prevent infinite recursion */
   [self setObject: n forKey: defaultName];
 }
 
-- (void) setInteger: (int)value forKey: (NSString*)defaultName
+- (void) setInteger: (NSInteger)value forKey: (NSString*)defaultName
 {
-  NSNumber	*n = [NSNumberClass numberWithInt: value];
+  NSNumber	*n = [NSNumberClass numberWithInteger: value];
 
   [self setObject: n forKey: defaultName];
 }
@@ -1158,6 +1158,7 @@ static BOOL isPlistObject(id o)
 	value, defaultName];
     }
 
+  value = [value copy];
   [_lock lock];
   obj = [_persDomains objectForKey: processName];
   if ([obj isKindOfClass: NSMutableDictionaryClass] == YES)
@@ -1173,6 +1174,7 @@ static BOOL isPlistObject(id o)
   [dict setObject: value forKey: defaultName];
   [self __changePersistentDomain: processName];
   [_lock unlock];
+  RELEASE(value);
 }
 
 - (void) setValue: (id)value forKey: (NSString*)defaultName
