@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSZone class reference</title>
-   $Date: 2008-06-08 11:38:33 +0100 (Sun, 08 Jun 2008) $ $Revision: 26606 $
+   $Date: 2009-04-25 23:37:21 -0600 (Sat, 25 Apr 2009) $ $Revision: 28250 $
 */
 
 /*  Design goals:
@@ -95,6 +95,7 @@
 #include "Foundation/NSString.h"
 #include "Foundation/NSZone.h"
 #include "Foundation/NSLock.h"
+#include "GSPrivate.h"
 
 /**
  * Try to get more memory - the normal process has failed.
@@ -102,14 +103,375 @@
  * Try to do some logging if possible.
  */
 void *
-GSOutOfMemory(size_t size, BOOL retry)
+GSOutOfMemory(NSUInteger size, BOOL retry)
 {
   fprintf(stderr, "GSOutOfMemory ... wanting %lu bytes.\n",
     (unsigned long)size);
   return 0;
 }
 
-#if	GS_WITH_GC == 0
+/* Default zone functions for default zone. */
+static void* default_malloc (NSZone *zone, size_t size);
+static void* default_realloc (NSZone *zone, void *ptr, size_t size);
+static void default_free (NSZone *zone, void *ptr);
+static void default_recycle (NSZone *zone);
+static BOOL default_check (NSZone *zone);
+static BOOL default_lookup (NSZone *zone, void *ptr);
+static struct NSZoneStats default_stats (NSZone *zone);
+
+static void*
+default_malloc (NSZone *zone, size_t size)
+{
+  void *mem;
+
+  mem = objc_malloc(size);
+  if (mem == NULL)
+    [NSException raise: NSMallocException
+                 format: @"Default zone has run out of memory"];
+  return mem;
+}
+
+static void*
+default_realloc (NSZone *zone, void *ptr, size_t size)
+{
+  void *mem;
+
+  if (size == 0)
+    {
+      objc_free(ptr);
+      return NULL;
+    }
+  if (ptr == 0)
+    {
+      mem = objc_malloc(size);
+      if (mem == NULL)
+	[NSException raise: NSMallocException
+		     format: @"Default zone has run out of memory"];
+      return mem;
+    }
+  mem = objc_realloc(ptr, size);
+  if (mem == NULL)
+    [NSException raise: NSMallocException
+                 format: @"Default zone has run out of memory"];
+  return mem;
+}
+
+static void
+default_free (NSZone *zone, void *ptr)
+{
+  objc_free(ptr);
+}
+
+static void
+default_recycle (NSZone *zone)
+{
+  /* Recycle the default zone?  Thou hast got to be kiddin'. */
+  [NSException raise: NSGenericException
+              format: @"Trying to recycle default zone"];
+}
+
+static BOOL
+default_check (NSZone *zone)
+{
+  /* We can't check memory managed by objc_malloc(). */
+  [NSException raise: NSGenericException
+	      format: @"No checking for default zone"];
+  return NO;
+}
+
+static BOOL
+default_lookup (NSZone *zone, void *ptr)
+{
+  /* Assume all memory is in default zone. */
+  return YES;
+}
+
+static struct NSZoneStats
+default_stats (NSZone *zone)
+{
+  struct NSZoneStats dummy = {0,0,0,0,0};
+
+  /* We can't obtain statistics from the memory managed by objc_malloc(). */
+  [NSException raise: NSGenericException
+	      format: @"No statistics for default zone"];
+  return dummy;
+}
+
+static NSZone default_zone =
+{
+  default_malloc, default_realloc, default_free, default_recycle,
+  default_check, default_lookup, default_stats, 0, @"default", 0
+};
+
+/*
+ * For backward compatibility.
+ */
+NSZone	*__nszone_private_hidden_default_zone = &default_zone;
+
+
+
+void
+NSSetZoneName (NSZone *zone, NSString *name)
+{
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  [gnustep_global_lock lock];
+  name = [name copy];
+  if (zone->name != nil)
+    [zone->name release];
+  zone->name = name;
+  [gnustep_global_lock unlock];
+}
+
+NSString*
+NSZoneName (NSZone *zone)
+{
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  return zone->name;
+}
+
+#if	GS_WITH_GC
+
+#if	defined(DEBUG)
+#define	GC_DEBUG	1
+#endif	/* DEBUG */
+
+#include <gc.h>
+
+/*
+ * Dummy zones used with garbage collection.
+ * The 'atomic' zone is for memory that will be assumed not to contain
+ * pointers for garbage collection purposes.
+ */
+static NSZone atomic_zone =
+{
+  0, 0, 0, 0, 0, 0, 0, 0, @"default", 0
+};
+
+void *
+NSAllocateCollectable(NSUInteger size, NSUInteger options)
+{
+  void	*ptr;
+
+  if (options & NSScannedOption)
+    {
+      if (options & NSCollectorDisabledOption)
+	{
+          ptr = (void*)GC_MALLOC_UNCOLLECTABLE(size);
+	  memset(ptr, '\0', size);
+	}
+      else
+	{
+          ptr = (void*)GC_MALLOC_IGNORE_OFF_PAGE(size);
+	  memset(ptr, '\0', size);
+	}
+    }
+  else
+    {
+      if (options & NSCollectorDisabledOption)
+	{
+	  ptr = (void*)calloc(1, size);
+	}
+      else
+	{
+	  ptr = (void*)GC_MALLOC_ATOMIC_IGNORE_OFF_PAGE(size);
+	  memset(ptr, '\0', size);
+	}
+    }
+  return ptr;
+}
+
+void *
+NSReallocateCollectable(void *ptr, NSUInteger size, NSUInteger options)
+{
+  if (ptr == 0)
+    {
+      ptr = NSAllocateCollectable(size, options);
+    }
+  else if (size > 0 && GC_base(ptr) == 0)
+    {
+      /* This is not collectable memory, so we don't know how big it is.
+       * that means we must use realloc first, to get a known size, and
+       * then, if needed, allocate a chunk the right size and copy to it.
+       */
+      ptr = realloc(ptr, size);
+      if (ptr != 0)
+	{
+	  if ((options & NSScannedOption)
+	    || !(options & NSCollectorDisabledOption))
+	    {
+	      void	*tmp = NSAllocateCollectable(size, options);
+
+	      memcpy(tmp, ptr, size);
+	      free(ptr);
+	      ptr = tmp;
+	    }
+	}
+    }
+  else
+    {
+      void	*tmp = NSAllocateCollectable(size, options);
+      unsigned	length = GC_size(ptr);
+
+      if (length > size)
+	{
+	  length = size;
+	}
+      if (tmp != 0 && length > 0)
+	{
+          memcpy(tmp, ptr, length);
+	}
+      GC_FREE(ptr);
+      ptr = tmp;
+    }
+  return ptr;
+}
+
+NSZone*
+NSCreateZone (NSUInteger start, NSUInteger gran, BOOL canFree)
+{
+  NSLog(@" *** Creating a zone while running GC is ignored.");
+  return &default_zone;
+}
+
+NSZone*
+NSDefaultMallocZone (void)
+{
+  return &default_zone;
+}
+
+NSZone*
+GSAtomicMallocZone (void)
+{
+  return &atomic_zone;
+}
+
+NSZone*
+NSZoneFromPointer (void *ptr)
+{
+  return &default_zone;
+}
+
+void
+NSRecycleZone (NSZone *zone)
+{
+}
+
+BOOL
+NSZoneCheck (NSZone *zone)
+{
+  return YES;
+}
+
+struct
+NSZoneStats NSZoneStats (NSZone *zone)
+{
+  struct NSZoneStats stats = { 0 };
+  return stats;
+}
+
+void
+GSMakeWeakPointer(Class theClass, const char *iVarName)
+{
+  class_ivar_set_gcinvisible(theClass, iVarName, YES);
+}
+
+BOOL
+GSAssignZeroingWeakPointer(void **destination, void *source)
+{
+  if (destination == 0)
+    {
+      return NO;	// Bad destination pointer
+    }
+  if (*destination == source)
+    {
+      return YES;	// Already assigned.
+    }
+  if (source != 0 && GC_base(source) == 0)
+    {
+      return NO;	// Source is not garbage collectable.
+    }
+  if (*destination != 0)
+    {
+      GC_unregister_disappearing_link(destination);
+    }
+  *destination = source;
+  if (source != 0)
+    {
+      GC_general_register_disappearing_link(destination, source);
+    }
+  return YES;
+}
+
+void*
+NSZoneMalloc (NSZone *zone, NSUInteger size)
+{
+  return NSZoneCalloc(zone, 1, size);
+}
+
+void*
+NSZoneCalloc (NSZone *zone, NSUInteger elems, NSUInteger bytes)
+{
+  size_t	size = elems * bytes;
+  void		*ptr;
+
+  if (zone == &atomic_zone)
+    {
+      ptr = (void*)GC_MALLOC_ATOMIC(size);
+      memset(ptr, '\0', size);
+    }
+  else
+    {
+      ptr = (void*)malloc(size);
+      if (ptr == 0)
+	{
+          ptr = GSOutOfMemory(size, NO);
+	}
+      if (ptr != 0)
+	{
+	  memset(ptr, '\0', size);
+	}
+    }
+  return ptr;
+}
+
+void*
+NSZoneRealloc (NSZone *zone, void *ptr, NSUInteger size)
+{
+  if (GC_base(ptr) != 0)
+    {
+      ptr = GC_REALLOC(ptr, size);
+    }
+  else
+    {
+      ptr = realloc(ptr, size);
+    }
+  return ptr;
+}
+
+void
+NSZoneFree (NSZone *zone, void *ptr)
+{
+  if (GC_base(ptr) != 0)
+    {
+      GC_FREE(ptr);
+    }
+  else
+    {
+      free(ptr);
+    }
+}
+
+BOOL
+GSPrivateIsCollectable(const void *ptr)
+{
+  if (GC_base((void*)ptr) == 0) return NO;
+  else return YES;
+}
+
+#else	/* GS_WITH_GC */
+
 
 /* Alignment */
 #ifdef ALIGN
@@ -355,17 +717,6 @@ struct _nfree_zone_struct
   size_t use;
 };
 
-
-
-/* Default zone functions for default zone. */
-static void* default_malloc (NSZone *zone, size_t size);
-static void* default_realloc (NSZone *zone, void *ptr, size_t size);
-static void default_free (NSZone *zone, void *ptr);
-static void default_recycle (NSZone *zone);
-static BOOL default_check (NSZone *zone);
-static BOOL default_lookup (NSZone *zone, void *ptr);
-static struct NSZoneStats default_stats (NSZone *zone);
-
 /* Memory management functions for freeable zones. */
 static void* fmalloc (NSZone *zone, size_t size);
 static void* frealloc (NSZone *zone, void *ptr, size_t size);
@@ -398,49 +749,11 @@ static void* rrealloc (NSZone *zone, void *ptr, size_t size);
 static void rffree (NSZone *zone, void *ptr);
 static void rnfree (NSZone *zone, void *ptr);
 
-
-static NSZone default_zone =
-{
-  default_malloc, default_realloc, default_free, default_recycle,
-  default_check, default_lookup, default_stats, DEFBLOCK, @"default", 0
-};
-
-/* Default zone.  Name is hopelessly long so that no one will ever
-   want to use it. ;) */
-NSZone* __nszone_private_hidden_default_zone = &default_zone;
-
 /*
  *	Lists of zones to be used to determine if a pointer is in a zone.
  */
 static NSZone	*zone_list = 0;
 
-/**
- * Searches and finds the zone ptr was allocated from.  The speed depends
- * upon the number of zones and their size.
- */
-GS_DECLARE NSZone*
-NSZoneFromPointer(void *ptr)
-{
-  NSZone	*zone;
-
-  if (ptr == 0) return 0;
-  if (zone_list == 0) return __nszone_private_hidden_default_zone;
-
-  /*
-   *	See if we can find the zone in our list of all zones.
-   */
-  [gnustep_global_lock lock];
-  for (zone = zone_list; zone != 0; zone = zone->next)
-    {
-      if ((zone->lookup)(zone, ptr) == YES)
-	{
-	  break;
-	}
-    }
-  [gnustep_global_lock unlock];
-  return (zone == 0) ? __nszone_private_hidden_default_zone : zone;
-}
-
 static inline void
 destroy_zone(NSZone* zone)
 {
@@ -458,84 +771,6 @@ destroy_zone(NSZone* zone)
   objc_free((void*)zone);
 }
 
-static void*
-default_malloc (NSZone *zone, size_t size)
-{
-  void *mem;
-
-  mem = objc_malloc(size);
-  if (mem == NULL)
-    [NSException raise: NSMallocException
-                 format: @"Default zone has run out of memory"];
-  return mem;
-}
-
-static void*
-default_realloc (NSZone *zone, void *ptr, size_t size)
-{
-  void *mem;
-
-  if (size == 0)
-    {
-      objc_free(ptr);
-      return NULL;
-    }
-  if (ptr == 0)
-    {
-      mem = objc_malloc(size);
-      if (mem == NULL)
-	[NSException raise: NSMallocException
-		     format: @"Default zone has run out of memory"];
-      return mem;
-    }
-  mem = objc_realloc(ptr, size);
-  if (mem == NULL)
-    [NSException raise: NSMallocException
-                 format: @"Default zone has run out of memory"];
-  return mem;
-}
-
-static void
-default_free (NSZone *zone, void *ptr)
-{
-  objc_free(ptr);
-}
-
-static void
-default_recycle (NSZone *zone)
-{
-  /* Recycle the default zone?  Thou hast got to be kiddin'. */
-  [NSException raise: NSGenericException
-              format: @"Trying to recycle default zone"];
-}
-
-static BOOL
-default_check (NSZone *zone)
-{
-  /* We can't check memory managed by objc_malloc(). */
-  [NSException raise: NSGenericException
-	      format: @"No checking for default zone"];
-  return NO;
-}
-
-static BOOL
-default_lookup (NSZone *zone, void *ptr)
-{
-  /* Assume all memory is in default zone. */
-  return YES;
-}
-
-static struct NSZoneStats
-default_stats (NSZone *zone)
-{
-  struct NSZoneStats dummy = {0,0,0,0,0};
-
-  /* We can't obtain statistics from the memory managed by objc_malloc(). */
-  [NSException raise: NSGenericException
-	      format: @"No statistics for default zone"];
-  return dummy;
-}
-
 /* Search the buffer to see if there is any memory chunks large enough
    to satisfy request using first fit.  If the memory chunk found has
    a size exactly equal to the one requested, remove it from the buffer
@@ -1588,16 +1823,31 @@ rrealloc (NSZone *zone, void *ptr, size_t size)
 
 static void rnfree (NSZone *zone, void *ptr);
 
-/**
- * Creates a new zone of start bytes, which will grow and shrink by
- * granularity bytes.  If canFree is 0, memory in zone is allocated but
- * never freed, meaning allocation will be very fast.  The whole zone can
- * still be freed with NSRecycleZone(), and you should still call NSZoneFree
- * on memory in the zone that is no longer needed, since a count of allocated
- * pointers is kept and must reach zero before freeing the zone.
- */
+GS_DECLARE NSZone*
+NSZoneFromPointer(void *ptr)
+{
+  NSZone	*zone;
+
+  if (ptr == 0) return 0;
+  if (zone_list == 0) return &default_zone;
+
+  /*
+   *	See if we can find the zone in our list of all zones.
+   */
+  [gnustep_global_lock lock];
+  for (zone = zone_list; zone != 0; zone = zone->next)
+    {
+      if ((zone->lookup)(zone, ptr) == YES)
+	{
+	  break;
+	}
+    }
+  [gnustep_global_lock unlock];
+  return (zone == 0) ? &default_zone : zone;
+}
+
 NSZone*
-NSCreateZone (size_t start, size_t gran, BOOL canFree)
+NSCreateZone (NSUInteger start, NSUInteger gran, BOOL canFree)
 {
   size_t i, startsize, granularity;
   NSZone *newZone;
@@ -1711,55 +1961,105 @@ NSCreateZone (size_t start, size_t gran, BOOL canFree)
   return newZone;
 }
 
-/**
- *  Allocates and returns cleared memory for elems items of size bytes, in the
- *  given zone.  Returns NULL if allocation of size 0 requested.  Raises
- *  <code>NSMallocException</code> if not enough free memory in zone to
- *  allocate and no more can be obtained from system.
- */
 void*
-NSZoneCalloc (NSZone *zone, size_t elems, size_t bytes)
+NSZoneCalloc (NSZone *zone, NSUInteger elems, NSUInteger bytes)
 {
   return memset(NSZoneMalloc(zone, elems*bytes), 0, elems*bytes);
 }
 
-/**
- * Sets name of the given zone (useful for debugging and logging).
- */
+void *
+NSAllocateCollectable(NSUInteger size, NSUInteger options)
+{
+  return NSZoneCalloc(NSDefaultMallocZone(), 1, size);
+}
+
+void *
+NSReallocateCollectable(void *ptr, NSUInteger size, NSUInteger options)
+{
+  return NSZoneRealloc(0, ptr, size);
+}
+
+NSZone*
+NSDefaultMallocZone (void)
+{
+  return &default_zone;
+}
+
+NSZone*
+GSAtomicMallocZone (void)
+{
+  return &default_zone;
+}
+
 void
-NSSetZoneName (NSZone *zone, NSString *name)
+GSMakeWeakPointer(Class theClass, const char *iVarName)
+{
+  return;
+}
+
+BOOL
+GSAssignZeroingWeakPointer(void **destination, void *source)
+{
+  if (destination == 0)
+    {
+      return NO;
+    }
+  *destination = source;
+  return YES;
+}
+
+void*
+NSZoneMalloc (NSZone *zone, NSUInteger size)
 {
   if (!zone)
     zone = NSDefaultMallocZone();
-  [gnustep_global_lock lock];
-  name = [name copy];
-  if (zone->name != nil)
-    [zone->name release];
-  zone->name = name;
-  [gnustep_global_lock unlock];
+  return (zone->malloc)(zone, size);
 }
 
-#else
-
-#include <gc.h>
-
-/*
- * Dummy zones used with garbage collection.
- * The 'atomic' zone is for memory that will be assumed not to contain
- * pointers for garbage collection purposes.
- */
-static NSZone default_zone =
+void* 
+NSZoneRealloc (NSZone *zone, void *ptr, NSUInteger size)
 {
-  0, 0, 0, 0, 0, 0, 0, 0, @"default", 0
-};
-NSZone* __nszone_private_hidden_default_zone = &default_zone;
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  return (zone->realloc)(zone, ptr, size);
+}
 
-static NSZone atomic_zone =
+void
+NSRecycleZone (NSZone *zone)
 {
-  0, 0, 0, 0, 0, 0, 0, 0, @"default", 0
-};
-NSZone* __nszone_private_hidden_atomic_zone = &atomic_zone;
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  (zone->recycle)(zone);
+}
 
+void
+NSZoneFree (NSZone *zone, void *ptr)
+{
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  (zone->free)(zone, ptr);
+}
+
+BOOL
+NSZoneCheck (NSZone *zone)
+{
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  return (zone->check)(zone);
+}
+
+struct NSZoneStats
+NSZoneStats (NSZone *zone)
+{
+  if (!zone)
+    zone = NSDefaultMallocZone();
+  return (zone->stats)(zone);
+}
+
+BOOL
+GSPrivateIsCollectable(const void *ptr)
+{
+  return NO;
+}
 
 #endif	/* GS_WITH_GC */
-
