@@ -195,7 +195,16 @@ cifframe_from_info (NSArgumentInfo *info, int numargs, void **retval)
 	full += cifframe_guess_struct_size(rtype);
       else
 	full += MAX(rtype->size, sizeof(smallret_t));
+      /* HACK ... not sure why, but on my 64bit intel system adding a bit
+       * more to the buffer size prevents writing outside the allocated
+       * memory by the ffi stuff.
+       */
+      full += 64;
+#if	GS_WITH_GC
+      cframe = buf = NSAllocateCollectable(full, NSScannedOption);
+#else
       cframe = buf = NSZoneCalloc(NSDefaultMallocZone(), full, 1);
+#endif
       if (cframe && retval)
 	{
 	  *retval = buf + pos;
@@ -203,7 +212,11 @@ cifframe_from_info (NSArgumentInfo *info, int numargs, void **retval)
     }
   else
     {
+#if	GS_WITH_GC
+      cframe = buf = NSAllocateCollectable(size, NSScannedOption);
+#else
       cframe = buf = NSZoneCalloc(NSDefaultMallocZone(), size, 1);
+#endif
     }
 
   if (cframe)
@@ -465,7 +478,7 @@ cifframe_type(const char *typePtr, const char **advance)
 	  {
 	    size += (align - (size % align));
 	  }
-	ftype = objc_malloc(size + maxtypes*sizeof(ffi_type));
+	ftype = objc_malloc(size + (maxtypes+1)*sizeof(ffi_type));
 	ftype->size = 0;
 	ftype->alignment = 0;
 	ftype->type = FFI_TYPE_STRUCT;
@@ -492,7 +505,9 @@ cifframe_type(const char *typePtr, const char **advance)
 	    if (types >= maxtypes)
 	      {
 		maxtypes *=2;
-		ftype = objc_realloc(ftype, size + maxtypes*sizeof(ffi_type));
+		ftype = objc_realloc(ftype,
+                  size + (maxtypes+1)*sizeof(ffi_type));
+	        ftype->elements = (void*)ftype + size;
 	      }
 	  }
 	ftype->elements[types] = NULL;
@@ -1035,8 +1050,9 @@ cifframe_do_call (DOContext *ctxt,
 	    }
 	}
     }
-
+#if	!GS_WITH_GC
   NSZoneFree(NSDefaultMallocZone(), ctxt->datToFree);
+#endif
   ctxt->datToFree = 0;
 
   return;
@@ -1115,12 +1131,17 @@ cifframe_build_return (NSInvocation *inv,
 		tmptype++;
 		retLength = objc_sizeof_type(tmptype);
 		/* Allocate memory to hold the value we're pointing to. */
+#if	GS_WITH_GC
+		*(void**)retval =
+		  NSAllocateCollectable(retLength, NSScannedOption);
+#else
 		*(void**)retval =
 		  NSZoneCalloc(NSDefaultMallocZone(), retLength, 1);
 		/* We are responsible for making sure this memory gets free'd
 		   eventually.  Ask NSData class to autorelease it. */
 		[NSData dataWithBytesNoCopy: *(void**)retval
 				     length: retLength];
+#endif
 		ctxt->type = tmptype;
 		ctxt->datum = *(void**)retval;
 		/* Decode the return value into the memory we allocated. */
@@ -1225,7 +1246,9 @@ cifframe_build_return (NSInvocation *inv,
 
   if (ctxt->datToFree != 0)
     {
+#if	!GS_WITH_GC
       NSZoneFree(NSDefaultMallocZone(), ctxt->datToFree);
+#endif
       ctxt->datToFree = 0;
     }
 
