@@ -20,7 +20,7 @@
    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
 
    <title>NSMessagePortNameServer class reference</title>
-   $Date: 2008-11-17 13:45:32 +0000 (Mon, 17 Nov 2008) $ $Revision: 27080 $
+   $Date: 2009-03-18 02:50:32 -0600 (Wed, 18 Mar 2009) $ $Revision: 28090 $
    */
 
 #include "Foundation/NSPortNameServer.h"
@@ -50,7 +50,7 @@ extern __declspec(dllimport) int	errno;
 
 static NSRecursiveLock *serverLock = nil;
 static NSMessagePortNameServer *defaultServer = nil;
-static NSMapTable portToNamesMap;
+static NSMapTable *portToNamesMap;
 static NSString	*registry;
 static HKEY	key;
 
@@ -76,7 +76,7 @@ static void clean_up_names(void)
       [defaultServer removePort: port];
     }
   NSEndMapTableEnumeration(&mEnum);
-  DESTROY(arp);
+  IF_NO_GC([arp release]);
   RegCloseKey(key);
   if (unknownThread == YES)
     {
